@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -70,7 +70,7 @@ Library for working with the tor process.
   **NAMED**         relay can be referred to by its nickname
   **RUNNING**       relay is currently usable
   **STABLE**        relay's suitable for long-lived circuits
-  **UNNAMED**       relay isn't presently bound to a nickname
+  **UNNAMED**       relay isn't currently bound to a nickname
   **V2DIR**         relay supports the v2 directory protocol
   **VALID**         relay has been validated
   ================= ===========
@@ -133,7 +133,7 @@ Library for working with the tor process.
   **TORPROTOCOL**           violation in the tor protocol
   **INTERNAL**              internal error
   **REQUESTED**             requested by the client via a TRUNCATE command
-  **HIBERNATING**           relay is presently hibernating
+  **HIBERNATING**           relay is currently hibernating
   **RESOURCELIMIT**         relay is out of memory, sockets, or circuit IDs
   **CONNECTFAILED**         unable to contact the relay
   **OR_IDENTITY**           relay had the wrong OR identification
@@ -406,7 +406,9 @@ Library for working with the tor process.
   HSDescAction    Description
   =============== ===========
   **REQUESTED**   uncached hidden service descriptor is being requested
+  **UPLOAD**      descriptor is being uploaded with HSPOST
   **RECEIVED**    hidden service descriptor has been retrieved
+  **UPLOADED**    descriptor was uploaded with HSPOST
   **IGNORE**      fetched descriptor was ignored because we already have its v0 descriptor
   **FAILED**      we were unable to retrieve the descriptor
   =============== ===========
@@ -420,6 +422,7 @@ Library for working with the tor process.
   =================== ===========
   **BAD_DESC**        descriptor was unparseable
   **QUERY_REJECTED**  hidden service directory refused to provide the descriptor
+  **UPLOAD_REJECTED** descriptor was rejected by the hidden service directory
   **NOT_FOUND**       descriptor with the given identifier wasn't found
   **UNEXPECTED**      failure type is unknown
   =================== ===========
@@ -438,7 +441,7 @@ Library for working with the tor process.
   ================= ===========
 """
 
-__version__ = '1.3.0'
+__version__ = '1.4.1'
 __author__ = 'Damian Johnson'
 __contact__ = 'atagar@torproject.org'
 __url__ = 'https://stem.torproject.org/'
@@ -491,6 +494,15 @@ __all__ = [
   'TimeoutSetType',
 ]
 
+import stem.prereq
+
+if stem.prereq.is_python_3():
+  str_type = str
+  int_type = int
+else:
+  str_type = unicode
+  int_type = long
+
 import stem.util.enum
 
 # Constant to indicate an undefined argument default. Usually we'd use None for
@@ -798,7 +810,9 @@ TokenBucket = stem.util.enum.UppercaseEnum(
 
 HSDescAction = stem.util.enum.UppercaseEnum(
   'REQUESTED',
+  'UPLOAD',
   'RECEIVED',
+  'UPLOADED',
   'IGNORE',
   'FAILED',
 )
@@ -806,6 +820,7 @@ HSDescAction = stem.util.enum.UppercaseEnum(
 HSDescReason = stem.util.enum.UppercaseEnum(
   'BAD_DESC',
   'QUERY_REJECTED',
+  'UPLOAD_REJECTED',
   'NOT_FOUND',
   'UNEXPECTED',
 )
