@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -50,6 +50,10 @@ __all__ = [
   'Descriptor',
 ]
 
+import base64
+import codecs
+import copy
+import hashlib
 import os
 import re
 import tarfile
@@ -59,6 +63,8 @@ import stem.util.enum
 import stem.util.str_tools
 import stem.util.system
 
+from stem import str_type
+
 try:
   # added in python 2.7
   from collections import OrderedDict
@@ -68,6 +74,7 @@ except ImportError:
 KEYWORD_CHAR = 'a-zA-Z0-9-'
 WHITESPACE = ' \t'
 KEYWORD_LINE = re.compile('^([%s]+)(?:[%s]+(.*))?$' % (KEYWORD_CHAR, WHITESPACE))
+SPECIFIC_KEYWORD_LINE = '^(%%s)(?:[%s]+(.*))?$' % WHITESPACE
 PGP_BLOCK_START = re.compile('^-----BEGIN ([%s%s]+)-----$' % (KEYWORD_CHAR, WHITESPACE))
 PGP_BLOCK_END = '-----END %s-----'
 
@@ -78,7 +85,7 @@ DocumentHandler = stem.util.enum.UppercaseEnum(
 )
 
 
-def parse_file(descriptor_file, descriptor_type = None, validate = True, document_handler = DocumentHandler.ENTRIES, **kwargs):
+def parse_file(descriptor_file, descriptor_type = None, validate = False, document_handler = DocumentHandler.ENTRIES, **kwargs):
   """
   Simple function to read the descriptor contents from a file, providing an
   iterator for its :class:`~stem.descriptor.__init__.Descriptor` contents.
@@ -87,7 +94,7 @@ def parse_file(descriptor_file, descriptor_type = None, validate = True, documen
   tries to determine the descriptor type based on the following...
 
   * The @type annotation on the first line. These are generally only found in
-    the `descriptor archives <https://metrics.torproject.org>`_.
+    the `CollecTor archives <https://collector.torproject.org/formats.html#relay-descriptors>`_.
 
   * The filename if it matches something from tor's data directory. For
     instance, tor's 'cached-descriptors' contains server descriptors.
@@ -114,10 +121,11 @@ def parse_file(descriptor_file, descriptor_type = None, validate = True, documen
   network-status-microdesc-consensus-3 1.0  :class:`~stem.descriptor.router_status_entry.RouterStatusEntryMicroV3` (with a :class:`~stem.descriptor.networkstatus.NetworkStatusDocumentV3`)
   bridge-network-status 1.0                 :class:`~stem.descriptor.router_status_entry.RouterStatusEntryV3` (with a :class:`~stem.descriptor.networkstatus.BridgeNetworkStatusDocument`)
   bridge-server-descriptor 1.0              :class:`~stem.descriptor.server_descriptor.BridgeDescriptor`
-  bridge-extra-info 1.1                     :class:`~stem.descriptor.extrainfo_descriptor.BridgeExtraInfoDescriptor`
+  bridge-extra-info 1.1 or 1.2              :class:`~stem.descriptor.extrainfo_descriptor.BridgeExtraInfoDescriptor`
   torperf 1.0                               **unsupported**
   bridge-pool-assignment 1.0                **unsupported**
   tordnsel 1.0                              :class:`~stem.descriptor.tordnsel.TorDNSEL`
+  hidden-service-descriptor 1.0             :class:`~stem.descriptor.hidden_service_descriptor.HiddenServiceDescriptor`
   ========================================= =====
 
   If you're using **python 3** then beware that the open() function defaults to
@@ -130,7 +138,7 @@ def parse_file(descriptor_file, descriptor_type = None, validate = True, documen
     my_descriptor_file = open(descriptor_path, 'rb')
 
   :param str,file,tarfile descriptor_file: path or opened file with the descriptor contents
-  :param str descriptor_type: `descriptor type <https://metrics.torproject.org/formats.html#descriptortypes>`_, this is guessed if not provided
+  :param str descriptor_type: `descriptor type <https://collector.torproject.org/formats.html>`_, this is guessed if not provided
   :param bool validate: checks the validity of the descriptor's content if
     **True**, skips these checks otherwise
   :param stem.descriptor.__init__.DocumentHandler document_handler: method in
@@ -149,7 +157,7 @@ def parse_file(descriptor_file, descriptor_type = None, validate = True, documen
 
   handler = None
 
-  if isinstance(descriptor_file, (bytes, unicode)):
+  if isinstance(descriptor_file, (bytes, str_type)):
     if stem.util.system.is_tarfile(descriptor_file):
       handler = _parse_file_for_tar_path
     else:
@@ -196,11 +204,11 @@ def parse_file(descriptor_file, descriptor_type = None, validate = True, documen
   else:
     # Cached descriptor handling. These contain multiple descriptors per file.
 
-    if filename == 'cached-descriptors':
+    if filename == 'cached-descriptors' or filename == 'cached-descriptors.new':
       file_parser = lambda f: stem.descriptor.server_descriptor._parse_file(f, validate = validate, **kwargs)
-    elif filename == 'cached-extrainfo':
+    elif filename == 'cached-extrainfo' or filename == 'cached-extrainfo.new':
       file_parser = lambda f: stem.descriptor.extrainfo_descriptor._parse_file(f, validate = validate, **kwargs)
-    elif filename == 'cached-microdescs':
+    elif filename == 'cached-microdescs' or filename == 'cached-microdescs.new':
       file_parser = lambda f: stem.descriptor.microdescriptor._parse_file(f, validate = validate, **kwargs)
     elif filename == 'cached-consensus':
       file_parser = lambda f: stem.descriptor.networkstatus._parse_file(f, validate = validate, document_handler = document_handler, **kwargs)
@@ -303,19 +311,102 @@ def _parse_metrics_file(descriptor_type, major_version, minor_version, descripto
 
     for desc in stem.descriptor.tordnsel._parse_file(descriptor_file, validate = validate, **kwargs):
       yield desc
+  elif descriptor_type == 'hidden-service-descriptor' and major_version == 1:
+    document_type = stem.descriptor.hidden_service_descriptor.HiddenServiceDescriptor
+
+    for desc in stem.descriptor.hidden_service_descriptor._parse_file(descriptor_file, validate = validate, **kwargs):
+      yield desc
   else:
     raise TypeError("Unrecognized metrics descriptor format. type: '%s', version: '%i.%i'" % (descriptor_type, major_version, minor_version))
 
 
+def _value(line, entries):
+  return entries[line][0][0]
+
+
+def _values(line, entries):
+  return [entry[0] for entry in entries[line]]
+
+
+def _parse_simple_line(keyword, attribute):
+  def _parse(descriptor, entries):
+    setattr(descriptor, attribute, _value(keyword, entries))
+
+  return _parse
+
+
+def _parse_bytes_line(keyword, attribute):
+  def _parse(descriptor, entries):
+    line_match = re.search(stem.util.str_tools._to_bytes('^(opt )?%s(?:[%s]+(.*))?$' % (keyword, WHITESPACE)), descriptor.get_bytes(), re.MULTILINE)
+    result = None
+
+    if line_match:
+      value = line_match.groups()[1]
+      result = b'' if value is None else value
+
+    setattr(descriptor, attribute, result)
+
+  return _parse
+
+
+def _parse_timestamp_line(keyword, attribute):
+  # "<keyword>" YYYY-MM-DD HH:MM:SS
+
+  def _parse(descriptor, entries):
+    value = _value(keyword, entries)
+
+    try:
+      setattr(descriptor, attribute, stem.util.str_tools._parse_timestamp(value))
+    except ValueError:
+      raise ValueError("Timestamp on %s line wasn't parsable: %s %s" % (keyword, keyword, value))
+
+  return _parse
+
+
+def _parse_forty_character_hex(keyword, attribute):
+  # format of fingerprints, sha1 digests, etc
+
+  def _parse(descriptor, entries):
+    value = _value(keyword, entries)
+
+    if not stem.util.tor_tools.is_hex_digits(value, 40):
+      raise ValueError('%s line had an invalid value (should be 40 hex characters): %s %s' % (keyword, keyword, value))
+
+    setattr(descriptor, attribute, value)
+
+  return _parse
+
+
+def _parse_key_block(keyword, attribute, expected_block_type, value_attribute = None):
+  def _parse(descriptor, entries):
+    value, block_type, block_contents = entries[keyword][0]
+
+    if not block_contents or block_type != expected_block_type:
+      raise ValueError("'%s' should be followed by a %s block, but was a %s" % (keyword, expected_block_type, block_type))
+
+    setattr(descriptor, attribute, block_contents)
+
+    if value_attribute:
+      setattr(descriptor, value_attribute, value)
+
+  return _parse
+
+
 class Descriptor(object):
   """
   Common parent for all types of descriptors.
   """
 
-  def __init__(self, contents):
+  ATTRIBUTES = {}  # mapping of 'attribute' => (default_value, parsing_function)
+  PARSER_FOR_LINE = {}  # line keyword to its associated parsing function
+
+  def __init__(self, contents, lazy_load = False):
     self._path = None
     self._archive_path = None
     self._raw_contents = contents
+    self._lazy_loading = lazy_load
+    self._entries = {}
+    self._unrecognized_lines = []
 
   def get_path(self):
     """
@@ -358,7 +449,49 @@ class Descriptor(object):
     :returns: **list** of lines of unrecognized content
     """
 
-    raise NotImplementedError
+    if self._lazy_loading:
+      # we need to go ahead and parse the whole document to figure this out
+      self._parse(self._entries, False)
+      self._lazy_loading = False
+
+    return list(self._unrecognized_lines)
+
+  def _parse(self, entries, validate, parser_for_line = None):
+    """
+    Parses a series of 'keyword => (value, pgp block)' mappings and applies
+    them as attributes.
+
+    :param dict entries: descriptor contents to be applied
+    :param bool validate: checks the validity of descriptor content if True
+    :param dict parsers: mapping of lines to the function for parsing it
+
+    :raises: **ValueError** if an error occurs in validation
+    """
+
+    if parser_for_line is None:
+      parser_for_line = self.PARSER_FOR_LINE
+
+    # set defaults
+
+    for attr in self.ATTRIBUTES:
+      if not hasattr(self, attr):
+        setattr(self, attr, copy.copy(self.ATTRIBUTES[attr][0]))
+
+    for keyword, values in list(entries.items()):
+      try:
+        if keyword in parser_for_line:
+          parser_for_line[keyword](self, entries)
+        else:
+          for value, block_type, block_contents in values:
+            line = '%s %s' % (keyword, value)
+
+            if block_contents:
+              line += '\n%s' % block_contents
+
+            self._unrecognized_lines.append(line)
+      except ValueError as exc:
+        if validate:
+          raise exc
 
   def _set_path(self, path):
     self._path = path
@@ -366,6 +499,123 @@ class Descriptor(object):
   def _set_archive_path(self, path):
     self._archive_path = path
 
+  def _name(self, is_plural = False):
+    return str(type(self))
+
+  def _digest_for_signature(self, signing_key, signature):
+    """
+    Provides the signed digest we should have given this key and signature.
+
+    :param str signing_key: key block used to make this signature
+    :param str signature: signed digest for this descriptor content
+
+    :returns: the digest string encoded in uppercase hex
+
+    :raises: ValueError if unable to provide a validly signed digest
+    """
+
+    if not stem.prereq.is_crypto_available():
+      raise ValueError('Generating the signed digest requires pycrypto')
+
+    from Crypto.Util import asn1
+    from Crypto.Util.number import bytes_to_long, long_to_bytes
+
+    # get the ASN.1 sequence
+
+    seq = asn1.DerSequence()
+    seq.decode(_bytes_for_block(signing_key))
+    modulus, public_exponent = seq[0], seq[1]
+
+    sig_as_bytes = _bytes_for_block(signature)
+    sig_as_long = bytes_to_long(sig_as_bytes)  # convert signature to an int
+    blocksize = 128  # block size will always be 128 for a 1024 bit key
+
+    # use the public exponent[e] & the modulus[n] to decrypt the int
+
+    decrypted_int = pow(sig_as_long, public_exponent, modulus)
+
+    # convert the int to a byte array
+
+    decrypted_bytes = long_to_bytes(decrypted_int, blocksize)
+
+    ############################################################################
+    # The decrypted bytes should have a structure exactly along these lines.
+    # 1 byte  - [null '\x00']
+    # 1 byte  - [block type identifier '\x01'] - Should always be 1
+    # N bytes - [padding '\xFF' ]
+    # 1 byte  - [separator '\x00' ]
+    # M bytes - [message]
+    # Total   - 128 bytes
+    # More info here http://www.ietf.org/rfc/rfc2313.txt
+    #                esp the Notes in section 8.1
+    ############################################################################
+
+    try:
+      if decrypted_bytes.index(b'\x00\x01') != 0:
+        raise ValueError('Verification failed, identifier missing')
+    except ValueError:
+      raise ValueError('Verification failed, malformed data')
+
+    try:
+      identifier_offset = 2
+
+      # find the separator
+      seperator_index = decrypted_bytes.index(b'\x00', identifier_offset)
+    except ValueError:
+      raise ValueError('Verification failed, seperator not found')
+
+    digest_hex = codecs.encode(decrypted_bytes[seperator_index + 1:], 'hex_codec')
+    return stem.util.str_tools._to_unicode(digest_hex.upper())
+
+  def _digest_for_content(self, start, end):
+    """
+    Provides the digest of our descriptor's content in a given range.
+
+    :param bytes start: start of the range to generate a digest for
+    :param bytes end: end of the range to generate a digest for
+
+    :returns: the digest string encoded in uppercase hex
+
+    :raises: ValueError if the digest canot be calculated
+    """
+
+    raw_descriptor = self.get_bytes()
+
+    start_index = raw_descriptor.find(start)
+    end_index = raw_descriptor.find(end, start_index)
+
+    if start_index == -1:
+      raise ValueError("Digest is for the range starting with '%s' but that isn't in our descriptor" % start)
+    elif end_index == -1:
+      raise ValueError("Digest is for the range ending with '%s' but that isn't in our descriptor" % end)
+
+    digest_content = raw_descriptor[start_index:end_index + len(end)]
+    digest_hash = hashlib.sha1(stem.util.str_tools._to_bytes(digest_content))
+    return stem.util.str_tools._to_unicode(digest_hash.hexdigest().upper())
+
+  def __getattr__(self, name):
+    # Our constructor sets these, but when unpickling we might lack them. This
+    # check is needed to avoid an infinite loop in that case.
+
+    if name in ('_lazy_loading', 'ATTRIBUTES'):
+      return super(Descriptor, self).__getattribute__(name)
+
+    # If attribute isn't already present we might be lazy loading it...
+
+    if self._lazy_loading and name in self.ATTRIBUTES:
+      default, parsing_function = self.ATTRIBUTES[name]
+
+      try:
+        parsing_function(self, self._entries)
+      except (ValueError, KeyError):
+        try:
+          # despite having a validation failure check to see if we set something
+          return super(Descriptor, self).__getattribute__(name)
+        except AttributeError:
+          setattr(self, name, copy.copy(default))
+
+    return super(Descriptor, self).__getattribute__(name)
+
   def __str__(self):
     if stem.prereq.is_python_3():
       return stem.util.str_tools._to_unicode(self._raw_contents)
@@ -373,33 +623,6 @@ class Descriptor(object):
       return self._raw_contents
 
 
-def _get_bytes_field(keyword, content):
-  """
-  Provides the value corresponding to the given keyword. This is handy to fetch
-  values specifically allowed to be arbitrary bytes prior to converting to
-  unicode.
-
-  :param str keyword: line to look up
-  :param bytes content: content to look through
-
-  :returns: **bytes** value on the given line, **None** if the line doesn't
-    exist
-
-  :raises: **ValueError** if the content isn't bytes
-  """
-
-  if not isinstance(content, bytes):
-    raise ValueError('Content must be bytes, got a %s' % type(content))
-
-  line_match = re.search(stem.util.str_tools._to_bytes('^(opt )?%s(?:[%s]+(.*))?$' % (keyword, WHITESPACE)), content, re.MULTILINE)
-
-  if line_match:
-    value = line_match.groups()[1]
-    return b'' if value is None else value
-  else:
-    return None
-
-
 def _read_until_keywords(keywords, descriptor_file, inclusive = False, ignore_first = False, skip = False, end_position = None, include_ending_keyword = False):
   """
   Reads from the descriptor file until we get to one of the given keywords or reach the
@@ -419,17 +642,25 @@ def _read_until_keywords(keywords, descriptor_file, inclusive = False, ignore_fi
     **True**
   """
 
-  content = None if skip else []
+  if skip:
+    content = None
+    content_append = lambda x: None
+  else:
+    content = []
+    content_append = content.append
+
   ending_keyword = None
 
-  if isinstance(keywords, (bytes, unicode)):
+  if isinstance(keywords, (bytes, str_type)):
     keywords = (keywords,)
 
   if ignore_first:
     first_line = descriptor_file.readline()
 
-    if content is not None and first_line is not None:
-      content.append(first_line)
+    if first_line:
+      content_append(first_line)
+
+  keyword_match = re.compile(SPECIFIC_KEYWORD_LINE % '|'.join(keywords))
 
   while True:
     last_position = descriptor_file.tell()
@@ -442,25 +673,19 @@ def _read_until_keywords(keywords, descriptor_file, inclusive = False, ignore_fi
     if not line:
       break  # EOF
 
-    line_match = KEYWORD_LINE.match(stem.util.str_tools._to_unicode(line))
+    line_match = keyword_match.match(stem.util.str_tools._to_unicode(line))
 
-    if not line_match:
-      # no spaces or tabs in the line
-      line_keyword = stem.util.str_tools._to_unicode(line.strip())
-    else:
-      line_keyword = line_match.groups()[0]
-
-    if line_keyword in keywords:
-      ending_keyword = line_keyword
+    if line_match:
+      ending_keyword = line_match.groups()[0]
 
       if not inclusive:
         descriptor_file.seek(last_position)
-      elif content is not None:
-        content.append(line)
+      else:
+        content_append(line)
 
       break
-    elif content is not None:
-      content.append(line)
+    else:
+      content_append(line)
 
   if include_ending_keyword:
     return (content, ending_keyword)
@@ -468,6 +693,24 @@ def _read_until_keywords(keywords, descriptor_file, inclusive = False, ignore_fi
     return content
 
 
+def _bytes_for_block(content):
+  """
+  Provides the base64 decoded content of a pgp-style block.
+
+  :param str content: block to be decoded
+
+  :returns: decoded block content
+
+  :raises: **TypeError** if this isn't base64 encoded content
+  """
+
+  # strip the '-----BEGIN RSA PUBLIC KEY-----' header and footer
+
+  content = ''.join(content.split('\n')[1:-1])
+
+  return base64.b64decode(stem.util.str_tools._to_bytes(content))
+
+
 def _get_pseudo_pgp_block(remaining_contents):
   """
   Checks if given contents begins with a pseudo-Open-PGP-style block and, if
@@ -475,7 +718,7 @@ def _get_pseudo_pgp_block(remaining_contents):
 
   :param list remaining_contents: lines to be checked for a public key block
 
-  :returns: **str** with the armor wrapped contents or None if it doesn't exist
+  :returns: **tuple** of the (block_type, content) or None if it doesn't exist
 
   :raises: **ValueError** if the contents starts with a key block but it's
     malformed (for instance, if it lacks an ending line)
@@ -499,7 +742,7 @@ def _get_pseudo_pgp_block(remaining_contents):
       block_lines.append(line)
 
       if line == end_line:
-        return '\n'.join(block_lines)
+        return (block_type, '\n'.join(block_lines))
   else:
     return None
 
@@ -530,6 +773,9 @@ def _get_descriptor_components(raw_contents, validate, extra_keywords = ()):
     value tuple, the second being a list of those entries.
   """
 
+  if isinstance(raw_contents, bytes):
+    raw_contents = stem.util.str_tools._to_unicode(raw_contents)
+
   entries = OrderedDict()
   extra_entries = []  # entries with a keyword in extra_keywords
   remaining_lines = raw_contents.split('\n')
@@ -569,7 +815,12 @@ def _get_descriptor_components(raw_contents, validate, extra_keywords = ()):
       value = ''
 
     try:
-      block_contents = _get_pseudo_pgp_block(remaining_lines)
+      block_attr = _get_pseudo_pgp_block(remaining_lines)
+
+      if block_attr:
+        block_type, block_contents = block_attr
+      else:
+        block_type, block_contents = None, None
     except ValueError as exc:
       if not validate:
         continue
@@ -579,7 +830,7 @@ def _get_descriptor_components(raw_contents, validate, extra_keywords = ()):
     if keyword in extra_keywords:
       extra_entries.append('%s %s' % (keyword, value))
     else:
-      entries.setdefault(keyword, []).append((value, block_contents))
+      entries.setdefault(keyword, []).append((value, block_type, block_contents))
 
   if extra_keywords:
     return entries, extra_entries
@@ -593,3 +844,4 @@ import stem.descriptor.extrainfo_descriptor
 import stem.descriptor.networkstatus
 import stem.descriptor.microdescriptor
 import stem.descriptor.tordnsel
+import stem.descriptor.hidden_service_descriptor
