@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -12,9 +12,13 @@ Toolkit for exporting descriptors to other formats.
   export_csv_file - Writes exported CSV output to a file
 """
 
-import cStringIO
 import csv
 
+try:
+  from cStringIO import StringIO
+except ImportError:
+  from io import StringIO
+
 import stem.descriptor
 import stem.prereq
 
@@ -41,7 +45,7 @@ def export_csv(descriptors, included_fields = (), excluded_fields = (), header =
   :raises: **ValueError** if descriptors contain more than one descriptor type
   """
 
-  output_buffer = cStringIO.StringIO()
+  output_buffer = StringIO()
   export_csv_file(output_buffer, descriptors, included_fields, excluded_fields, header)
   return output_buffer.getvalue()
 
@@ -81,7 +85,7 @@ def export_csv_file(output_file, descriptors, included_fields = (), excluded_fie
 
   if included_fields:
     for field in included_fields:
-      if not field in desc_attr:
+      if field not in desc_attr:
         raise ValueError("%s does not have a '%s' attribute, valid fields are: %s" % (descriptor_type_label, field, ', '.join(desc_attr)))
   else:
     included_fields = [attr for attr in desc_attr if not attr.startswith('_')]
