@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -79,14 +79,20 @@ and picks up where it left off if run again...
 
 import mimetypes
 import os
-import Queue
 import tarfile
 import threading
 
+try:
+  import queue
+except ImportError:
+  import Queue as queue
+
 import stem.descriptor
 import stem.prereq
 import stem.util.system
 
+from stem import str_type
+
 # flag to indicate when the reader thread is out of descriptor files to read
 FINISHED = 'DONE'
 
@@ -180,7 +186,7 @@ def load_processed_files(path):
       if not line:
         continue  # skip blank lines
 
-      if not ' ' in line:
+      if ' ' not in line:
         raise TypeError('Malformed line: %s' % line)
 
       path, timestamp = line.rsplit(' ', 1)
@@ -221,7 +227,7 @@ def save_processed_files(path, processed_files):
     raise IOError(exc)
 
   with open(path, 'w') as output_file:
-    for path, timestamp in processed_files.items():
+    for path, timestamp in list(processed_files.items()):
       if not os.path.isabs(path):
         raise TypeError('Only absolute paths are acceptable: %s' % path)
 
@@ -257,15 +263,15 @@ class DescriptorReader(object):
   :param dict kwargs: additional arguments for the descriptor constructor
   """
 
-  def __init__(self, target, validate = True, follow_links = False, buffer_size = 100, persistence_path = None, document_handler = stem.descriptor.DocumentHandler.ENTRIES, **kwargs):
-    if isinstance(target, (bytes, unicode)):
+  def __init__(self, target, validate = False, follow_links = False, buffer_size = 100, persistence_path = None, document_handler = stem.descriptor.DocumentHandler.ENTRIES, **kwargs):
+    if isinstance(target, (bytes, str_type)):
       self._targets = [target]
     else:
       self._targets = target
 
     # expand any relative paths we got
 
-    target = map(os.path.abspath, target)
+    self._targets = list(map(os.path.abspath, self._targets))
 
     self._validate = validate
     self._follow_links = follow_links
@@ -288,7 +294,7 @@ class DescriptorReader(object):
     # Descriptors that we have read but not yet provided to the caller. A
     # FINISHED entry is used by the reading thread to indicate the end.
 
-    self._unreturned_descriptors = Queue.Queue(buffer_size)
+    self._unreturned_descriptors = queue.Queue(buffer_size)
 
     if self._persistence_path:
       try:
@@ -316,7 +322,7 @@ class DescriptorReader(object):
     """
 
     # make sure that we only provide back absolute paths
-    return dict((os.path.abspath(k), v) for (k, v) in self._processed_files.items())
+    return dict((os.path.abspath(k), v) for (k, v) in list(self._processed_files.items()))
 
   def set_processed_files(self, processed_files):
     """
@@ -400,7 +406,7 @@ class DescriptorReader(object):
       try:
         while True:
           self._unreturned_descriptors.get_nowait()
-      except Queue.Empty:
+      except queue.Empty:
         pass
 
       self._reader_thread.join()
@@ -447,7 +453,7 @@ class DescriptorReader(object):
             break
           else:
             yield descriptor
-        except Queue.Empty:
+        except queue.Empty:
           self._iter_notice.wait()
           self._iter_notice.clear()
 
@@ -537,7 +543,7 @@ class DescriptorReader(object):
                 return
 
               desc._set_path(os.path.abspath(target))
-              desc._set_archive_path(entry.name)
+              desc._set_archive_path(tar_entry.name)
               self._unreturned_descriptors.put(desc)
               self._iter_notice.set()
           except TypeError as exc:
