@@ -1,4 +1,4 @@
-# Copyright 2013-2014, Damian Johnson and The Tor Project
+# Copyright 2013-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -77,9 +77,13 @@ import random
 import sys
 import threading
 import time
-import urllib2
 import zlib
 
+try:
+    import urllib.request as urllib
+except ImportError:
+    import urllib2 as urllib
+
 import stem.descriptor
 
 from stem import Flag
@@ -212,7 +216,7 @@ class Query(object):
     the same as running **query.run(True)** (default is **False**)
   """
 
-  def __init__(self, resource, descriptor_type = None, endpoints = None, retries = 2, fall_back_to_authority = False, timeout = None, start = True, block = False, validate = True, document_handler = stem.descriptor.DocumentHandler.ENTRIES, **kwargs):
+  def __init__(self, resource, descriptor_type = None, endpoints = None, retries = 2, fall_back_to_authority = False, timeout = None, start = True, block = False, validate = False, document_handler = stem.descriptor.DocumentHandler.ENTRIES, **kwargs):
     if not resource.startswith('/'):
       raise ValueError("Resources should start with a '/': %s" % resource)
 
@@ -353,7 +357,7 @@ class Query(object):
       self.download_url = self._pick_url(use_authority)
 
       self.start_time = time.time()
-      response = urllib2.urlopen(self.download_url, timeout = self.timeout).read()
+      response = urllib.urlopen(self.download_url, timeout = self.timeout).read()
 
       if self.download_url.endswith('.z'):
         response = zlib.decompress(response)
@@ -649,7 +653,7 @@ class DirectoryAuthority(object):
      Added the is_bandwidth_authority attribute.
 
   :var str nickname: nickname of the authority
-  :var str address: IP address of the authority, presently they're all IPv4 but
+  :var str address: IP address of the authority, currently they're all IPv4 but
     this may not always be the case
   :var int or_port: port on which the relay services relay traffic
   :var int dir_port: port on which directory information is available
@@ -742,7 +746,7 @@ DIRECTORY_AUTHORITIES = {
   ),
   'Faravahar': DirectoryAuthority(
     nickname = 'Faravahar',
-    address = '154.35.32.5',
+    address = '154.35.175.225',
     or_port = 443,
     dir_port = 80,
     is_bandwidth_authority = False,
