@@ -1,12 +1,14 @@
-# Copyright 2013-2014, Damian Johnson and The Tor Project
+# Copyright 2013-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
 Parsing for `TorDNSEL <https://www.torproject.org/projects/tordnsel.html.en>`_
 exit list files.
-"""
 
-import datetime
+::
+
+  TorDNSEL - Exit list provided by TorDNSEL
+"""
 
 import stem.util.connection
 import stem.util.str_tools
@@ -19,7 +21,7 @@ from stem.descriptor import (
 )
 
 
-def _parse_file(tordnsel_file, validate = True, **kwargs):
+def _parse_file(tordnsel_file, validate = False, **kwargs):
   """
   Iterates over a tordnsel file.
 
@@ -72,8 +74,8 @@ class TorDNSEL(Descriptor):
 
   def _parse(self, entries, validate):
 
-    for keyword, values in entries.items():
-      value, block_content = values[0]
+    for keyword, values in list(entries.items()):
+      value, block_type, block_content = values[0]
 
       if validate and block_content:
         raise ValueError('Unexpected block content: %s' % block_content)
@@ -85,18 +87,18 @@ class TorDNSEL(Descriptor):
         self.fingerprint = value
       elif keyword == 'Published':
         try:
-          self.published = datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
+          self.published = stem.util.str_tools._parse_timestamp(value)
         except ValueError:
           if validate:
             raise ValueError("Published time wasn't parsable: %s" % value)
       elif keyword == 'LastStatus':
         try:
-          self.last_status = datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
+          self.last_status = stem.util.str_tools._parse_timestamp(value)
         except ValueError:
           if validate:
             raise ValueError("LastStatus time wasn't parsable: %s" % value)
       elif keyword == 'ExitAddress':
-        for value, block_content in values:
+        for value, block_type, block_content in values:
           address, date = value.split(' ', 1)
 
           if validate:
@@ -106,7 +108,7 @@ class TorDNSEL(Descriptor):
               raise ValueError('Unexpected block content: %s' % block_content)
 
           try:
-            date = datetime.datetime.strptime(date, '%Y-%m-%d %H:%M:%S')
+            date = stem.util.str_tools._parse_timestamp(date)
             self.exit_addresses.append((address, date))
           except ValueError:
             if validate:
