@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -11,6 +11,8 @@ Parses replies from the control socket.
   convert - translates a ControlMessage into a particular response subclass
 
   ControlMessage - Message that's read from the control socket.
+    |- SingleLineResponse - Simple tor response only including a single line of information.
+    |
     |- from_str - provides a ControlMessage for the given string
     |- is_ok - response had a 250 status
     |- content - provides the parsed message content
@@ -26,11 +28,10 @@ Parses replies from the control socket.
     |- peek_key - provides the key of the next entry
     |- pop - removes and returns the next entry
     +- pop_mapping - removes and returns the next entry as a KEY=VALUE mapping
-
-  SingleLineResponse - Simple tor response only including a single line of information.
 """
 
 __all__ = [
+  'add_onion',
   'events',
   'getinfo',
   'getconf',
@@ -43,9 +44,13 @@ __all__ = [
 ]
 
 import re
-import StringIO
 import threading
 
+try:
+  from StringIO import StringIO
+except ImportError:
+  from io import StringIO
+
 import stem.socket
 
 KEY_ARG = re.compile('^(\S+)=')
@@ -99,24 +104,26 @@ def convert(response_type, message, **kwargs):
       or response_type isn't supported
   """
 
+  import stem.response.add_onion
+  import stem.response.authchallenge
   import stem.response.events
   import stem.response.getinfo
   import stem.response.getconf
-  import stem.response.protocolinfo
-  import stem.response.authchallenge
   import stem.response.mapaddress
+  import stem.response.protocolinfo
 
   if not isinstance(message, ControlMessage):
     raise TypeError('Only able to convert stem.response.ControlMessage instances')
 
   response_types = {
+    'ADD_ONION': stem.response.add_onion.AddOnionResponse,
+    'AUTHCHALLENGE': stem.response.authchallenge.AuthChallengeResponse,
     'EVENT': stem.response.events.Event,
     'GETINFO': stem.response.getinfo.GetInfoResponse,
     'GETCONF': stem.response.getconf.GetConfResponse,
     'MAPADDRESS': stem.response.mapaddress.MapAddressResponse,
     'SINGLELINE': SingleLineResponse,
     'PROTOCOLINFO': stem.response.protocolinfo.ProtocolInfoResponse,
-    'AUTHCHALLENGE': stem.response.authchallenge.AuthChallengeResponse,
   }
 
   try:
@@ -149,7 +156,7 @@ class ControlMessage(object):
     :returns: stem.response.ControlMessage instance
     """
 
-    msg = stem.socket.recv_message(StringIO.StringIO(content))
+    msg = stem.socket.recv_message(StringIO(content))
 
     if msg_type is not None:
       convert(msg_type, msg, **kwargs)
