@@ -1,7 +1,6 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
-import datetime
 import io
 import re
 import time
@@ -12,6 +11,7 @@ import stem.descriptor.router_status_entry
 import stem.response
 import stem.version
 
+from stem import str_type, int_type
 from stem.util import connection, log, str_tools, tor_tools
 
 # Matches keyword=value arguments. This can't be a simple "(.*)=(.*)" pattern
@@ -27,7 +27,7 @@ class Event(stem.response.ControlMessage):
   """
   Base for events we receive asynchronously, as described in section 4.1 of the
   `control-spec
-  <https://gitweb.torproject.org/torspec.git/blob/HEAD:/control-spec.txt>`_.
+  <https://gitweb.torproject.org/torspec.git/tree/control-spec.txt>`_.
 
   :var str type: event type
   :var int arrived_at: unix timestamp for when the message arrived
@@ -49,7 +49,7 @@ class Event(stem.response.ControlMessage):
     if not str(self).strip():
       raise stem.ProtocolError('Received a blank tor event. Events must at the very least have a type.')
 
-    self.type = str(self).split().pop(0)
+    self.type = str(self).split()[0]
     self.arrived_at = arrived_at
 
     # if we're a recognized event type then translate ourselves into that subclass
@@ -142,11 +142,11 @@ class Event(stem.response.ControlMessage):
     attr_values = getattr(self, attr)
 
     if attr_values:
-      if isinstance(attr_values, (bytes, unicode)):
+      if isinstance(attr_values, (bytes, str_type)):
         attr_values = [attr_values]
 
       for value in attr_values:
-        if not value in attr_enum:
+        if value not in attr_enum:
           log_id = 'event.%s.unknown_%s.%s' % (self.type.lower(), attr, value)
           unrecognized_msg = "%s event had an unrecognized %s (%s). Maybe a new addition to the control protocol? Full Event: '%s'" % (self.type, attr, value, self)
           log.log_once(log_id, log.INFO, unrecognized_msg)
@@ -190,12 +190,12 @@ class AddrMapEvent(Event):
         self.expiry = None
       else:
         try:
-          self.expiry = datetime.datetime.strptime(self.expiry, '%Y-%m-%d %H:%M:%S')
+          self.expiry = stem.util.str_tools._parse_timestamp(self.expiry)
         except ValueError:
           raise stem.ProtocolError('Unable to parse date in ADDRMAP event: %s' % self)
 
     if self.utc_expiry is not None:
-      self.utc_expiry = datetime.datetime.strptime(self.utc_expiry, '%Y-%m-%d %H:%M:%S')
+      self.utc_expiry = stem.util.str_tools._parse_timestamp(self.utc_expiry)
 
     if self.cached is not None:
       if self.cached == 'YES':
@@ -227,9 +227,12 @@ class AuthDirNewDescEvent(Event):
 
     if len(lines) < 5:
       raise stem.ProtocolError("AUTHDIR_NEWDESCS events must contain lines for at least the type, action, message, descriptor, and terminating 'OK'")
-    elif not lines[-1] == 'OK':
+    elif lines[-1] != 'OK':
       raise stem.ProtocolError("AUTHDIR_NEWDESCS doesn't end with an 'OK'")
 
+    # TODO: For stem 2.0.0 we should consider changing 'descriptor' to a
+    # ServerDescriptor instance.
+
     self.action = lines[1]
     self.message = lines[2]
     self.descriptor = '\n'.join(lines[3:-1])
@@ -256,8 +259,8 @@ class BandwidthEvent(Event):
     elif not self.read.isdigit() or not self.written.isdigit():
       raise stem.ProtocolError("A BW event's bytes sent and received should be a positive numeric value, received: %s" % self)
 
-    self.read = long(self.read)
-    self.written = long(self.written)
+    self.read = int_type(self.read)
+    self.written = int_type(self.written)
 
 
 class BuildTimeoutSetEvent(Event):
@@ -326,6 +329,11 @@ class CircuitEvent(Event):
   The CIRC event was one of the first Control Protocol V1 events and was
   introduced in tor version 0.1.1.1-alpha.
 
+  .. versionchanged:: 1.4.0
+     Added the socks_username and socks_password attributes which is used for
+     `stream isolation
+     <https://gitweb.torproject.org/torspec.git/tree/proposals/171-separate-streams.txt>`_.
+
   :var str id: circuit identifier
   :var stem.CircStatus status: reported status for the circuit
   :var tuple path: relays involved in the circuit, these are
@@ -338,6 +346,8 @@ class CircuitEvent(Event):
   :var datetime created: time when the circuit was created or cannibalized
   :var stem.CircClosureReason reason: reason for the circuit to be closed
   :var stem.CircClosureReason remote_reason: remote side's reason for the circuit to be closed
+  :var str socks_username: username for using this circuit
+  :var str socks_password: password for using this circuit
   """
 
   _POSITIONAL_ARGS = ('id', 'status', 'path')
@@ -349,6 +359,8 @@ class CircuitEvent(Event):
     'TIME_CREATED': 'created',
     'REASON': 'reason',
     'REMOTE_REASON': 'remote_reason',
+    'SOCKS_USERNAME': 'socks_username',
+    'SOCKS_PASSWORD': 'socks_password',
   }
 
   def _parse(self):
@@ -377,7 +389,7 @@ class CircuitEvent(Event):
     if not isinstance(other, CircuitEvent):
       return False
 
-    for attr in ('id', 'status', 'path', 'build_flags', 'purpose', 'hs_state', 'rend_query', 'created', 'reason', 'remote_reason'):
+    for attr in ('id', 'status', 'path', 'build_flags', 'purpose', 'hs_state', 'rend_query', 'created', 'reason', 'remote_reason', 'socks_username', 'socks_port'):
       my_attr = getattr(self, attr)
       other_attr = getattr(other, attr)
 
@@ -487,13 +499,13 @@ class ClientsSeenEvent(Event):
 
   def _parse(self):
     if self.start_time is not None:
-      self.start_time = datetime.datetime.strptime(self.start_time, '%Y-%m-%d %H:%M:%S')
+      self.start_time = stem.util.str_tools._parse_timestamp(self.start_time)
 
     if self.locales is not None:
       locale_to_count = {}
 
       for entry in self.locales.split(','):
-        if not '=' in entry:
+        if '=' not in entry:
           raise stem.ProtocolError("The CLIENTS_SEEN's CountrySummary should be a comma separated listing of '<locale>=<count>' mappings: %s" % self)
 
         locale, count = entry.split('=', 1)
@@ -513,7 +525,7 @@ class ClientsSeenEvent(Event):
       protocol_to_count = {}
 
       for entry in self.ip_versions.split(','):
-        if not '=' in entry:
+        if '=' not in entry:
           raise stem.ProtocolError("The CLIENTS_SEEN's IPVersions should be a comma separated listing of '<protocol>=<count>' mappings: %s" % self)
 
         protocol, count = entry.split('=', 1)
@@ -609,12 +621,15 @@ class GuardEvent(Event):
 
 class HSDescEvent(Event):
   """
-  Event triggered when we fetch a hidden service descriptor that presently isn't in our cache.
+  Event triggered when we fetch a hidden service descriptor that currently isn't in our cache.
 
   The HS_DESC event was introduced in tor version 0.2.5.2-alpha.
 
   .. versionadded:: 1.2.0
 
+  .. versionchanged:: 1.3.0
+     Added the reason attribute.
+
   :var stem.HSDescAction action: what is happening with the descriptor
   :var str address: hidden service address
   :var stem.HSAuth authentication: service's authentication method
@@ -622,10 +637,12 @@ class HSDescEvent(Event):
   :var str directory_fingerprint: hidden service directory's finterprint
   :var str directory_nickname: hidden service directory's nickname if it was provided
   :var str descriptor_id: descriptor identifier
+  :var stem.HSDescReason reason: reason the descriptor failed to be fetched
   """
 
   _VERSION_ADDED = stem.version.Requirement.EVENT_HS_DESC
   _POSITIONAL_ARGS = ('action', 'address', 'authentication', 'directory', 'descriptor_id')
+  _KEYWORD_ARGS = {'REASON': 'reason'}
 
   def _parse(self):
     self.directory_fingerprint = None
@@ -641,6 +658,47 @@ class HSDescEvent(Event):
     self._log_if_unrecognized('authentication', stem.HSAuth)
 
 
+class HSDescContentEvent(Event):
+  """
+  Provides the content of hidden service descriptors we fetch.
+
+  The HS_DESC_CONTENT event was introduced in tor version 0.2.7.1-alpha.
+
+  .. versionadded:: 1.4.0
+
+  :var str address: hidden service address
+  :var str descriptor_id: descriptor identifier
+  :var str directory: hidden service directory servicing the request
+  :var str directory_fingerprint: hidden service directory's finterprint
+  :var str directory_nickname: hidden service directory's nickname if it was provided
+  :var stem.descriptor.hidden_service_descriptor.HiddenServiceDescriptor descriptor: descriptor that was retrieved
+  """
+
+  _VERSION_ADDED = stem.version.Requirement.EVENT_HS_DESC_CONTENT
+  _POSITIONAL_ARGS = ('address', 'descriptor_id', 'directory')
+
+  def _parse(self):
+    if self.address == 'UNKNOWN':
+      self.address = None
+
+    self.directory_fingerprint = None
+    self.directory_nickname = None
+
+    try:
+      self.directory_fingerprint, self.directory_nickname = \
+        stem.control._parse_circ_entry(self.directory)
+    except stem.ProtocolError:
+      raise stem.ProtocolError("HS_DESC_CONTENT's directory doesn't match a ServerSpec: %s" % self)
+
+    # skip the first line (our positional arguments) and last ('OK')
+
+    desc_content = str_tools._to_bytes('\n'.join(str(self).splitlines()[1:-1]))
+    self.descriptor = None
+
+    if desc_content:
+      self.descriptor = list(stem.descriptor.hidden_service_descriptor._parse_file(io.BytesIO(desc_content)))[0]
+
+
 class LogEvent(Event):
   """
   Tor logging event. These are the most visible kind of event since, by
@@ -681,6 +739,9 @@ class NetworkStatusEvent(Event):
   def _parse(self):
     content = str(self).lstrip('NS\n').rstrip('\nOK')
 
+    # TODO: For stem 2.0.0 consider changing 'desc' to 'descriptors' to match
+    # our other events.
+
     self.desc = list(stem.descriptor.router_status_entry._parse_file(
       io.BytesIO(str_tools._to_bytes(content)),
       True,
@@ -706,6 +767,9 @@ class NewConsensusEvent(Event):
   def _parse(self):
     content = str(self).lstrip('NEWCONSENSUS\n').rstrip('\nOK')
 
+    # TODO: For stem 2.0.0 consider changing 'desc' to 'descriptors' to match
+    # our other events.
+
     self.desc = list(stem.descriptor.router_status_entry._parse_file(
       io.BytesIO(str_tools._to_bytes(content)),
       True,
@@ -779,7 +843,7 @@ class ORConnEvent(Event):
       self.endpoint_fingerprint, self.endpoint_nickname = \
         stem.control._parse_circ_entry(self.endpoint)
     except stem.ProtocolError:
-      if not ':' in self.endpoint:
+      if ':' not in self.endpoint:
         raise stem.ProtocolError("ORCONN endpoint is neither a relay nor 'address:port': %s" % self)
 
       address, port = self.endpoint.split(':', 1)
@@ -848,7 +912,8 @@ class StatusEvent(Event):
 
   :var stem.StatusType status_type: category of the status event
   :var stem.Runlevel runlevel: runlevel of the logged message
-  :var str message: logged message
+  :var str action: activity that caused this message
+  :var dict arguments: attributes about the event
   """
 
   _POSITIONAL_ARGS = ('runlevel', 'action')
@@ -864,6 +929,13 @@ class StatusEvent(Event):
     else:
       raise ValueError("BUG: Unrecognized status type (%s), likely an EVENT_TYPE_TO_CLASS addition without revising how 'status_type' is assigned." % self.type)
 
+    # Just an alias for our parent class' keyword_args since that already
+    # parses these for us. Unlike our other event types Tor commonly supplies
+    # arbitrary key/value pairs for these, so making an alias here to better
+    # draw attention that the StatusEvent will likely have them.
+
+    self.arguments = self.keyword_args
+
     self._log_if_unrecognized('runlevel', stem.Runlevel)
 
 
@@ -876,7 +948,8 @@ class StreamEvent(Event):
 
   :var str id: stream identifier
   :var stem.StreamStatus status: reported status for the stream
-  :var str circ_id: circuit that the stream is attached to
+  :var str circ_id: circuit that the stream is attached to, this is **None** of
+    the stream is unattached
   :var str target: destination of the stream
   :var str target_address: destination address (ip, hostname, or '(Tor_internal)')
   :var int target_port: destination port
@@ -902,7 +975,7 @@ class StreamEvent(Event):
     if self.target is None:
       raise stem.ProtocolError("STREAM event didn't have a target: %s" % self)
     else:
-      if not ':' in self.target:
+      if ':' not in self.target:
         raise stem.ProtocolError("Target location must be of the form 'address:port': %s" % self)
 
       address, port = self.target.rsplit(':', 1)
@@ -917,7 +990,7 @@ class StreamEvent(Event):
       self.source_address = None
       self.source_port = None
     else:
-      if not ':' in self.source_addr:
+      if ':' not in self.source_addr:
         raise stem.ProtocolError("Source location must be of the form 'address:port': %s" % self)
 
       address, port = self.source_addr.split(':', 1)
@@ -963,8 +1036,8 @@ class StreamBwEvent(Event):
     elif not self.read.isdigit() or not self.written.isdigit():
       raise stem.ProtocolError("A STREAM_BW event's bytes sent and received should be a positive numeric value, received: %s" % self)
 
-    self.read = long(self.read)
-    self.written = long(self.written)
+    self.read = int_type(self.read)
+    self.written = int_type(self.written)
 
 
 class TransportLaunchedEvent(Event):
@@ -986,7 +1059,7 @@ class TransportLaunchedEvent(Event):
   _VERSION_ADDED = stem.version.Requirement.EVENT_TRANSPORT_LAUNCHED
 
   def _parse(self):
-    if not self.type in ('server', 'client'):
+    if self.type not in ('server', 'client'):
       raise stem.ProtocolError("Transport type should either be 'server' or 'client': %s" % self)
 
     if not connection.is_valid_ipv4_address(self.address) and \
@@ -1037,8 +1110,8 @@ class ConnectionBandwidthEvent(Event):
     elif not tor_tools.is_valid_connection_id(self.id):
       raise stem.ProtocolError("Connection IDs must be one to sixteen alphanumeric characters, got '%s': %s" % (self.id, self))
 
-    self.read = long(self.read)
-    self.written = long(self.written)
+    self.read = int_type(self.read)
+    self.written = int_type(self.written)
 
     self._log_if_unrecognized('type', stem.ConnectionType)
 
@@ -1077,8 +1150,8 @@ class CircuitBandwidthEvent(Event):
     elif not tor_tools.is_valid_circuit_id(self.id):
       raise stem.ProtocolError("Circuit IDs must be one to sixteen alphanumeric characters, got '%s': %s" % (self.id, self))
 
-    self.read = long(self.read)
-    self.written = long(self.written)
+    self.read = int_type(self.read)
+    self.written = int_type(self.written)
 
 
 class CellStatsEvent(Event):
@@ -1204,7 +1277,7 @@ def _parse_cell_type_mapping(mapping):
   results = {}
 
   for entry in mapping.split(','):
-    if not ':' in entry:
+    if ':' not in entry:
       raise stem.ProtocolError("Mappings are expected to be of the form 'key:value', got '%s': %s" % (entry, mapping))
 
     key, value = entry.split(':', 1)
@@ -1236,6 +1309,7 @@ EVENT_TYPE_TO_CLASS = {
   'ERR': LogEvent,
   'GUARD': GuardEvent,
   'HS_DESC': HSDescEvent,
+  'HS_DESC_CONTENT': HSDescContentEvent,
   'INFO': LogEvent,
   'NEWCONSENSUS': NewConsensusEvent,
   'NEWDESC': NewDescEvent,
