@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 import stem.response
@@ -48,7 +48,7 @@ class GetConfResponse(stem.response.ControlMessage):
       else:
         key, value = (line.pop(), None)
 
-      if not key in self.entries:
+      if key not in self.entries:
         self.entries[key] = []
 
       if value is not None:
