@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 import stem.response
@@ -97,7 +97,7 @@ class ProtocolInfoResponse(stem.response.ControlMessage):
 
             # our auth_methods should have a single AuthMethod.UNKNOWN entry if
             # any unknown authentication methods exist
-            if not AuthMethod.UNKNOWN in auth_methods:
+            if AuthMethod.UNKNOWN not in auth_methods:
               auth_methods.append(AuthMethod.UNKNOWN)
 
         # parse optional COOKIEFILE mapping (quoted and can have escapes)
