@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -157,6 +157,8 @@ Here's an expanation of what happened...
     +- get_value - provides the value for a given key as a string
 """
 
+import inspect
+import os
 import threading
 
 from stem.util import log
@@ -234,7 +236,7 @@ def get_config(handle):
   :param str handle: unique identifier used to access this config instance
   """
 
-  if not handle in CONFS:
+  if handle not in CONFS:
     CONFS[handle] = Config()
 
   return CONFS[handle]
@@ -242,9 +244,13 @@ def get_config(handle):
 
 def uses_settings(handle, path, lazy_load = True):
   """
-  Provides a function that can be used as an annotation for other functions
-  that require settings to be loaded. Functions with this annotation will be
-  provided with the configuration as its 'config' keyword argument.
+  Provides a function that can be used as a decorator for other functions that
+  require settings to be loaded. Functions with this decorator will be provided
+  with the configuration as its 'config' keyword argument.
+
+  .. versionchanged:: 1.3.0
+     Omits the 'config' argument if the funcion we're decorating doesn't accept
+     it.
 
   ::
 
@@ -256,14 +262,14 @@ def uses_settings(handle, path, lazy_load = True):
 
   :param str handle: hande for the configuration
   :param str path: path where the configuration should be loaded from
-  :param bool lazy_load: loads the configuration file when the annotation is
+  :param bool lazy_load: loads the configuration file when the decorator is
     used if true, otherwise it's loaded right away
 
-  :returns: **function** that can be used as an annotation to provide the
+  :returns: **function** that can be used as a decorator to provide the
     configuration
 
   :raises: **IOError** if we fail to read the configuration file, if
-    **lazy_load** is true then this arises when we use the annotation
+    **lazy_load** is true then this arises when we use the decorator
   """
 
   config = get_config(handle)
@@ -272,17 +278,20 @@ def uses_settings(handle, path, lazy_load = True):
     config.load(path)
     config.set('settings_loaded', 'true')
 
-  def annotation(func):
+  def decorator(func):
     def wrapped(*args, **kwargs):
       if lazy_load and not config.get('settings_loaded', False):
         config.load(path)
         config.set('settings_loaded', 'true')
 
-      return func(*args, config = config, **kwargs)
+      if 'config' in inspect.getargspec(func).args:
+        return func(*args, config = config, **kwargs)
+      else:
+        return func(*args, **kwargs)
 
     return wrapped
 
-  return annotation
+  return decorator
 
 
 def parse_enum(key, value, enumeration):
@@ -350,7 +359,7 @@ def parse_enum_csv(key, value, enumeration, count = None):
     raise ValueError("The count must be None, an int, or two value tuple. Got '%s' (%s)'" % (count, type(count)))
 
   result = []
-  enum_keys = [k.upper() for k in enumeration.keys()]
+  enum_keys = [k.upper() for k in list(enumeration.keys())]
   enum_values = list(enumeration)
 
   for val in values:
@@ -453,10 +462,14 @@ class Config(object):
   def load(self, path = None):
     """
     Reads in the contents of the given path, adding its configuration values
-    to our current contents.
+    to our current contents. If the path is a directory then this loads each
+    of the files, recursively.
 
-    :param str path: file path to be loaded, this uses the last loaded path if
-      not provided
+    .. versionchanged:: 1.3.0
+       Added support for directories.
+
+    :param str path: file or directory path to be loaded, this uses the last
+      loaded path if not provided
 
     :raises:
       * **IOError** if we fail to read the file (it doesn't exist, insufficient
@@ -469,6 +482,13 @@ class Config(object):
     elif not self._path:
       raise ValueError('Unable to load configuration: no path provided')
 
+    if os.path.isdir(self._path):
+      for root, dirnames, filenames in os.walk(self._path):
+        for filename in filenames:
+          self.load(os.path.join(root, filename))
+
+      return
+
     with open(self._path, 'r') as config_file:
       read_contents = config_file.readlines()
 
@@ -573,7 +593,7 @@ class Config(object):
     :returns: **list** if strings for the configuration keys we've loaded
     """
 
-    return self._contents.keys()
+    return list(self._contents.keys())
 
   def unused_keys(self):
     """
@@ -687,14 +707,14 @@ class Config(object):
     elif isinstance(default, tuple):
       val = tuple(val)
     elif isinstance(default, dict):
-      valMap = OrderedDict()
+      val_map = OrderedDict()
       for entry in val:
         if '=>' in entry:
-          entryKey, entryVal = entry.split('=>', 1)
-          valMap[entryKey.strip()] = entryVal.strip()
+          entry_key, entry_val = entry.split('=>', 1)
+          val_map[entry_key.strip()] = entry_val.strip()
         else:
           log.debug('Ignoring invalid %s config entry (expected a mapping, but "%s" was missing "=>")' % (key, entry))
-      val = valMap
+      val = val_map
 
     return val
 
