@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -26,18 +26,21 @@ Connection and networking based utility functions.
   Method for resolving a process' connections.
 
   .. versionadded:: 1.1.0
+  .. versionchanged:: 1.4.0
+     Added **NETSTAT_WINDOWS**.
 
-  ================= ===========
-  Resolver          Description
-  ================= ===========
-  **PROC**          /proc contents
-  **NETSTAT**       netstat command
-  **SS**            ss command
-  **LSOF**          lsof command
-  **SOCKSTAT**      sockstat command under *nix
-  **BSD_SOCKSTAT**  sockstat command under FreeBSD
-  **BSD_PROCSTAT**  procstat command under FreeBSD
-  ================= ===========
+  ====================  ===========
+  Resolver              Description
+  ====================  ===========
+  **PROC**              /proc contents
+  **NETSTAT**           netstat
+  **NETSTAT_WINDOWS**   netstat command under Windows
+  **SS**                ss command
+  **LSOF**              lsof command
+  **SOCKSTAT**          sockstat command under *nix
+  **BSD_SOCKSTAT**      sockstat command under FreeBSD
+  **BSD_PROCSTAT**      procstat command under FreeBSD
+  ====================  ===========
 """
 
 import collections
@@ -50,6 +53,7 @@ import re
 import stem.util.proc
 import stem.util.system
 
+from stem import str_type
 from stem.util import conf, enum, log
 
 # Connection resolution is risky to log about since it's highly likely to
@@ -62,6 +66,7 @@ LOG_CONNECTION_RESOLUTION = False
 Resolver = enum.Enum(
   ('PROC', 'proc'),
   ('NETSTAT', 'netstat'),
+  ('NETSTAT_WINDOWS', 'netstat (windows)'),
   ('SS', 'ss'),
   ('LSOF', 'lsof'),
   ('SOCKSTAT', 'sockstat'),
@@ -90,6 +95,9 @@ RESOLVER_COMMAND = {
   # -n = prevents dns lookups, -p = include process
   Resolver.NETSTAT: 'netstat -np',
 
+  # -a = show all TCP/UDP connections, -n = numeric addresses and ports, -o = include pid
+  Resolver.NETSTAT_WINDOWS: 'netstat -ano',
+
   # -n = numeric ports, -p = include process, -t = tcp sockets, -u = udp sockets
   Resolver.SS: 'ss -nptu',
 
@@ -112,6 +120,9 @@ RESOLVER_FILTER = {
   # tcp        0    586 192.168.0.1:44284       38.229.79.2:443         ESTABLISHED 15843/tor
   Resolver.NETSTAT: '^{protocol}\s+.*\s+{local_address}:{local_port}\s+{remote_address}:{remote_port}\s+ESTABLISHED\s+{pid}/{name}\s*$',
 
+  # tcp        586 192.168.0.1:44284       38.229.79.2:443         ESTABLISHED 15843
+  Resolver.NETSTAT_WINDOWS: '^\s*{protocol}\s+{local_address}:{local_port}\s+{remote_address}:{remote_port}\s+ESTABLISHED\s+{pid}\s*$',
+
   # tcp    ESTAB      0      0           192.168.0.20:44415       38.229.79.2:443    users:(("tor",15843,9))
   Resolver.SS: '^{protocol}\s+ESTAB\s+.*\s+{local_address}:{local_port}\s+{remote_address}:{remote_port}\s+users:\(\("{name}",{pid},[0-9]+\)\)$',
 
@@ -131,14 +142,14 @@ RESOLVER_FILTER = {
 
 def get_connections(resolver, process_pid = None, process_name = None):
   """
-  Retrieves a list of the current connections for a given process. The provides
-  a list of Connection instances, which have five attributes...
+  Retrieves a list of the current connections for a given process. This
+  provides a list of Connection instances, which have five attributes...
 
-    * local_address (str)
-    * local_port (int)
-    * remote_address (str)
-    * remote_port (int)
-    * protocol (str, generally either 'tcp' or 'udp')
+    * **local_address** (str)
+    * **local_port** (int)
+    * **remote_address** (str)
+    * **remote_port** (int)
+    * **protocol** (str, generally either 'tcp' or 'udp')
 
   .. versionadded:: 1.1.0
 
@@ -170,7 +181,10 @@ def get_connections(resolver, process_pid = None, process_name = None):
     except ValueError:
       raise ValueError('Process pid was non-numeric: %s' % process_pid)
 
-  if process_pid is None and resolver in (Resolver.PROC, Resolver.BSD_PROCSTAT):
+  if process_pid is None and process_name and resolver == Resolver.NETSTAT_WINDOWS:
+    process_pid = stem.util.system.pid_by_name(process_name)
+
+  if process_pid is None and resolver in (Resolver.NETSTAT_WINDOWS, Resolver.PROC, Resolver.BSD_PROCSTAT):
     raise ValueError('%s resolution requires a pid' % resolver)
 
   if resolver == Resolver.PROC:
@@ -247,7 +261,6 @@ def system_resolvers(system = None):
 
   :returns: **list** of :data:`~stem.util.connection.Resolver` instances available on this platform
   """
-
   if system is None:
     if stem.util.system.is_gentoo():
       system = 'Gentoo'
@@ -255,11 +268,11 @@ def system_resolvers(system = None):
       system = platform.system()
 
   if system == 'Windows':
-    resolvers = []
+    resolvers = [Resolver.NETSTAT_WINDOWS]
   elif system in ('Darwin', 'OpenBSD'):
     resolvers = [Resolver.LSOF]
   elif system == 'FreeBSD':
-    # Netstat is available, but lacks a '-p' equivilant so we can't associate
+    # Netstat is available, but lacks a '-p' equivalent so we can't associate
     # the results to processes. The platform also has a ss command, but it
     # belongs to a spreadsheet application.
 
@@ -271,7 +284,7 @@ def system_resolvers(system = None):
 
   # remove any that aren't in the user's PATH
 
-  resolvers = filter(lambda r: stem.util.system.is_available(RESOLVER_COMMAND[r]), resolvers)
+  resolvers = [r for r in resolvers if stem.util.system.is_available(RESOLVER_COMMAND[r])]
 
   # proc resolution, by far, outperforms the others so defaults to this is able
 
@@ -336,7 +349,7 @@ def is_valid_ipv4_address(address):
   :returns: **True** if input is a valid IPv4 address, **False** otherwise
   """
 
-  if not isinstance(address, (bytes, unicode)):
+  if not isinstance(address, (bytes, str_type)):
     return False
 
   # checks if theres four period separated values
@@ -479,12 +492,12 @@ def expand_ipv6_address(address):
 
   # expands collapsed groupings, there can only be a single '::' in a valid
   # address
-  if "::" in address:
+  if '::' in address:
     missing_groups = 7 - address.count(':')
     address = address.replace('::', '::' + ':' * missing_groups)
 
   # inserts missing zeros
-  for index in xrange(8):
+  for index in range(8):
     start = index * 5
     end = address.index(':', start) if index != 7 else len(address)
     missing_zeros = 4 - (end - start)
@@ -515,7 +528,7 @@ def get_mask_ipv4(bits):
   mask_bin = _get_binary(2 ** bits - 1, 32)[::-1]
 
   # breaks it into eight character groupings
-  octets = [mask_bin[8 * i:8 * (i + 1)] for i in xrange(4)]
+  octets = [mask_bin[8 * i:8 * (i + 1)] for i in range(4)]
 
   # converts each octet into its integer value
   return '.'.join([str(int(octet, 2)) for octet in octets])
@@ -542,7 +555,7 @@ def get_mask_ipv6(bits):
   mask_bin = _get_binary(2 ** bits - 1, 128)[::-1]
 
   # breaks it into sixteen character groupings
-  groupings = [mask_bin[16 * i:16 * (i + 1)] for i in xrange(8)]
+  groupings = [mask_bin[16 * i:16 * (i + 1)] for i in range(8)]
 
   # converts each group into its hex value
   return ':'.join(['%04x' % int(group, 2) for group in groupings]).upper()
