@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -40,7 +40,7 @@ constructed as simple type listings...
     +- __iter__ - iterator over our enum keys
 """
 
-import stem.util.str_tools
+from stem import str_type
 
 
 def UppercaseEnum(*args):
@@ -70,12 +70,14 @@ class Enum(object):
   """
 
   def __init__(self, *args):
+    from stem.util.str_tools import _to_camel_case
+
     # ordered listings of our keys and values
     keys, values = [], []
 
     for entry in args:
-      if isinstance(entry, (bytes, unicode)):
-        key, val = entry, stem.util.str_tools._to_camel_case(entry)
+      if isinstance(entry, (bytes, str_type)):
+        key, val = entry, _to_camel_case(entry)
       elif isinstance(entry, tuple) and len(entry) == 2:
         key, val = entry
       else:
@@ -121,7 +123,7 @@ class Enum(object):
     :raises: **ValueError** if no such element exists
     """
 
-    if not value in self._values:
+    if value not in self._values:
       raise ValueError('No such enumeration exists: %s (options: %s)' % (value, ', '.join(self._values)))
 
     next_index = (self._values.index(value) + 1) % len(self._values)
@@ -138,7 +140,7 @@ class Enum(object):
     :raises: **ValueError** if no such element exists
     """
 
-    if not value in self._values:
+    if value not in self._values:
       raise ValueError('No such enumeration exists: %s (options: %s)' % (value, ', '.join(self._values)))
 
     prev_index = (self._values.index(value) - 1) % len(self._values)
