@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -198,13 +198,17 @@ class LogBuffer(logging.Handler):
   """
   Basic log handler that listens for stem events and stores them so they can be
   read later. Log entries are cleared as they are read.
+
+    .. versionchanged:: 1.4.0
+       Added the yield_records argument.
   """
 
-  def __init__(self, runlevel):
+  def __init__(self, runlevel, yield_records = False):
     # TODO: At least in python 2.6 logging.Handler has a bug in that it doesn't
     # extend object, causing our super() call to fail. When we drop python 2.6
     # support we should switch back to using super() instead.
-    #super(LogBuffer, self).__init__(level = logging_level(runlevel))
+    #
+    # super(LogBuffer, self).__init__(level = logging_level(runlevel))
 
     logging.Handler.__init__(self, level = logging_level(runlevel))
 
@@ -213,13 +217,15 @@ class LogBuffer(logging.Handler):
       datefmt = '%m/%d/%Y %H:%M:%S')
 
     self._buffer = []
+    self._yield_records = yield_records
 
   def is_empty(self):
     return not bool(self._buffer)
 
   def __iter__(self):
     while self._buffer:
-      yield self.formatter.format(self._buffer.pop(0))
+      record = self._buffer.pop(0)
+      yield record if self._yield_records else self.formatter.format(record)
 
   def emit(self, record):
     self._buffer.append(record)
@@ -234,7 +240,7 @@ class _StdoutLogger(logging.Handler):
       datefmt = '%m/%d/%Y %H:%M:%S')
 
   def emit(self, record):
-    print self.formatter.format(record)
+    print(self.formatter.format(record))
 
 
 def log_to_stdout(runlevel):
