@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -15,22 +15,27 @@ Dave Daeschler, Giampaolo Rodola' and is under the BSD license.
 **These functions are not being vended to stem users. They may change in the
 future, use them at your own risk.**
 
+.. versionchanged:: 1.3.0
+   Dropped the get_* prefix from several function names. The old names still
+   work, but are deprecated aliases.
+
 **Module Overview:**
 
 ::
 
   is_available - checks if proc utilities can be used on this system
-  get_system_start_time - unix timestamp for when the system started
-  get_physical_memory - memory available on this system
-  get_cwd - provides the current working directory for a process
-  get_uid - provides the user id a process is running under
-  get_memory_usage - provides the memory usage of a process
-  get_stats - queries statistics about a process
-  get_connections - provides the connections made by a process
+  system_start_time - unix timestamp for when the system started
+  physical_memory - memory available on this system
+  cwd - provides the current working directory for a process
+  uid - provides the user id a process is running under
+  memory_usage - provides the memory usage of a process
+  stats - queries statistics about a process
+  file_descriptors_used - number of file descriptors used by a process
+  connections - provides the connections made by a process
 
 .. data:: Stat (enum)
 
-  Types of data available via the :func:`~stem.util.proc.get_stats` function.
+  Types of data available via the :func:`~stem.util.proc.stats` function.
 
   ============== ===========
   Stat           Description
@@ -93,7 +98,7 @@ def is_available():
 
 
 @lru_cache()
-def get_system_start_time():
+def system_start_time():
   """
   Provides the unix time (seconds since epoch) when the system started.
 
@@ -116,7 +121,7 @@ def get_system_start_time():
 
 
 @lru_cache()
-def get_physical_memory():
+def physical_memory():
   """
   Provides the total physical memory on the system in bytes.
 
@@ -138,7 +143,7 @@ def get_physical_memory():
     raise exc
 
 
-def get_cwd(pid):
+def cwd(pid):
   """
   Provides the current working directory for the given process.
 
@@ -166,7 +171,7 @@ def get_cwd(pid):
   return cwd
 
 
-def get_uid(pid):
+def uid(pid):
   """
   Provides the user ID the given process is running under.
 
@@ -191,7 +196,7 @@ def get_uid(pid):
     raise exc
 
 
-def get_memory_usage(pid):
+def memory_usage(pid):
   """
   Provides the memory usage in bytes for the given process.
 
@@ -224,7 +229,7 @@ def get_memory_usage(pid):
     raise exc
 
 
-def get_stats(pid, *stat_types):
+def stats(pid, *stat_types):
   """
   Provides process specific information. See the :data:`~stem.util.proc.Stat`
   enum for valid options.
@@ -280,19 +285,46 @@ def get_stats(pid, *stat_types):
         results.append(str(float(stat_comp[14]) / CLOCK_TICKS))
     elif stat_type == Stat.START_TIME:
       if pid == 0:
-        return get_system_start_time()
+        return system_start_time()
       else:
         # According to documentation, starttime is in field 21 and the unit is
         # jiffies (clock ticks). We divide it for clock ticks, then add the
         # uptime to get the seconds since the epoch.
         p_start_time = float(stat_comp[21]) / CLOCK_TICKS
-        results.append(str(p_start_time + get_system_start_time()))
+        results.append(str(p_start_time + system_start_time()))
 
   _log_runtime(parameter, stat_path, start_time)
   return tuple(results)
 
 
-def get_connections(pid):
+def file_descriptors_used(pid):
+  """
+  Provides the number of file descriptors currently being used by a process.
+
+  .. versionadded:: 1.3.0
+
+  :param int pid: process id of the process to be queried
+
+  :returns: **int** of the number of file descriptors used
+
+  :raises: **IOError** if it can't be determined
+  """
+
+  try:
+    pid = int(pid)
+
+    if pid < 0:
+      raise IOError("Process pids can't be negative: %s" % pid)
+  except (ValueError, TypeError):
+    raise IOError('Process pid was non-numeric: %s' % pid)
+
+  try:
+    return len(os.listdir('/proc/%i/fd' % pid))
+  except Exception as exc:
+    raise IOError('Unable to check number of file descriptors used: %s' % exc)
+
+
+def connections(pid):
   """
   Queries connection related information from the proc contents. This provides
   similar results to netstat, lsof, sockstat, and other connection resolution
@@ -307,11 +339,13 @@ def get_connections(pid):
   :raises: **IOError** if it can't be determined
   """
 
-  if isinstance(pid, str):
-    try:
-      pid = int(pid)
-    except ValueError:
-      raise IOError('Process pid was non-numeric: %s' % pid)
+  try:
+    pid = int(pid)
+
+    if pid < 0:
+      raise IOError("Process pids can't be negative: %s" % pid)
+  except (ValueError, TypeError):
+    raise IOError('Process pid was non-numeric: %s' % pid)
 
   if pid == 0:
     return []
@@ -331,9 +365,12 @@ def get_connections(pid):
 
       if fd_name.startswith('socket:['):
         inodes.append(fd_name[8:-1])
-    except OSError:
+    except OSError as exc:
+      if not os.path.exists(fd_path):
+        continue  # descriptors may shift while we're in the middle of iterating over them
+
       # most likely couldn't be read due to permissions
-      exc = IOError('unable to determine file descriptor destination: %s' % fd_path)
+      exc = IOError('unable to determine file descriptor destination (%s): %s' % (exc, fd_path))
       _log_failure(parameter, exc)
       raise exc
 
@@ -496,3 +533,15 @@ def _log_failure(parameter, exc):
   """
 
   log.debug('proc call failed (%s): %s' % (parameter, exc))
+
+# TODO: drop with stem 2.x
+# We renamed our methods to drop a redundant 'get_*' prefix, so alias the old
+# names for backward compatability.
+
+get_system_start_time = system_start_time
+get_physical_memory = physical_memory
+get_cwd = cwd
+get_uid = uid
+get_memory_usage = memory_usage
+get_stats = stats
+get_connections = connections
