@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -6,42 +6,49 @@ Helper functions for working with the underlying system. These are mostly os
 dependent, only working on linux, osx, and bsd. In almost all cases they're
 best-effort, providing **None** if the lookup fails.
 
+.. versionchanged:: 1.3.0
+   Dropped the get_* prefix from several function names. The old names still
+   work, but are deprecated aliases.
+
 **Module Overview:**
 
 ::
 
   is_windows - checks if we're running on windows
   is_mac - checks if we're running on a mac
+  is_gentoo - checks if we're running on gentoo
   is_bsd - checks if we're running on the bsd family of operating systems
 
   is_available - determines if a command is available on this system
   is_running - determines if a given process is running
   call - runs the given system command and provides back the results
 
-  get_name_by_pid - gets the name for a process by the given pid
-  get_pid_by_name - gets the pid for a process by the given name
-  get_pid_by_port - gets the pid for a process listening to a given port
-  get_pid_by_open_file - gets the pid for the process with an open file
-  get_cwd - provides the current working directory for a given process
-  get_user - provides the user a process is running under
-  get_start_time - provides the unix timestamp when the process started
-  get_bsd_jail_id - provides the BSD jail id a given process is running within
-  get_bsd_jail_path - provides the path of the given BSD jail
+  name_by_pid - gets the name for a process by the given pid
+  pid_by_name - gets the pid for a process by the given name
+  pid_by_port - gets the pid for a process listening to a given port
+  pid_by_open_file - gets the pid for the process with an open file
+  cwd - provides the current working directory for a given process
+  user - provides the user a process is running under
+  start_time - provides the unix timestamp when the process started
+  tail - provides lines from the end of a file
+  bsd_jail_id - provides the BSD jail id a given process is running within
+  bsd_jail_path - provides the path of the given BSD jail
 
   is_tarfile - checks if the given path is a tarball
   expand_path - expands relative paths and ~ entries
   files_with_suffix - provides files with the given suffix
 
-
   get_process_name - provides our process' name
   set_process_name - changes our process' name
 """
 
 import ctypes
 import ctypes.util
+import distutils.spawn
 import mimetypes
 import os
 import platform
+import re
 import subprocess
 import tarfile
 import time
@@ -49,7 +56,7 @@ import time
 import stem.util.proc
 import stem.util.str_tools
 
-from stem import UNDEFINED
+from stem import UNDEFINED, str_type
 from stem.util import log
 
 # Mapping of commands to if they're available or not.
@@ -82,6 +89,8 @@ GET_CWD_LSOF = 'lsof -a -p %s -d cwd -Fn'
 GET_BSD_JAIL_ID_PS = 'ps -p %s -o jid'
 GET_BSD_JAIL_PATH = 'jls -j %s'
 
+BLOCK_SIZE = 1024
+
 # flag for setting the process name, found in '/usr/include/linux/prctl.h'
 
 PR_SET_NAME = 15
@@ -137,9 +146,19 @@ def is_mac():
   return platform.system() == 'Darwin'
 
 
+def is_gentoo():
+  """
+  Checks if we're running on Gentoo.
+
+  :returns: **bool** to indicate if we're on Gentoo
+  """
+
+  return os.path.exists('/etc/gentoo-release')
+
+
 def is_bsd():
   """
-  Checks if we are within the BSD family of operating systems. This presently
+  Checks if we are within the BSD family of operating systems. This currently
   recognizes Macs, FreeBSD, and OpenBSD but may be expanded later.
 
   :returns: **bool** to indicate if we're on a BSD OS
@@ -175,17 +194,7 @@ def is_available(command, cached=True):
   elif cached and command in CMD_AVAILABLE_CACHE:
     return CMD_AVAILABLE_CACHE[command]
   else:
-    cmd_exists = False
-    for path in os.environ['PATH'].split(os.pathsep):
-      cmd_path = os.path.join(path, command)
-
-      if is_windows():
-        cmd_path += '.exe'
-
-      if os.path.exists(cmd_path) and os.access(cmd_path, os.X_OK):
-        cmd_exists = True
-        break
-
+    cmd_exists = distutils.spawn.find_executable(command) is not None
     CMD_AVAILABLE_CACHE[command] = cmd_exists
     return cmd_exists
 
@@ -227,13 +236,13 @@ def is_running(command):
       command_listing = call(secondary_resolver, None)
 
     if command_listing:
-      command_listing = map(unicode.strip, command_listing)
+      command_listing = map(str_type.strip, command_listing)
       return command in command_listing
 
   return None
 
 
-def get_name_by_pid(pid):
+def name_by_pid(pid):
   """
   Attempts to determine the name a given process is running under (not
   including arguments). This uses...
@@ -252,7 +261,7 @@ def get_name_by_pid(pid):
 
   if stem.util.proc.is_available():
     try:
-      process_name = stem.util.proc.get_stats(pid, stem.util.proc.Stat.COMMAND)[0]
+      process_name = stem.util.proc.stats(pid, stem.util.proc.Stat.COMMAND)[0]
     except IOError:
       pass
 
@@ -265,7 +274,10 @@ def get_name_by_pid(pid):
   #   vim
 
   if not process_name:
-    results = call(GET_NAME_BY_PID_PS % pid)
+    try:
+      results = call(GET_NAME_BY_PID_PS % pid)
+    except OSError:
+      results = None
 
     if results and len(results) == 2 and results[0] == 'COMMAND':
       process_name = results[1].strip()
@@ -273,7 +285,7 @@ def get_name_by_pid(pid):
   return process_name
 
 
-def get_pid_by_name(process_name, multiple = False):
+def pid_by_name(process_name, multiple = False):
   """
   Attempts to determine the process id for a running process, using...
 
@@ -284,6 +296,7 @@ def get_pid_by_name(process_name, multiple = False):
     3. ps -o pid -C <name> (linux)
        ps axc | egrep " <name>$" (bsd)
     4. lsof -tc <name>
+    5. tasklist | str <name>.exe
 
   :param str process_name: process name for which to fetch the pid
   :param bool multiple: provides a list of all pids if **True**, otherwise
@@ -309,7 +322,7 @@ def get_pid_by_name(process_name, multiple = False):
 
     if results:
       try:
-        pids = map(int, results)
+        pids = list(map(int, results))
 
         if multiple:
           return pids
@@ -330,7 +343,7 @@ def get_pid_by_name(process_name, multiple = False):
 
     if results and len(results) == 1:
       try:
-        pids = map(int, results[0].split())
+        pids = list(map(int, results[0].split()))
 
         if multiple:
           return pids
@@ -362,7 +375,7 @@ def get_pid_by_name(process_name, multiple = False):
 
       if results:
         try:
-          pids = map(int, results[1:])
+          pids = list(map(int, results[1:]))
 
           if multiple:
             return pids
@@ -380,7 +393,7 @@ def get_pid_by_name(process_name, multiple = False):
         results = [r.split()[0] for r in results if r.endswith(' %s' % process_name)]
 
         try:
-          pids = map(int, results)
+          pids = list(map(int, results))
 
           if multiple:
             return pids
@@ -408,7 +421,7 @@ def get_pid_by_name(process_name, multiple = False):
 
     if results:
       try:
-        pids = map(int, results)
+        pids = list(map(int, results))
 
         if multiple:
           return pids
@@ -417,11 +430,33 @@ def get_pid_by_name(process_name, multiple = False):
       except ValueError:
         pass
 
+  if is_available('tasklist') and is_windows():
+    if not process_name.endswith('.exe'):
+      process_name = process_name + '.exe'
+
+    process_ids = []
+
+    results = stem.util.system.call('tasklist', None)
+
+    if results:
+      tasklist_regex = re.compile('^\s*%s\s+(?P<pid>[0-9]*)' % process_name)
+
+      for line in results:
+        match = tasklist_regex.search(line)
+
+        if match:
+          process_ids.append(int(match.group('pid')))
+
+      if multiple:
+        return process_ids
+      elif len(process_ids) > 0:
+        return process_ids[0]
+
   log.debug("failed to resolve a pid for '%s'" % process_name)
   return [] if multiple else None
 
 
-def get_pid_by_port(port):
+def pid_by_port(port):
   """
   Attempts to determine the process id for a process with the given port,
   using...
@@ -541,7 +576,7 @@ def get_pid_by_port(port):
   return None  # all queries failed
 
 
-def get_pid_by_open_file(path):
+def pid_by_open_file(path):
   """
   Attempts to determine the process id for a process with the given open file,
   using...
@@ -579,7 +614,7 @@ def get_pid_by_open_file(path):
   return None  # all queries failed
 
 
-def get_cwd(pid):
+def cwd(pid):
   """
   Provides the working directory of the given process.
 
@@ -592,12 +627,12 @@ def get_cwd(pid):
   # try fetching via the proc contents if it's available
   if stem.util.proc.is_available():
     try:
-      return stem.util.proc.get_cwd(pid)
+      return stem.util.proc.cwd(pid)
     except IOError:
       pass
 
   # Fall back to a pwdx query. This isn't available on BSD.
-  logging_prefix = 'get_cwd(%s):' % pid
+  logging_prefix = 'cwd(%s):' % pid
 
   if is_available('pwdx'):
     # pwdx results are of the form:
@@ -640,7 +675,7 @@ def get_cwd(pid):
       # p2683
       # n/proc/2683/cwd (readlink: Permission denied)
 
-      if not ' ' in lsof_result:
+      if ' ' not in lsof_result:
         return lsof_result
     else:
       log.debug('%s we got unexpected output from lsof: %s' % (logging_prefix, results))
@@ -648,7 +683,7 @@ def get_cwd(pid):
   return None  # all queries failed
 
 
-def get_user(pid):
+def user(pid):
   """
   Provides the user a process is running under.
 
@@ -665,7 +700,7 @@ def get_user(pid):
     try:
       import pwd  # only available on unix platforms
 
-      uid = stem.util.proc.get_uid(pid)
+      uid = stem.util.proc.uid(pid)
 
       if uid and uid.isdigit():
         return pwd.getpwuid(int(uid)).pw_name
@@ -681,7 +716,7 @@ def get_user(pid):
   return None
 
 
-def get_start_time(pid):
+def start_time(pid):
   """
   Provides the unix timestamp when the given process started.
 
@@ -696,7 +731,7 @@ def get_start_time(pid):
 
   if stem.util.proc.is_available():
     try:
-      return float(stem.util.proc.get_stats(pid, stem.util.proc.Stat.START_TIME)[0])
+      return float(stem.util.proc.stats(pid, stem.util.proc.Stat.START_TIME)[0])
     except IOError:
       pass
 
@@ -712,7 +747,60 @@ def get_start_time(pid):
   return None
 
 
-def get_bsd_jail_id(pid):
+def tail(target, lines = None):
+  """
+  Provides lines of a file starting with the end. For instance,
+  'tail -n 50 /tmp/my_log' could be done with...
+
+  ::
+
+    reversed(list(tail('/tmp/my_log', 50)))
+
+  :param str,file target: path or file object to read from
+  :param int lines: number of lines to read
+
+  :returns: **generator** that reads lines, starting with the end
+
+  :raises: **IOError** if unable to read the file
+  """
+
+  if isinstance(target, str):
+    with open(target) as target_file:
+      for line in tail(target_file, lines):
+        yield line
+
+      return
+
+  # based on snippet from...
+  # https://stackoverflow.com/questions/136168/get-last-n-lines-of-a-file-with-python-similar-to-tail
+
+  target.seek(0, 2)  # go to the end of the file
+  block_end_byte = target.tell()
+  block_number = -1
+  content = ''
+
+  while (lines is None or lines > 0) and block_end_byte > 0:
+    if (block_end_byte - BLOCK_SIZE > 0):
+      # read the last block we haven't yet read
+      target.seek(block_number * BLOCK_SIZE, 2)
+      content, completed_lines = (target.read(BLOCK_SIZE) + content).split('\n', 1)
+    else:
+      # reached the start of the file, just read what's left
+      target.seek(0, 0)
+      completed_lines = target.read(block_end_byte) + content
+
+    for line in reversed(completed_lines.splitlines()):
+      if lines is None or lines > 0:
+        if lines is not None:
+          lines -= 1
+
+        yield line
+
+    block_end_byte -= BLOCK_SIZE
+    block_number -= 1
+
+
+def bsd_jail_id(pid):
   """
   Gets the jail id for a process. These seem to only exist for FreeBSD (this
   style for jails does not exist on Linux, OSX, or OpenBSD).
@@ -742,12 +830,12 @@ def get_bsd_jail_id(pid):
   if os_name == 'FreeBSD':
     log.warn('Unable to get the jail id for process %s.' % pid)
   else:
-    log.debug('get_bsd_jail_id(%s): jail ids do not exist on %s' % (pid, os_name))
+    log.debug('bsd_jail_id(%s): jail ids do not exist on %s' % (pid, os_name))
 
   return 0
 
 
-def get_bsd_jail_path(jid):
+def bsd_jail_path(jid):
   """
   Provides the path of the given FreeBSD jail.
 
@@ -869,7 +957,7 @@ def call(command, default = UNDEFINED, ignore_exit_status = False):
   results. This is not actually ran in a shell so pipes and other shell syntax
   are not permitted.
 
-  :param str command: command to be issued
+  :param str,list command: command to be issued
   :param object default: response if the query fails
   :param bool ignore_exit_status: reports failure if our command's exit status
     was non-zero
@@ -879,11 +967,16 @@ def call(command, default = UNDEFINED, ignore_exit_status = False):
   :raises: **OSError** if this fails and no default was provided
   """
 
+  if isinstance(command, str):
+    command_list = command.split(' ')
+  else:
+    command_list = command
+
   try:
-    is_shell_command = command.split(' ')[0] in SHELL_COMMANDS
+    is_shell_command = command_list[0] in SHELL_COMMANDS
 
     start_time = time.time()
-    process = subprocess.Popen(command.split(), stdout = subprocess.PIPE, stderr = subprocess.PIPE, shell = is_shell_command)
+    process = subprocess.Popen(command_list, stdout = subprocess.PIPE, stderr = subprocess.PIPE, shell = is_shell_command)
 
     stdout, stderr = process.communicate()
     stdout, stderr = stdout.strip(), stderr.strip()
@@ -948,7 +1041,7 @@ def get_process_name():
 
       args, argc = [], argc_t()
 
-      for i in xrange(100):
+      for i in range(100):
         # The ending index can be either None or raise a ValueError when
         # accessed...
         #
@@ -1066,3 +1159,18 @@ def _set_proc_title(process_name):
     # AttributeError: dlsym(0x7fff6a41d1e0, setproctitle): symbol not found
 
     pass
+
+
+# TODO: drop with stem 2.x
+# We renamed our methods to drop a redundant 'get_*' prefix, so alias the old
+# names for backward compatability.
+
+get_name_by_pid = name_by_pid
+get_pid_by_name = pid_by_name
+get_pid_by_port = pid_by_port
+get_pid_by_open_file = pid_by_open_file
+get_cwd = cwd
+get_user = user
+get_start_time = start_time
+get_bsd_jail_id = bsd_jail_id
+get_bsd_jail_path = bsd_jail_path
