@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -15,6 +15,7 @@ Miscellaneous utility functions for working with tor.
   is_valid_circuit_id - checks if a string is a valid tor circuit id
   is_valid_stream_id - checks if a string is a valid tor stream id
   is_valid_connection_id - checks if a string is a valid tor connection id
+  is_valid_hidden_service_address - checks if a string is a valid hidden service address
   is_hex_digits - checks if a string is only made up of hex digits
 """
 
@@ -33,11 +34,13 @@ import re
 # case insensitive. Tor doesn't define this in the spec so flipping a coin
 # and going with case insensitive.
 
-HEX_DIGIT = '[0-9a-fA-F]'
-FINGERPRINT_PATTERN = re.compile('^%s{40}$' % HEX_DIGIT)
 NICKNAME_PATTERN = re.compile('^[a-zA-Z0-9]{1,19}$')
 CIRC_ID_PATTERN = re.compile('^[a-zA-Z0-9]{1,16}$')
 
+# Hidden service addresses are sixteen base32 characters.
+
+HS_ADDRESS_PATTERN = re.compile('^[a-z2-7]{16}$')
+
 
 def is_valid_fingerprint(entry, check_prefix = False):
   """
@@ -51,15 +54,16 @@ def is_valid_fingerprint(entry, check_prefix = False):
   :returns: **True** if the string could be a relay fingerprint, **False** otherwise
   """
 
-  if not isinstance(entry, (str, unicode)):
+  try:
+    if check_prefix:
+      if not entry or entry[0] != '$':
+        return False
+
+      entry = entry[1:]
+
+    return is_hex_digits(entry, 40)
+  except TypeError:
     return False
-  elif check_prefix:
-    if not entry or entry[0] != '$':
-      return False
-
-    entry = entry[1:]
-
-  return bool(FINGERPRINT_PATTERN.match(entry))
 
 
 def is_valid_nickname(entry):
@@ -71,11 +75,11 @@ def is_valid_nickname(entry):
   :returns: **True** if the string could be a nickname, **False** otherwise
   """
 
-  if not isinstance(entry, (str, unicode)):
+  try:
+    return bool(NICKNAME_PATTERN.match(entry))
+  except TypeError:
     return False
 
-  return bool(NICKNAME_PATTERN.match(entry))
-
 
 def is_valid_circuit_id(entry):
   """
@@ -84,11 +88,11 @@ def is_valid_circuit_id(entry):
   :returns: **True** if the string could be a circuit id, **False** otherwise
   """
 
-  if not isinstance(entry, (str, unicode)):
+  try:
+    return bool(CIRC_ID_PATTERN.match(entry))
+  except TypeError:
     return False
 
-  return bool(CIRC_ID_PATTERN.match(entry))
-
 
 def is_valid_stream_id(entry):
   """
@@ -112,6 +116,20 @@ def is_valid_connection_id(entry):
   return is_valid_circuit_id(entry)
 
 
+def is_valid_hidden_service_address(entry):
+  """
+  Checks if a string is a valid format for being a hidden service address (not
+  including the '.onion' suffix).
+
+  :returns: **True** if the string could be a hidden service address, **False** otherwise
+  """
+
+  try:
+    return bool(HS_ADDRESS_PATTERN.match(entry))
+  except TypeError:
+    return False
+
+
 def is_hex_digits(entry, count):
   """
   Checks if a string is the given number of hex digits. Digits represented by
@@ -120,7 +138,14 @@ def is_hex_digits(entry, count):
   :param str entry: string to be checked
   :param int count: number of hex digits to be checked for
 
-  :returns: **True** if the string matches this number
+  :returns: **True** if the given number of hex digits, **False** otherwise
   """
 
-  return bool(re.match('^%s{%i}$' % (HEX_DIGIT, count), entry))
+  try:
+    if len(entry) != count:
+      return False
+
+    int(entry, 16)  # attempt to convert it as hex
+    return True
+  except (ValueError, TypeError):
+    return False
