@@ -2,31 +2,35 @@
 Integration tests for the connect_* convenience functions.
 """
 
-import StringIO
 import sys
 import unittest
 
+try:
+  from StringIO import StringIO
+except ImportError:
+  from io import StringIO
+
 import stem.connection
 import test.runner
 
+from test.runner import require_controller
+
 
 class TestConnect(unittest.TestCase):
   def setUp(self):
     # prevents the function from printing to the real stdout
     self.original_stdout = sys.stdout
-    sys.stdout = StringIO.StringIO()
+    sys.stdout = StringIO()
 
   def tearDown(self):
     sys.stdout = self.original_stdout
 
+  @require_controller
   def test_connect(self):
     """
     Basic sanity checks for the connect function.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     control_socket = stem.connection.connect(
@@ -38,14 +42,12 @@ class TestConnect(unittest.TestCase):
 
     test.runner.exercise_controller(self, control_socket)
 
+  @require_controller
   def test_connect_port(self):
     """
     Basic sanity checks for the connect_port function.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     control_socket = stem.connection.connect_port(
@@ -58,16 +60,14 @@ class TestConnect(unittest.TestCase):
       test.runner.exercise_controller(self, control_socket)
       control_socket.close()
     else:
-      self.assertEquals(control_socket, None)
+      self.assertEqual(control_socket, None)
 
+  @require_controller
   def test_connect_socket_file(self):
     """
     Basic sanity checks for the connect_socket_file function.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     control_socket = stem.connection.connect_socket_file(
@@ -80,4 +80,4 @@ class TestConnect(unittest.TestCase):
       test.runner.exercise_controller(self, control_socket)
       control_socket.close()
     else:
-      self.assertEquals(control_socket, None)
+      self.assertEqual(control_socket, None)
