@@ -8,10 +8,13 @@ import time
 import unittest
 
 import stem.control
-import test.runner
 import stem.socket
 import stem.util.system
 
+import test.runner
+
+from test.runner import require_controller
+
 
 class StateObserver(object):
   """
@@ -35,72 +38,64 @@ class StateObserver(object):
 
 
 class TestBaseController(unittest.TestCase):
+  @require_controller
   def test_connect_repeatedly(self):
     """
     Connects and closes the socket repeatedly. This is a simple attempt to
     trigger concurrency issues.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif stem.util.system.is_mac():
+    if stem.util.system.is_mac():
       test.runner.skip(self, '(ticket #6235)')
       return
 
     with test.runner.get_runner().get_tor_socket() as control_socket:
       controller = stem.control.BaseController(control_socket)
 
-      for _ in xrange(250):
+      for _ in range(250):
         controller.connect()
         controller.close()
 
+  @require_controller
   def test_msg(self):
     """
     Tests a basic query with the msg() method.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_socket() as control_socket:
       controller = stem.control.BaseController(control_socket)
       test.runner.exercise_controller(self, controller)
 
+  @require_controller
   def test_msg_invalid(self):
     """
     Tests the msg() method against an invalid controller command.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_socket() as control_socket:
       controller = stem.control.BaseController(control_socket)
       response = controller.msg('invalid')
-      self.assertEquals('Unrecognized command "invalid"', str(response))
+      self.assertEqual('Unrecognized command "invalid"', str(response))
 
+  @require_controller
   def test_msg_invalid_getinfo(self):
     """
     Tests the msg() method against a non-existant GETINFO option.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_socket() as control_socket:
       controller = stem.control.BaseController(control_socket)
       response = controller.msg('GETINFO blarg')
-      self.assertEquals('Unrecognized key "blarg"', str(response))
+      self.assertEqual('Unrecognized key "blarg"', str(response))
 
+  @require_controller
   def test_msg_repeatedly(self):
     """
     Connects, sends a burst of messages, and closes the socket repeatedly. This
     is a simple attempt to trigger concurrency issues.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif stem.util.system.is_mac():
+    if stem.util.system.is_mac():
       test.runner.skip(self, '(ticket #6235)')
       return
 
@@ -108,7 +103,7 @@ class TestBaseController(unittest.TestCase):
       controller = stem.control.BaseController(control_socket)
 
       def run_getinfo():
-        for _ in xrange(150):
+        for _ in range(150):
           try:
             controller.msg('GETINFO version')
             controller.msg('GETINFO blarg')
@@ -118,19 +113,20 @@ class TestBaseController(unittest.TestCase):
 
       message_threads = []
 
-      for _ in xrange(5):
+      for _ in range(5):
         msg_thread = threading.Thread(target = run_getinfo)
         message_threads.append(msg_thread)
         msg_thread.setDaemon(True)
         msg_thread.start()
 
-      for index in xrange(100):
+      for index in range(100):
         controller.connect()
         controller.close()
 
       for msg_thread in message_threads:
         msg_thread.join()
 
+  @require_controller
   def test_asynchronous_event_handling(self):
     """
     Check that we can both receive asynchronous events while hammering our
@@ -138,9 +134,6 @@ class TestBaseController(unittest.TestCase):
     listeners will still receive all of the enqueued events.
     """
 
-    if test.runner.require_control(self):
-      return
-
     class ControlledListener(stem.control.BaseController):
       """
       Controller that blocks event handling until told to do so.
@@ -187,31 +180,27 @@ class TestBaseController(unittest.TestCase):
       for bw_event in controller.received_events:
         self.assertTrue(re.match('BW [0-9]+ [0-9]+', str(bw_event)))
         self.assertTrue(re.match('650 BW [0-9]+ [0-9]+\r\n', bw_event.raw_content()))
-        self.assertEquals(('650', ' '), bw_event.content()[0][:2])
+        self.assertEqual(('650', ' '), bw_event.content()[0][:2])
 
+  @require_controller
   def test_get_latest_heartbeat(self):
     """
     Basic check for get_latest_heartbeat().
     """
 
-    if test.runner.require_control(self):
-      return
-
     # makes a getinfo query, then checks that the heartbeat is close to now
     with test.runner.get_runner().get_tor_socket() as control_socket:
       controller = stem.control.BaseController(control_socket)
       controller.msg('GETINFO version')
       self.assertTrue((time.time() - controller.get_latest_heartbeat()) < 5)
 
+  @require_controller
   def test_status_notifications(self):
     """
     Checks basic functionality of the add_status_listener() and
     remove_status_listener() methods.
     """
 
-    if test.runner.require_control(self):
-      return
-
     state_observer = StateObserver()
 
     with test.runner.get_runner().get_tor_socket(False) as control_socket:
@@ -219,15 +208,15 @@ class TestBaseController(unittest.TestCase):
       controller.add_status_listener(state_observer.listener, False)
 
       controller.close()
-      self.assertEquals(controller, state_observer.controller)
-      self.assertEquals(stem.control.State.CLOSED, state_observer.state)
+      self.assertEqual(controller, state_observer.controller)
+      self.assertEqual(stem.control.State.CLOSED, state_observer.state)
       self.assertTrue(state_observer.timestamp <= time.time())
       self.assertTrue(state_observer.timestamp > time.time() - 1.0)
       state_observer.reset()
 
       controller.connect()
-      self.assertEquals(controller, state_observer.controller)
-      self.assertEquals(stem.control.State.INIT, state_observer.state)
+      self.assertEqual(controller, state_observer.controller)
+      self.assertEqual(stem.control.State.INIT, state_observer.state)
       self.assertTrue(state_observer.timestamp <= time.time())
       self.assertTrue(state_observer.timestamp > time.time() - 1.0)
       state_observer.reset()
@@ -235,8 +224,8 @@ class TestBaseController(unittest.TestCase):
       # cause the socket to shut down without calling close()
       controller.msg('Blarg!')
       self.assertRaises(stem.SocketClosed, controller.msg, 'blarg')
-      self.assertEquals(controller, state_observer.controller)
-      self.assertEquals(stem.control.State.CLOSED, state_observer.state)
+      self.assertEqual(controller, state_observer.controller)
+      self.assertEqual(stem.control.State.CLOSED, state_observer.state)
       self.assertTrue(state_observer.timestamp <= time.time())
       self.assertTrue(state_observer.timestamp > time.time() - 1.0)
       state_observer.reset()
@@ -244,9 +233,9 @@ class TestBaseController(unittest.TestCase):
       # remove listener and make sure we don't get further notices
       controller.remove_status_listener(state_observer.listener)
       controller.connect()
-      self.assertEquals(None, state_observer.controller)
-      self.assertEquals(None, state_observer.state)
-      self.assertEquals(None, state_observer.timestamp)
+      self.assertEqual(None, state_observer.controller)
+      self.assertEqual(None, state_observer.state)
+      self.assertEqual(None, state_observer.timestamp)
       state_observer.reset()
 
       # add with spawn as true, we need a little delay on this since we then
@@ -255,8 +244,8 @@ class TestBaseController(unittest.TestCase):
       controller.add_status_listener(state_observer.listener, True)
       controller.close()
       time.sleep(0.1)  # not much work going on so this doesn't need to be much
-      self.assertEquals(controller, state_observer.controller)
-      self.assertEquals(stem.control.State.CLOSED, state_observer.state)
+      self.assertEqual(controller, state_observer.controller)
+      self.assertEqual(stem.control.State.CLOSED, state_observer.state)
       self.assertTrue(state_observer.timestamp <= time.time())
       self.assertTrue(state_observer.timestamp > time.time() - 1.0)
       state_observer.reset()
