@@ -2,6 +2,7 @@
 Integration tests for the stem.control.Controller class.
 """
 
+import hashlib
 import os
 import shutil
 import socket
@@ -26,25 +27,58 @@ from stem.control import EventType, Listener, State
 from stem.exit_policy import ExitPolicy
 from stem.version import Requirement
 
+from test.util import register_new_capability
+
+from test.runner import (
+  require_controller,
+  require_version,
+  require_online,
+  only_run_once,
+)
+
 # Router status entry for a relay with a nickname other than 'Unnamed'. This is
 # used for a few tests that need to look up a relay.
 
 TEST_ROUTER_STATUS_ENTRY = None
 
 
+def random_fingerprint():
+  """
+  Provides a random 40 character hex string.
+  """
+
+  return hashlib.sha1(stem.util.str_tools._to_bytes(str(time.time()))).hexdigest().upper()
+
+
 class TestController(unittest.TestCase):
-  # TODO: We should find an event we can reliably trigger rather than using BW
-  # events with sleeps. This both slows our tests and can be unreliable on
-  # systems under a heavy load.
+  @only_run_once
+  @require_controller
+  def test_missing_capabilities(self):
+    """
+    Check to see if tor supports any events, signals, or features that we
+    don't.
+    """
+
+    with test.runner.get_runner().get_tor_controller() as controller:
+      for event in controller.get_info('events/names').split():
+        if event not in EventType:
+          register_new_capability('Event', event)
+
+      for signal in controller.get_info('signal/names').split():
+        if signal not in Signal:
+          register_new_capability('Signal', signal)
+
+      # new features should simply be added to enable_feature()'s docs
+
+      for feature in controller.get_info('features/names').split():
+        if feature not in ('EXTENDED_EVENTS', 'VERBOSE_NAMES'):
+          register_new_capability('Feature', feature)
 
   def test_from_port(self):
     """
     Basic sanity check for the from_port constructor.
     """
 
-    if test.runner.require_control(self):
-      return
-
     if test.runner.Torrc.PORT in test.runner.get_runner().get_options():
       with stem.control.Controller.from_port(port = test.runner.CONTROL_PORT) as controller:
         self.assertTrue(isinstance(controller, stem.control.Controller))
@@ -56,25 +90,19 @@ class TestController(unittest.TestCase):
     Basic sanity check for the from_socket_file constructor.
     """
 
-    if test.runner.require_control(self):
-      return
-
     if test.runner.Torrc.SOCKET in test.runner.get_runner().get_options():
       with stem.control.Controller.from_socket_file(path = test.runner.CONTROL_SOCKET_PATH) as controller:
         self.assertTrue(isinstance(controller, stem.control.Controller))
     else:
       self.assertRaises(stem.SocketError, stem.control.Controller.from_socket_file, test.runner.CONTROL_SOCKET_PATH)
 
+  @require_controller
+  @require_version(Requirement.EVENT_SIGNAL)
   def test_reset_notification(self):
     """
     Checks that a notificiation listener is... well, notified of SIGHUPs.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_version(self, stem.version.Requirement.EVENT_SIGNAL):
-      return
-
     with test.runner.get_runner().get_tor_controller() as controller:
       received_events = []
 
@@ -105,15 +133,13 @@ class TestController(unittest.TestCase):
 
       controller.reset_conf('__OwningControllerProcess')
 
+  @require_controller
   def test_event_handling(self):
     """
     Add a couple listeners for various events and make sure that they receive
     them. Then remove the listeners.
     """
 
-    if test.runner.require_control(self):
-      return
-
     event_notice1, event_notice2 = threading.Event(), threading.Event()
     event_buffer1, event_buffer2 = [], []
 
@@ -128,17 +154,18 @@ class TestController(unittest.TestCase):
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
-      controller.add_event_listener(listener1, EventType.BW)
-      controller.add_event_listener(listener2, EventType.BW, EventType.DEBUG)
+      controller.add_event_listener(listener1, EventType.CONF_CHANGED)
+      controller.add_event_listener(listener2, EventType.CONF_CHANGED, EventType.DEBUG)
 
-      # BW events occure at the rate of one per second, so wait a bit to let
-      # some accumulate.
+      # The NodeFamily is a harmless option we can toggle
+      controller.set_conf('NodeFamily', random_fingerprint())
 
-      event_notice1.wait(4)
-      self.assertTrue(len(event_buffer1) >= 1)
+      # Wait for the event. Assert that we get it within 10 seconds
+      event_notice1.wait(10)
+      self.assertEqual(len(event_buffer1), 1)
       event_notice1.clear()
 
-      event_notice2.wait(4)
+      event_notice2.wait(10)
       self.assertTrue(len(event_buffer2) >= 1)
       event_notice2.clear()
 
@@ -148,30 +175,27 @@ class TestController(unittest.TestCase):
 
       buffer2_size = len(event_buffer2)
 
-      event_notice1.wait(4)
-      self.assertTrue(len(event_buffer1) >= 2)
+      controller.set_conf('NodeFamily', random_fingerprint())
+      event_notice1.wait(10)
+      self.assertEqual(len(event_buffer1), 2)
+      event_notice1.clear()
 
-      event_notice2.wait(4)
       self.assertEqual(buffer2_size, len(event_buffer2))
 
       for event in event_buffer1:
         self.assertTrue(isinstance(event, stem.response.events.Event))
-        self.assertEqual(2, len(event.positional_args))
+        self.assertEqual(0, len(event.positional_args))
         self.assertEqual({}, event.keyword_args)
 
-        self.assertTrue(isinstance(event, stem.response.events.BandwidthEvent))
-        self.assertTrue(hasattr(event, 'read'))
-        self.assertTrue(hasattr(event, 'written'))
+        self.assertTrue(isinstance(event, stem.response.events.ConfChangedEvent))
 
+  @require_controller
   def test_reattaching_listeners(self):
     """
     Checks that event listeners are re-attached when a controller disconnects
     then reconnects to tor.
     """
 
-    if test.runner.require_control(self):
-      return
-
     event_notice = threading.Event()
     event_buffer = []
 
@@ -243,14 +267,12 @@ class TestController(unittest.TestCase):
         event_notice.wait(4)
         self.assertTrue(len(event_buffer) >= 1)
 
+  @require_controller
   def test_getinfo(self):
     """
     Exercises GETINFO with valid and invalid queries.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -283,14 +305,12 @@ class TestController(unittest.TestCase):
       self.assertEqual({}, controller.get_info([]))
       self.assertEqual({}, controller.get_info([], {}))
 
+  @require_controller
   def test_get_version(self):
     """
     Test that the convenient method get_version() works.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -298,15 +318,13 @@ class TestController(unittest.TestCase):
       self.assertTrue(isinstance(version, stem.version.Version))
       self.assertEqual(version, runner.get_tor_version())
 
+  @require_controller
   def test_get_exit_policy(self):
     """
     Sanity test for get_exit_policy(). We have the default policy (no
     ExitPolicy set) which is a little... long due to the boilerplate.
     """
 
-    if test.runner.require_control(self):
-      return
-
     expected = ExitPolicy(
       'reject 0.0.0.0/8:*',
       'reject 169.254.0.0/16:*',
@@ -343,28 +361,24 @@ class TestController(unittest.TestCase):
       policy_str = policy_str[:public_addr_start] + policy_str[public_addr_end:]
       self.assertEqual(str(expected), policy_str)
 
+  @require_controller
   def test_authenticate(self):
     """
     Test that the convenient method authenticate() works.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller(False) as controller:
       controller.authenticate(test.runner.CONTROL_PASSWORD)
       test.runner.exercise_controller(self, controller)
 
+  @require_controller
   def test_protocolinfo(self):
     """
     Test that the convenient method protocolinfo() works.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller(False) as controller:
@@ -390,14 +404,12 @@ class TestController(unittest.TestCase):
 
       self.assertEqual(tuple(auth_methods), protocolinfo.auth_methods)
 
+  @require_controller
   def test_getconf(self):
     """
     Exercises GETCONF with valid and invalid queries.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -455,21 +467,20 @@ class TestController(unittest.TestCase):
       self.assertEqual({}, controller.get_conf_map('', 'la-di-dah'))
       self.assertEqual({}, controller.get_conf_map([], 'la-di-dah'))
 
+  @require_controller
   def test_hidden_services_conf(self):
     """
     Exercises the hidden service family of methods (get_hidden_service_conf,
     set_hidden_service_conf, create_hidden_service, and remove_hidden_service).
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     test_dir = runner.get_test_dir()
     service1_path = os.path.join(test_dir, 'test_hidden_service1')
     service2_path = os.path.join(test_dir, 'test_hidden_service2')
     service3_path = os.path.join(test_dir, 'test_hidden_service3')
+    service4_path = os.path.join(test_dir, 'test_hidden_service4')
     empty_service_path = os.path.join(test_dir, 'test_hidden_service_empty')
 
     with runner.get_tor_controller() as controller:
@@ -537,26 +548,143 @@ class TestController(unittest.TestCase):
 
         controller.remove_hidden_service(hs_path, 8989)
         self.assertEqual(3, len(controller.get_hidden_service_conf()))
+
+        # add a new service, this time with client authentication
+
+        hs_path = os.path.join(os.getcwd(), service4_path)
+        hs_attributes = controller.create_hidden_service(hs_path, 8888, auth_type = 'basic', client_names = ['c1', 'c2'])
+
+        self.assertEqual(2, len(hs_attributes.hostname.splitlines()))
+        self.assertEqual(2, len(hs_attributes.hostname_for_client))
+        self.assertTrue(hs_attributes.hostname_for_client['c1'].endswith('.onion'))
+        self.assertTrue(hs_attributes.hostname_for_client['c2'].endswith('.onion'))
+
+        conf = controller.get_hidden_service_conf()
+        self.assertEqual(4, len(conf))
+        self.assertEqual(1, len(conf[hs_path]['HiddenServicePort']))
+
+        # remove a hidden service
+
+        controller.remove_hidden_service(hs_path, 8888)
+        self.assertEqual(3, len(controller.get_hidden_service_conf()))
       finally:
         controller.set_hidden_service_conf({})  # drop hidden services created during the test
 
         # clean up the hidden service directories created as part of this test
 
-        for path in (service1_path, service2_path, service3_path):
+        for path in (service1_path, service2_path, service3_path, service4_path):
           try:
             shutil.rmtree(path)
           except:
             pass
 
+  @require_controller
+  @require_version(Requirement.ADD_ONION)
+  def test_without_ephemeral_hidden_services(self):
+    """
+    Exercises ephemeral hidden service methods when none are present.
+    """
+
+    with test.runner.get_runner().get_tor_controller() as controller:
+      self.assertEqual([], controller.list_ephemeral_hidden_services())
+      self.assertEqual([], controller.list_ephemeral_hidden_services(detached = True))
+      self.assertEqual(False, controller.remove_ephemeral_hidden_service('gfzprpioee3hoppz'))
+
+  @require_controller
+  @require_version(Requirement.ADD_ONION)
+  def test_with_ephemeral_hidden_services(self):
+    """
+    Exercises creating ephemeral hidden services and methods when they're
+    present.
+    """
+
+    runner = test.runner.get_runner()
+
+    with runner.get_tor_controller() as controller:
+      for ports in (4567890, [4567, 4567890], {4567: 'not_an_address:4567'}):
+        try:
+          # try creating a service with an invalid port
+          response = controller.create_ephemeral_hidden_service(ports)
+          self.fail("we should've raised a stem.ProtocolError")
+        except stem.ProtocolError as exc:
+          self.assertEqual("ADD_ONION response didn't have an OK status: Invalid VIRTPORT/TARGET", str(exc))
+
+      response = controller.create_ephemeral_hidden_service(4567)
+      self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services())
+      self.assertTrue(response.private_key is not None)
+
+      # drop the service
+
+      self.assertEqual(True, controller.remove_ephemeral_hidden_service(response.service_id))
+      self.assertEqual([], controller.list_ephemeral_hidden_services())
+
+      # recreate the service with the same private key
+
+      recreate_response = controller.create_ephemeral_hidden_service(4567, key_type = response.private_key_type, key_content = response.private_key)
+      self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services())
+      self.assertEqual(response.service_id, recreate_response.service_id)
+
+      # the response only includes the private key when making a new one
+
+      self.assertEqual(None, recreate_response.private_key)
+      self.assertEqual(None, recreate_response.private_key_type)
+
+      # create a service where we never see the private key
+
+      response = controller.create_ephemeral_hidden_service(4568, discard_key = True)
+      self.assertTrue(response.service_id in controller.list_ephemeral_hidden_services())
+      self.assertEqual(None, response.private_key)
+      self.assertEqual(None, response.private_key_type)
+
+      # other controllers shouldn't be able to see these hidden services
+
+      with runner.get_tor_controller() as second_controller:
+        self.assertEqual(2, len(controller.list_ephemeral_hidden_services()))
+        self.assertEqual(0, len(second_controller.list_ephemeral_hidden_services()))
+
+  @require_controller
+  @require_version(Requirement.ADD_ONION)
+  def test_with_detached_ephemeral_hidden_services(self):
+    """
+    Exercises creating detached ephemeral hidden services and methods when
+    they're present.
+    """
+
+    runner = test.runner.get_runner()
+
+    with runner.get_tor_controller() as controller:
+      response = controller.create_ephemeral_hidden_service(4567, detached = True)
+      self.assertEqual([], controller.list_ephemeral_hidden_services())
+      self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services(detached = True))
+
+      # drop and recreate the service
+
+      self.assertEqual(True, controller.remove_ephemeral_hidden_service(response.service_id))
+      self.assertEqual([], controller.list_ephemeral_hidden_services(detached = True))
+      controller.create_ephemeral_hidden_service(4567, key_type = response.private_key_type, key_content = response.private_key, detached = True)
+      self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services(detached = True))
+
+      # other controllers should be able to see this service, and drop it
+
+      with runner.get_tor_controller() as second_controller:
+        self.assertEqual([response.service_id], second_controller.list_ephemeral_hidden_services(detached = True))
+        self.assertEqual(True, second_controller.remove_ephemeral_hidden_service(response.service_id))
+        self.assertEqual([], controller.list_ephemeral_hidden_services(detached = True))
+
+        # recreate the service and confirms that it outlives this controller
+
+        response = second_controller.create_ephemeral_hidden_service(4567, detached = True)
+
+      self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services(detached = True))
+      controller.remove_ephemeral_hidden_service(response.service_id)
+
+  @require_controller
   def test_set_conf(self):
     """
     Exercises set_conf(), reset_conf(), and set_options() methods with valid
     and invalid requests.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
     tmpdir = tempfile.mkdtemp()
 
@@ -623,16 +751,13 @@ class TestController(unittest.TestCase):
 
         shutil.rmtree(tmpdir)
 
+  @require_controller
+  @require_version(Requirement.LOADCONF)
   def test_loadconf(self):
     """
     Exercises Controller.load_conf with valid and invalid requests.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_version(self, stem.version.Requirement.LOADCONF):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -664,10 +789,8 @@ class TestController(unittest.TestCase):
         controller.load_conf(oldconf)
         controller.reset_conf('__OwningControllerProcess')
 
+  @require_controller
   def test_saveconf(self):
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     # only testing for success, since we need to run out of disk space to test
@@ -686,14 +809,12 @@ class TestController(unittest.TestCase):
         controller.save_conf()
         controller.reset_conf('__OwningControllerProcess')
 
+  @require_controller
   def test_get_ports(self):
     """
     Test Controller.get_ports against a running tor instance.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -709,14 +830,12 @@ class TestController(unittest.TestCase):
       else:
         self.assertEqual([], controller.get_ports(Listener.CONTROL))
 
+  @require_controller
   def test_get_listeners(self):
     """
     Test Controller.get_listeners against a running tor instance.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -732,42 +851,28 @@ class TestController(unittest.TestCase):
       else:
         self.assertEqual([], controller.get_listeners(Listener.CONTROL))
 
+  @require_controller
   def test_get_socks_listeners(self):
     """
     Test Controller.get_socks_listeners against a running tor instance.
     """
 
-    if test.runner.require_control(self):
-      return
-
-    runner = test.runner.get_runner()
-
-    with runner.get_tor_controller() as controller:
+    with test.runner.get_runner().get_tor_controller() as controller:
       self.assertEqual([('127.0.0.1', 1112)], controller.get_socks_listeners())
 
+  @require_controller
+  @require_online
+  @require_version(stem.version.Version('0.1.2.2-alpha'))
   def test_enable_feature(self):
     """
     Test Controller.enable_feature with valid and invalid inputs.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
-      if not test.runner.require_version(self, stem.version.Version('0.1.2.2-alpha')):
-        controller.enable_feature('VERBOSE_NAMES')
-
       self.assertTrue(controller.is_feature_enabled('VERBOSE_NAMES'))
 
-      orconn_output = controller.get_info('orconn-status')
-
-      # the orconn-status results will be empty if we don't have a connection
-      if orconn_output == '':
-        if test.runner.require_online(self):
-          return
-
       self.assertTrue('VERBOSE_NAMES' in controller._enabled_features)
       self.assertRaises(stem.InvalidArguments, controller.enable_feature, ['NOT', 'A', 'FEATURE'])
 
@@ -778,14 +883,12 @@ class TestController(unittest.TestCase):
       else:
         self.fail()
 
+  @require_controller
   def test_signal(self):
     """
     Test controller.signal with valid and invalid signals.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_controller() as controller:
       # valid signal
       controller.signal('CLEARDNSCACHE')
@@ -793,14 +896,12 @@ class TestController(unittest.TestCase):
       # invalid signals
       self.assertRaises(stem.InvalidArguments, controller.signal, 'FOOBAR')
 
+  @require_controller
   def test_newnym_availability(self):
     """
     Test the is_newnym_available and get_newnym_wait methods.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_controller() as controller:
       self.assertEqual(True, controller.is_newnym_available())
       self.assertEqual(0.0, controller.get_newnym_wait())
@@ -810,14 +911,10 @@ class TestController(unittest.TestCase):
       self.assertEqual(False, controller.is_newnym_available())
       self.assertTrue(controller.get_newnym_wait() > 9.0)
 
+  @require_controller
+  @require_online
+  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_extendcircuit(self):
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-    elif test.runner.require_version(self, Requirement.EXTENDCIRCUIT_PATH_OPTIONAL):
-      return
-
     with test.runner.get_runner().get_tor_controller() as controller:
       circuit_id = controller.extend_circuit('0')
 
@@ -830,18 +927,14 @@ class TestController(unittest.TestCase):
       self.assertRaises(stem.InvalidRequest, controller.extend_circuit, '0', 'thisroutershouldntexistbecausestemexists!@##$%#')
       self.assertRaises(stem.InvalidRequest, controller.extend_circuit, '0', 'thisroutershouldntexistbecausestemexists!@##$%#', 'foo')
 
+  @require_controller
+  @require_online
+  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_repurpose_circuit(self):
     """
     Tests Controller.repurpose_circuit with valid and invalid input.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-    elif test.runner.require_version(self, Requirement.EXTENDCIRCUIT_PATH_OPTIONAL):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -857,18 +950,14 @@ class TestController(unittest.TestCase):
       self.assertRaises(stem.InvalidRequest, controller.repurpose_circuit, 'f934h9f3h4', 'fooo')
       self.assertRaises(stem.InvalidRequest, controller.repurpose_circuit, '4', 'fooo')
 
+  @require_controller
+  @require_online
+  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_close_circuit(self):
     """
     Tests Controller.close_circuit with valid and invalid input.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-    elif test.runner.require_version(self, Requirement.EXTENDCIRCUIT_PATH_OPTIONAL):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -888,17 +977,14 @@ class TestController(unittest.TestCase):
       self.assertRaises(stem.InvalidArguments, controller.close_circuit, circuit_id + '1024')
       self.assertRaises(stem.InvalidRequest, controller.close_circuit, '')
 
+  @require_controller
+  @require_online
   def test_get_streams(self):
     """
     Tests Controller.get_streams().
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-
-    host = '38.229.72.14'   # www.torproject.org
+    host = socket.gethostbyname('www.torproject.org')
     port = 443
 
     runner = test.runner.get_runner()
@@ -916,16 +1002,13 @@ class TestController(unittest.TestCase):
 
     self.assertTrue('%s:%s' % (host, port) in [stream.target for stream in streams])
 
+  @require_controller
+  @require_online
   def test_close_stream(self):
     """
     Tests Controller.close_stream with valid and invalid input.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -958,12 +1041,9 @@ class TestController(unittest.TestCase):
 
       self.assertRaises(stem.InvalidArguments, controller.close_stream, 'blarg')
 
+  @require_controller
+  @require_online
   def test_mapaddress(self):
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
@@ -974,7 +1054,7 @@ class TestController(unittest.TestCase):
 
       # try up to 10 times to rule out transient network failures
 
-      for _ in xrange(10):
+      for _ in range(10):
         try:
           s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
           s.settimeout(30)
@@ -999,18 +1079,14 @@ class TestController(unittest.TestCase):
       ip_addr = response[response.find(b'\r\n\r\n'):].strip()
       self.assertTrue(stem.util.connection.is_valid_ipv4_address(stem.util.str_tools._to_unicode(ip_addr)))
 
+  @require_controller
+  @require_online
+  @require_version(Requirement.MICRODESCRIPTOR_IS_DEFAULT)
   def test_get_microdescriptor(self):
     """
     Basic checks for get_microdescriptor().
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_version(self, Requirement.MICRODESCRIPTOR_IS_DEFAULT):
-      return
-    elif test.runner.require_online(self):
-      return
-
     with test.runner.get_runner().get_tor_controller() as controller:
       # we should balk at invalid content
       self.assertRaises(ValueError, controller.get_microdescriptor, '')
@@ -1028,6 +1104,7 @@ class TestController(unittest.TestCase):
 
       self.assertEqual(md_by_fingerprint, md_by_nickname)
 
+  @require_controller
   def test_get_microdescriptors(self):
     """
     Fetches a few descriptors via the get_microdescriptors() method.
@@ -1035,9 +1112,7 @@ class TestController(unittest.TestCase):
 
     runner = test.runner.get_runner()
 
-    if test.runner.require_control(self):
-      return
-    elif not os.path.exists(runner.get_test_dir('cached-descriptors')):
+    if not os.path.exists(runner.get_test_dir('cached-descriptors')):
       test.runner.skip(self, '(no cached microdescriptors)')
       return
 
@@ -1051,6 +1126,7 @@ class TestController(unittest.TestCase):
         if count > 10:
           break
 
+  @require_controller
   def test_get_server_descriptor(self):
     """
     Basic checks for get_server_descriptor().
@@ -1058,9 +1134,7 @@ class TestController(unittest.TestCase):
 
     runner = test.runner.get_runner()
 
-    if test.runner.require_control(self):
-      return
-    elif runner.get_tor_version() >= Requirement.MICRODESCRIPTOR_IS_DEFAULT:
+    if runner.get_tor_version() >= Requirement.MICRODESCRIPTOR_IS_DEFAULT:
       test.runner.skip(self, '(requires server descriptors)')
       return
 
@@ -1082,6 +1156,7 @@ class TestController(unittest.TestCase):
 
       self.assertEqual(desc_by_fingerprint, desc_by_nickname)
 
+  @require_controller
   def test_get_server_descriptors(self):
     """
     Fetches a few descriptors via the get_server_descriptors() method.
@@ -1089,9 +1164,7 @@ class TestController(unittest.TestCase):
 
     runner = test.runner.get_runner()
 
-    if test.runner.require_control(self):
-      return
-    elif runner.get_tor_version() >= Requirement.MICRODESCRIPTOR_IS_DEFAULT:
+    if runner.get_tor_version() >= Requirement.MICRODESCRIPTOR_IS_DEFAULT:
       test.runner.skip(self, '(requires server descriptors)')
       return
 
@@ -1110,16 +1183,13 @@ class TestController(unittest.TestCase):
         if count > 10:
           break
 
+  @require_controller
+  @require_online
   def test_get_network_status(self):
     """
     Basic checks for get_network_status().
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-
     with test.runner.get_runner().get_tor_controller() as controller:
       # we should balk at invalid content
       self.assertRaises(ValueError, controller.get_network_status, '')
@@ -1137,6 +1207,8 @@ class TestController(unittest.TestCase):
 
       self.assertEqual(desc_by_fingerprint, desc_by_nickname)
 
+  @require_controller
+  @require_online
   def test_get_network_statuses(self):
     """
     Fetches a few descriptors via the get_network_statuses() method.
@@ -1144,11 +1216,6 @@ class TestController(unittest.TestCase):
 
     runner = test.runner.get_runner()
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-
     with runner.get_tor_controller() as controller:
       count = 0
 
@@ -1156,27 +1223,50 @@ class TestController(unittest.TestCase):
         self.assertTrue(desc.fingerprint is not None)
         self.assertTrue(desc.nickname is not None)
 
-        unrecognized_lines = desc.get_unrecognized_lines()
-
-        if unrecognized_lines:
-          self.fail('Unrecognized descriptor content: %s' % unrecognized_lines)
+        for line in desc.get_unrecognized_lines():
+          register_new_capability('Consensus Line', line)
 
         count += 1
         if count > 10:
           break
 
-  def test_attachstream(self):
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-    elif test.runner.require_version(self, Requirement.EXTENDCIRCUIT_PATH_OPTIONAL):
-      return
+  @require_controller
+  @require_online
+  @require_version(Requirement.HSFETCH)
+  def test_get_hidden_service_descriptor(self):
+    """
+    Fetches a few descriptors via the get_hidden_service_descriptor() method.
+    """
 
-    host = '38.229.72.14'   # www.torproject.org
+    runner = test.runner.get_runner()
+
+    with runner.get_tor_controller() as controller:
+      # fetch the descriptor for DuckDuckGo
+
+      desc = controller.get_hidden_service_descriptor('3g2upl4pq6kufc4m.onion')
+      self.assertTrue('MIGJAoGBAJ' in desc.permanent_key)
+
+      # try to fetch something that doesn't exist
+
+      try:
+        desc = controller.get_hidden_service_descriptor('m4cfuk6qp4lpu2g3')
+        self.fail("Didn't expect m4cfuk6qp4lpu2g3.onion to exist, but provided: %s" % desc)
+      except stem.DescriptorUnavailable as exc:
+        self.assertEqual('No running hidden service at m4cfuk6qp4lpu2g3.onion', str(exc))
+
+      # ... but shouldn't fail if we have a default argument or aren't awaiting the descriptor
+
+      self.assertEqual('pop goes the weasel', controller.get_hidden_service_descriptor('m4cfuk6qp4lpu2g5', 'pop goes the weasel'))
+      self.assertEqual(None, controller.get_hidden_service_descriptor('m4cfuk6qp4lpu2g5', await_result = False))
+
+  @require_controller
+  @require_online
+  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
+  def test_attachstream(self):
+    host = socket.gethostbyname('www.torproject.org')
     port = 80
 
-    circuit_id = None
+    circuit_id, streams = None, []
 
     def handle_streamcreated(stream):
       if stream.status == 'NEW' and circuit_id:
@@ -1184,7 +1274,7 @@ class TestController(unittest.TestCase):
 
     with test.runner.get_runner().get_tor_controller() as controller:
       # try 10 times to build a circuit we can connect through
-      for i in xrange(10):
+      for i in range(10):
         controller.add_event_listener(handle_streamcreated, stem.control.EventType.STREAM)
         controller.set_conf('__LeaveStreamsUnattached', '1')
 
@@ -1208,26 +1298,40 @@ class TestController(unittest.TestCase):
     self.assertTrue(our_stream.circ_id)
     self.assertTrue(circuit_id)
 
-    self.assertEquals(our_stream.circ_id, circuit_id)
+    self.assertEqual(our_stream.circ_id, circuit_id)
 
+  @require_controller
+  @require_online
+  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_get_circuits(self):
     """
     Fetches circuits via the get_circuits() method.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_online(self):
-      return
-    elif test.runner.require_version(self, Requirement.EXTENDCIRCUIT_PATH_OPTIONAL):
-      return
-
-    runner = test.runner.get_runner()
-    with runner.get_tor_controller() as controller:
+    with test.runner.get_runner().get_tor_controller() as controller:
       new_circ = controller.new_circuit()
       circuits = controller.get_circuits()
       self.assertTrue(new_circ in [circ.id for circ in circuits])
 
+  @require_controller
+  def test_transition_to_relay(self):
+    """
+    Transitions Tor to turn into a relay, then back to a client. This helps to
+    catch transition issues such as the one cited in :trac:`14901`.
+    """
+
+    with test.runner.get_runner().get_tor_controller() as controller:
+      self.assertEqual(None, controller.get_conf('OrPort'))
+
+      # DisableNetwork ensures no port is actually opened
+      controller.set_options({'OrPort': '9090', 'DisableNetwork': '1'})
+
+      # TODO once tor 0.2.7.x exists, test that we can generate a descriptor on demand.
+
+      self.assertEqual('9090', controller.get_conf('OrPort'))
+      controller.reset_conf('OrPort', 'DisableNetwork')
+      self.assertEqual(None, controller.get_conf('OrPort'))
+
   def _get_router_status_entry(self, controller):
     """
     Provides a router status entry for a relay with a nickname other than
