@@ -8,8 +8,12 @@ import unittest
 import stem.descriptor
 import test.runner
 
+from test.runner import only_run_once
+from test.util import register_new_capability
+
 
 class TestMicrodescriptor(unittest.TestCase):
+  @only_run_once
   def test_cached_microdescriptors(self):
     """
     Parses the cached microdescriptor file in our data directory, checking that
@@ -17,9 +21,6 @@ class TestMicrodescriptor(unittest.TestCase):
     descriptor additions.
     """
 
-    if test.runner.only_run_once(self, 'test_cached_microdescriptors'):
-      return
-
     descriptor_path = test.runner.get_runner().get_test_dir('cached-microdescs')
 
     if not os.path.exists(descriptor_path):
@@ -27,8 +28,6 @@ class TestMicrodescriptor(unittest.TestCase):
       return
 
     with open(descriptor_path, 'rb') as descriptor_file:
-      for desc in stem.descriptor.parse_file(descriptor_file, 'microdescriptor 1.0'):
-        unrecognized_lines = desc.get_unrecognized_lines()
-
-        if unrecognized_lines:
-          self.fail('Unrecognized microdescriptor content: %s' % unrecognized_lines)
+      for desc in stem.descriptor.parse_file(descriptor_file, 'microdescriptor 1.0', validate = True):
+        for line in desc.get_unrecognized_lines():
+          register_new_capability('Microdescriptor Line', line)
