@@ -4,6 +4,7 @@ Integration tests for stem.descriptor.remote.
 
 import unittest
 
+import stem.descriptor
 import stem.descriptor.extrainfo_descriptor
 import stem.descriptor.microdescriptor
 import stem.descriptor.networkstatus
@@ -12,8 +13,39 @@ import stem.descriptor.router_status_entry
 import stem.descriptor.server_descriptor
 import test.runner
 
+from test.runner import (
+  require_online,
+  only_run_once,
+)
+
 
 class TestDescriptorDownloader(unittest.TestCase):
+  @require_online
+  @only_run_once
+  def test_authorities_are_up_to_date(self):
+    """
+    Check that our hardcoded directory authority data matches the present
+    consensus.
+    """
+
+    downloader = stem.descriptor.remote.DescriptorDownloader()
+    consensus = downloader.get_consensus(document_handler = stem.descriptor.DocumentHandler.BARE_DOCUMENT).run()[0]
+
+    for auth in consensus.directory_authorities:
+      stem_auth = stem.descriptor.remote.get_authorities().get(auth.nickname)
+
+      if not stem_auth:
+        self.fail("%s isn't a recognized directory authority in stem" % auth.nickname)
+
+      for attr in ('address', 'v3ident', 'or_port', 'dir_port'):
+        if auth.nickname == 'moria1' and attr == 'address':
+          continue  # skip due to https://trac.torproject.org/projects/tor/ticket/14955
+
+        if getattr(auth, attr) != getattr(stem_auth, attr):
+          self.fail('%s has %s %s, but we expected %s' % (auth.nickname, attr, getattr(auth, attr), getattr(stem_auth, attr)))
+
+  @require_online
+  @only_run_once
   def test_using_authorities(self):
     """
     Fetches a descriptor from each of the directory authorities. This is
@@ -24,11 +56,6 @@ class TestDescriptorDownloader(unittest.TestCase):
     then this test will need to be updated.
     """
 
-    if test.runner.require_online(self):
-      return
-    elif test.runner.only_run_once(self, 'test_using_authorities'):
-      return
-
     queries = []
 
     for nickname, authority in stem.descriptor.remote.get_authorities().items():
@@ -37,6 +64,7 @@ class TestDescriptorDownloader(unittest.TestCase):
         'server-descriptor 1.0',
         endpoints = [(authority.address, authority.dir_port)],
         timeout = 30,
+        validate = True,
       ), authority))
 
     for query, authority in queries:
@@ -48,31 +76,25 @@ class TestDescriptorDownloader(unittest.TestCase):
       self.assertEqual(1, len(descriptors))
       self.assertEqual('moria1', descriptors[0].nickname)
 
+  @require_online
+  @only_run_once
   def test_use_directory_mirrors(self):
     """
     Checks that we can fetch and use a list of directory mirrors.
     """
 
-    if test.runner.require_online(self):
-      return
-    elif test.runner.only_run_once(self, 'test_use_directory_mirrors'):
-      return
-
     downloader = stem.descriptor.remote.DescriptorDownloader()
     downloader.use_directory_mirrors()
     self.assertTrue(len(downloader._endpoints) > 50)
 
+  @require_online
+  @only_run_once
   def test_get_server_descriptors(self):
     """
     Exercises the downloader's get_server_descriptors() method.
     """
 
-    if test.runner.require_online(self):
-      return
-    elif test.runner.only_run_once(self, 'test_get_server_descriptors'):
-      return
-
-    downloader = stem.descriptor.remote.DescriptorDownloader()
+    downloader = stem.descriptor.remote.DescriptorDownloader(validate = True)
 
     # Fetch a single descriptor and a batch. I'd love to also exercise
     # retrieving all descriptors, but that adds roughly a minute to the runtime
@@ -98,17 +120,14 @@ class TestDescriptorDownloader(unittest.TestCase):
 
     self.assertEqual(2, len(list(multiple_query)))
 
+  @require_online
+  @only_run_once
   def test_get_extrainfo_descriptors(self):
     """
     Exercises the downloader's get_extrainfo_descriptors() method.
     """
 
-    if test.runner.require_online(self):
-      return
-    elif test.runner.only_run_once(self, 'test_get_extrainfo_descriptors'):
-      return
-
-    downloader = stem.descriptor.remote.DescriptorDownloader()
+    downloader = stem.descriptor.remote.DescriptorDownloader(validate = True)
 
     single_query = downloader.get_extrainfo_descriptors('9695DFC35FFEB861329B9F1AB04C46397020CE31')
 
@@ -127,6 +146,8 @@ class TestDescriptorDownloader(unittest.TestCase):
 
     self.assertEqual(2, len(list(multiple_query)))
 
+  @require_online
+  @only_run_once
   def test_get_microdescriptors(self):
     """
     Exercises the downloader's get_microdescriptors() method.
@@ -134,15 +155,10 @@ class TestDescriptorDownloader(unittest.TestCase):
 
     # TODO: method needs to be fixed - not quite sure what's going wrong...
 
-    test.runner.skip(self, '(test presently broken)')
+    test.runner.skip(self, '(test currently broken)')
     return
 
-    if test.runner.require_online(self):
-      return
-    elif test.runner.only_run_once(self, 'test_get_microdescriptors'):
-      return
-
-    downloader = stem.descriptor.remote.DescriptorDownloader()
+    downloader = stem.descriptor.remote.DescriptorDownloader(validate = True)
 
     single_query = downloader.get_microdescriptors('6dCl6ab8CLo0LeMjxi/MZgVJiZgWN8WKTesWPBMtyTo')
 
@@ -161,17 +177,14 @@ class TestDescriptorDownloader(unittest.TestCase):
 
     self.assertEqual(2, len(list(multiple_query)))
 
+  @require_online
+  @only_run_once
   def test_get_consensus(self):
     """
     Exercises the downloader's get_consensus() method.
     """
 
-    if test.runner.require_online(self):
-      return
-    elif test.runner.only_run_once(self, 'test_get_consensus'):
-      return
-
-    downloader = stem.descriptor.remote.DescriptorDownloader()
+    downloader = stem.descriptor.remote.DescriptorDownloader(validate = True)
 
     consensus_query = downloader.get_consensus()
     consensus_query.run()
@@ -180,17 +193,14 @@ class TestDescriptorDownloader(unittest.TestCase):
     self.assertTrue(len(consensus) > 50)
     self.assertTrue(isinstance(consensus[0], stem.descriptor.router_status_entry.RouterStatusEntryV3))
 
+  @require_online
+  @only_run_once
   def test_get_key_certificates(self):
     """
     Exercises the downloader's get_key_certificates() method.
     """
 
-    if test.runner.require_online(self):
-      return
-    elif test.runner.only_run_once(self, 'test_get_key_certificates'):
-      return
-
-    downloader = stem.descriptor.remote.DescriptorDownloader()
+    downloader = stem.descriptor.remote.DescriptorDownloader(validate = True)
 
     single_query = downloader.get_key_certificates('D586D18309DED4CD6D57C18FDB97EFA96D330566')
 
