@@ -11,6 +11,7 @@ import stem.util.system
 import stem.version
 import test.runner
 
+from test.runner import require_controller
 from test.integ.util.system import filter_system_call
 
 try:
@@ -21,15 +22,13 @@ except ImportError:
 
 
 class TestProtocolInfo(unittest.TestCase):
+  @require_controller
   def test_parsing(self):
     """
     Makes a PROTOCOLINFO query and processes the response for our control
     connection.
     """
 
-    if test.runner.require_control(self):
-      return
-
     control_socket = test.runner.get_runner().get_tor_socket(False)
     control_socket.send('PROTOCOLINFO 1')
     protocolinfo_response = control_socket.recv()
@@ -45,6 +44,7 @@ class TestProtocolInfo(unittest.TestCase):
 
     self.assert_matches_test_config(protocolinfo_response)
 
+  @require_controller
   @patch('stem.util.proc.is_available', Mock(return_value = False))
   @patch('stem.util.system.is_available', Mock(return_value = True))
   def test_get_protocolinfo_path_expansion(self):
@@ -58,9 +58,6 @@ class TestProtocolInfo(unittest.TestCase):
     with the 'RELATIVE' target.
     """
 
-    if test.runner.require_control(self):
-      return
-
     if test.runner.Torrc.PORT in test.runner.get_runner().get_options():
       lookup_prefixes = (
         stem.util.system.GET_PID_BY_PORT_NETSTAT,
@@ -90,6 +87,7 @@ class TestProtocolInfo(unittest.TestCase):
       self.assertTrue(control_socket.is_alive())
       control_socket.close()
 
+  @require_controller
   def test_multiple_protocolinfo_calls(self):
     """
     Tests making repeated PROTOCOLINFO queries. This use case is interesting
@@ -97,23 +95,18 @@ class TestProtocolInfo(unittest.TestCase):
     re-establish it.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_socket(False) as control_socket:
       for _ in range(5):
         protocolinfo_response = stem.connection.get_protocolinfo(control_socket)
         self.assert_matches_test_config(protocolinfo_response)
 
+  @require_controller
   def test_pre_disconnected_query(self):
     """
     Tests making a PROTOCOLINFO query when previous use of the socket had
     already disconnected it.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_socket(False) as control_socket:
       # makes a couple protocolinfo queries outside of get_protocolinfo first
       control_socket.send('PROTOCOLINFO 1')
