@@ -9,16 +9,19 @@ import stem.socket
 import stem.version
 import test.runner
 
+from test.runner import (
+  require_controller,
+  require_version,
+)
+
 
 class TestControlMessage(unittest.TestCase):
+  @require_controller
   def test_unestablished_socket(self):
     """
     Checks message parsing when we have a valid but unauthenticated socket.
     """
 
-    if test.runner.require_control(self):
-      return
-
     # If an unauthenticated connection gets a message besides AUTHENTICATE or
     # PROTOCOLINFO then tor will give an 'Authentication required.' message and
     # hang up.
@@ -27,10 +30,10 @@ class TestControlMessage(unittest.TestCase):
     control_socket.send('GETINFO version')
 
     auth_required_response = control_socket.recv()
-    self.assertEquals('Authentication required.', str(auth_required_response))
-    self.assertEquals(['Authentication required.'], list(auth_required_response))
-    self.assertEquals('514 Authentication required.\r\n', auth_required_response.raw_content())
-    self.assertEquals([('514', ' ', 'Authentication required.')], auth_required_response.content())
+    self.assertEqual('Authentication required.', str(auth_required_response))
+    self.assertEqual(['Authentication required.'], list(auth_required_response))
+    self.assertEqual('514 Authentication required.\r\n', auth_required_response.raw_content())
+    self.assertEqual([('514', ' ', 'Authentication required.')], auth_required_response.content())
 
     # The socket's broken but doesn't realize it yet. These use cases are
     # checked in more depth by the ControlSocket integ tests.
@@ -54,67 +57,58 @@ class TestControlMessage(unittest.TestCase):
     self.assertRaises(stem.SocketClosed, control_socket.send, 'GETINFO version')
     self.assertRaises(stem.SocketClosed, control_socket.recv)
 
+  @require_controller
   def test_invalid_command(self):
     """
     Parses the response for a command which doesn't exist.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_socket() as control_socket:
       control_socket.send('blarg')
       unrecognized_command_response = control_socket.recv()
-      self.assertEquals('Unrecognized command "blarg"', str(unrecognized_command_response))
-      self.assertEquals(['Unrecognized command "blarg"'], list(unrecognized_command_response))
-      self.assertEquals('510 Unrecognized command "blarg"\r\n', unrecognized_command_response.raw_content())
-      self.assertEquals([('510', ' ', 'Unrecognized command "blarg"')], unrecognized_command_response.content())
+      self.assertEqual('Unrecognized command "blarg"', str(unrecognized_command_response))
+      self.assertEqual(['Unrecognized command "blarg"'], list(unrecognized_command_response))
+      self.assertEqual('510 Unrecognized command "blarg"\r\n', unrecognized_command_response.raw_content())
+      self.assertEqual([('510', ' ', 'Unrecognized command "blarg"')], unrecognized_command_response.content())
 
+  @require_controller
   def test_invalid_getinfo(self):
     """
     Parses the response for a GETINFO query which doesn't exist.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_socket() as control_socket:
       control_socket.send('GETINFO blarg')
       unrecognized_key_response = control_socket.recv()
-      self.assertEquals('Unrecognized key "blarg"', str(unrecognized_key_response))
-      self.assertEquals(['Unrecognized key "blarg"'], list(unrecognized_key_response))
-      self.assertEquals('552 Unrecognized key "blarg"\r\n', unrecognized_key_response.raw_content())
-      self.assertEquals([('552', ' ', 'Unrecognized key "blarg"')], unrecognized_key_response.content())
+      self.assertEqual('Unrecognized key "blarg"', str(unrecognized_key_response))
+      self.assertEqual(['Unrecognized key "blarg"'], list(unrecognized_key_response))
+      self.assertEqual('552 Unrecognized key "blarg"\r\n', unrecognized_key_response.raw_content())
+      self.assertEqual([('552', ' ', 'Unrecognized key "blarg"')], unrecognized_key_response.content())
 
+  @require_controller
   def test_getinfo_config_file(self):
     """
     Parses the 'GETINFO config-file' response.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
     torrc_dst = runner.get_torrc_path()
 
     with runner.get_tor_socket() as control_socket:
       control_socket.send('GETINFO config-file')
       config_file_response = control_socket.recv()
-      self.assertEquals('config-file=%s\nOK' % torrc_dst, str(config_file_response))
-      self.assertEquals(['config-file=%s' % torrc_dst, 'OK'], list(config_file_response))
-      self.assertEquals('250-config-file=%s\r\n250 OK\r\n' % torrc_dst, config_file_response.raw_content())
-      self.assertEquals([('250', '-', 'config-file=%s' % torrc_dst), ('250', ' ', 'OK')], config_file_response.content())
+      self.assertEqual('config-file=%s\nOK' % torrc_dst, str(config_file_response))
+      self.assertEqual(['config-file=%s' % torrc_dst, 'OK'], list(config_file_response))
+      self.assertEqual('250-config-file=%s\r\n250 OK\r\n' % torrc_dst, config_file_response.raw_content())
+      self.assertEqual([('250', '-', 'config-file=%s' % torrc_dst), ('250', ' ', 'OK')], config_file_response.content())
 
+  @require_controller
+  @require_version(stem.version.Requirement.GETINFO_CONFIG_TEXT)
   def test_getinfo_config_text(self):
     """
     Parses the 'GETINFO config-text' response.
     """
 
-    if test.runner.require_control(self):
-      return
-    elif test.runner.require_version(self, stem.version.Requirement.GETINFO_CONFIG_TEXT):
-      return
-
     runner = test.runner.get_runner()
 
     # We can't be certain of the order, and there may be extra config-text
@@ -150,21 +144,19 @@ class TestControlMessage(unittest.TestCase):
         self.assertTrue('%s\r\n' % torrc_entry in config_text_response.raw_content())
         self.assertTrue('%s' % torrc_entry in config_text_response.content()[0][2])
 
+  @require_controller
   def test_bw_event(self):
     """
     Issues 'SETEVENTS BW' and parses a couple events.
     """
 
-    if test.runner.require_control(self):
-      return
-
     with test.runner.get_runner().get_tor_socket() as control_socket:
       control_socket.send('SETEVENTS BW')
       setevents_response = control_socket.recv()
-      self.assertEquals('OK', str(setevents_response))
-      self.assertEquals(['OK'], list(setevents_response))
-      self.assertEquals('250 OK\r\n', setevents_response.raw_content())
-      self.assertEquals([('250', ' ', 'OK')], setevents_response.content())
+      self.assertEqual('OK', str(setevents_response))
+      self.assertEqual(['OK'], list(setevents_response))
+      self.assertEqual('250 OK\r\n', setevents_response.raw_content())
+      self.assertEqual([('250', ' ', 'OK')], setevents_response.content())
 
       # Tor will emit a BW event once per second. Parsing two of them.
 
@@ -172,4 +164,4 @@ class TestControlMessage(unittest.TestCase):
         bw_event = control_socket.recv()
         self.assertTrue(re.match('BW [0-9]+ [0-9]+', str(bw_event)))
         self.assertTrue(re.match('650 BW [0-9]+ [0-9]+\r\n', bw_event.raw_content()))
-        self.assertEquals(('650', ' '), bw_event.content()[0][:2])
+        self.assertEqual(('650', ' '), bw_event.content()[0][:2])
