@@ -19,9 +19,6 @@ except ImportError:
   from mock import Mock, patch
 
 
-IS_EXTRA_TOR_RUNNING = None
-
-
 def filter_system_call(prefixes):
   """
   Provides a functor that passes calls on to the stem.util.system.call()
@@ -73,7 +70,12 @@ class TestSystem(unittest.TestCase):
       test.runner.skip(self, '(ps unavailable)')
       return
 
-    self.assertTrue(stem.util.system.is_running('tor'))
+    # Check to see if the command we started tor with is running. The process
+    # might be running under another name so need to check for 'tor.real' too
+    # (#15449).
+
+    tor_cmd = test.runner.get_runner().get_tor_command(True)
+    self.assertTrue(stem.util.system.is_running(tor_cmd) or stem.util.system.is_running('tor.real'))
     self.assertFalse(stem.util.system.is_running('blarg_and_stuff'))
 
   def test_pid_by_name(self):
@@ -82,16 +84,14 @@ class TestSystem(unittest.TestCase):
     will fail if there's other tor instances running.
     """
 
-    if stem.util.system.is_windows():
-      test.runner.skip(self, '(unavailable on windows)')
-      return
-    elif self._is_extra_tor_running():
+    if self._is_extra_tor_running():
       test.runner.skip(self, '(multiple tor instances)')
       return
 
     tor_pid = test.runner.get_runner().get_pid()
-    self.assertEquals(tor_pid, stem.util.system.pid_by_name('tor'))
-    self.assertEquals(None, stem.util.system.pid_by_name('blarg_and_stuff'))
+    tor_cmd = test.runner.get_runner().get_tor_command(True)
+    self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
+    self.assertEqual(None, stem.util.system.pid_by_name('blarg_and_stuff'))
 
   def test_pid_by_name_pgrep(self):
     """
@@ -113,7 +113,8 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       tor_pid = test.runner.get_runner().get_pid()
-      self.assertEquals(tor_pid, stem.util.system.pid_by_name('tor'))
+      tor_cmd = test.runner.get_runner().get_tor_command(True)
+      self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
 
   def test_pid_by_name_pidof(self):
     """
@@ -135,7 +136,8 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       tor_pid = test.runner.get_runner().get_pid()
-      self.assertEquals(tor_pid, stem.util.system.pid_by_name('tor'))
+      tor_cmd = test.runner.get_runner().get_tor_command()
+      self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
 
   def test_pid_by_name_ps_linux(self):
     """
@@ -160,7 +162,8 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       tor_pid = test.runner.get_runner().get_pid()
-      self.assertEquals(tor_pid, stem.util.system.pid_by_name('tor'))
+      tor_cmd = test.runner.get_runner().get_tor_command(True)
+      self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
 
   def test_pid_by_name_ps_bsd(self):
     """
@@ -185,7 +188,8 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       tor_pid = test.runner.get_runner().get_pid()
-      self.assertEquals(tor_pid, stem.util.system.pid_by_name('tor'))
+      tor_cmd = test.runner.get_runner().get_tor_command(True)
+      self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
 
   def test_pid_by_name_lsof(self):
     """
@@ -210,17 +214,32 @@ class TestSystem(unittest.TestCase):
     with patch('stem.util.system.call') as call_mock:
       call_mock.side_effect = call_replacement
 
+      tor_cmd = test.runner.get_runner().get_tor_command(True)
       our_tor_pid = test.runner.get_runner().get_pid()
-      all_tor_pids = stem.util.system.pid_by_name('tor', multiple = True)
+      all_tor_pids = stem.util.system.pid_by_name(tor_cmd, multiple = True)
 
       if len(all_tor_pids) == 1:
-        self.assertEquals(our_tor_pid, all_tor_pids[0])
+        self.assertEqual(our_tor_pid, all_tor_pids[0])
+
+  def test_pid_by_name_tasklist(self):
+    """
+    Tests the pid_by_name function with a tasklist response.
+    """
+
+    if self._is_extra_tor_running():
+      test.runner.skip(self, '(multiple tor instances)')
+      return
+    elif not stem.util.system.is_available('tasklist'):
+      test.runner.skip(self, '(tasklist unavailable)')
+      return
+
+    runner = test.runner.get_runner()
+    self.assertEqual(runner.get_pid(), stem.util.system.pid_by_name(runner.get_tor_command(True)))
 
   def test_pid_by_port(self):
     """
     Checks general usage of the stem.util.system.pid_by_port function.
     """
-
     runner = test.runner.get_runner()
     if stem.util.system.is_windows():
       test.runner.skip(self, '(unavailable on windows)')
@@ -241,8 +260,8 @@ class TestSystem(unittest.TestCase):
       return
 
     tor_pid, tor_port = runner.get_pid(), test.runner.CONTROL_PORT
-    self.assertEquals(tor_pid, stem.util.system.pid_by_port(tor_port))
-    self.assertEquals(None, stem.util.system.pid_by_port(99999))
+    self.assertEqual(tor_pid, stem.util.system.pid_by_port(tor_port))
+    self.assertEqual(None, stem.util.system.pid_by_port(99999))
 
   def test_pid_by_port_netstat(self):
     """
@@ -274,7 +293,7 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       tor_pid = test.runner.get_runner().get_pid()
-      self.assertEquals(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))
+      self.assertEqual(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))
 
   def test_pid_by_port_sockstat(self):
     """
@@ -303,7 +322,7 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       tor_pid = test.runner.get_runner().get_pid()
-      self.assertEquals(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))
+      self.assertEqual(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))
 
   def test_pid_by_port_lsof(self):
     """
@@ -332,7 +351,7 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       tor_pid = test.runner.get_runner().get_pid()
-      self.assertEquals(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))
+      self.assertEqual(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))
 
   def test_pid_by_open_file(self):
     """
@@ -341,11 +360,11 @@ class TestSystem(unittest.TestCase):
 
     # check a directory that exists, but isn't claimed by any application
     tmpdir = tempfile.mkdtemp()
-    self.assertEquals(None, stem.util.system.pid_by_open_file(tmpdir))
+    self.assertEqual(None, stem.util.system.pid_by_open_file(tmpdir))
 
     # check a directory that doesn't exist
     os.rmdir(tmpdir)
-    self.assertEquals(None, stem.util.system.pid_by_open_file(tmpdir))
+    self.assertEqual(None, stem.util.system.pid_by_open_file(tmpdir))
 
   def test_cwd(self):
     """
@@ -362,8 +381,8 @@ class TestSystem(unittest.TestCase):
       return
 
     runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
-    self.assertEquals(tor_cwd, stem.util.system.cwd(runner_pid))
-    self.assertEquals(None, stem.util.system.cwd(99999))
+    self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))
+    self.assertEqual(None, stem.util.system.cwd(99999))
 
   def test_cwd_pwdx(self):
     """
@@ -389,7 +408,7 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
-      self.assertEquals(tor_cwd, stem.util.system.cwd(runner_pid))
+      self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))
 
   def test_cwd_lsof(self):
     """
@@ -415,7 +434,7 @@ class TestSystem(unittest.TestCase):
       call_mock.side_effect = call_replacement
 
       runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
-      self.assertEquals(tor_cwd, stem.util.system.cwd(runner_pid))
+      self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))
 
   def test_user_none(self):
     """
@@ -504,7 +523,7 @@ class TestSystem(unittest.TestCase):
     properly).
     """
 
-    self.assertEquals(0, stem.util.system.bsd_jail_id(99999))
+    self.assertEqual(0, stem.util.system.bsd_jail_id(99999))
 
   def test_expand_path(self):
     """
@@ -524,15 +543,15 @@ class TestSystem(unittest.TestCase):
       test.runner.skip(self, '(running as root)')
       return
 
-    self.assertEquals(os.getcwd(), stem.util.system.expand_path('.'))
-    self.assertEquals(os.getcwd(), stem.util.system.expand_path('./'))
-    self.assertEquals(os.path.join(os.getcwd(), 'foo'), stem.util.system.expand_path('./foo'))
+    self.assertEqual(os.getcwd(), stem.util.system.expand_path('.'))
+    self.assertEqual(os.getcwd(), stem.util.system.expand_path('./'))
+    self.assertEqual(os.path.join(os.getcwd(), 'foo'), stem.util.system.expand_path('./foo'))
 
     home_dir, username = os.path.expanduser('~'), getpass.getuser()
-    self.assertEquals(home_dir, stem.util.system.expand_path('~'))
-    self.assertEquals(home_dir, stem.util.system.expand_path('~/'))
-    self.assertEquals(home_dir, stem.util.system.expand_path('~%s' % username))
-    self.assertEquals(os.path.join(home_dir, 'foo'), stem.util.system.expand_path('~%s/foo' % username))
+    self.assertEqual(home_dir, stem.util.system.expand_path('~'))
+    self.assertEqual(home_dir, stem.util.system.expand_path('~/'))
+    self.assertEqual(home_dir, stem.util.system.expand_path('~%s' % username))
+    self.assertEqual(os.path.join(home_dir, 'foo'), stem.util.system.expand_path('~%s/foo' % username))
 
   def test_set_process_name(self):
     """
@@ -554,18 +573,14 @@ class TestSystem(unittest.TestCase):
     # isn't the end of the world. It's just used to skip tests if they should
     # legitemately fail.
 
-    global IS_EXTRA_TOR_RUNNING
-
-    if IS_EXTRA_TOR_RUNNING is None:
-      if stem.util.system.is_windows():
-        # TODO: not sure how to check for this on windows
-        IS_EXTRA_TOR_RUNNING = False
-      elif not stem.util.system.is_bsd():
-        pgrep_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PGREP % 'tor')
-        IS_EXTRA_TOR_RUNNING = len(pgrep_results) > 1
-      else:
-        ps_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PS_BSD)
-        results = [r for r in ps_results if r.endswith(' tor')]
-        IS_EXTRA_TOR_RUNNING = len(results) > 1
-
-    return IS_EXTRA_TOR_RUNNING
+    if stem.util.system.is_windows():
+      # TODO: not sure how to check for this on windows
+      return False
+    elif not stem.util.system.is_bsd():
+      tor_cmd = test.runner.get_runner().get_tor_command(True)
+      pgrep_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PGREP % tor_cmd)
+      return len(pgrep_results) > 1
+    else:
+      ps_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PS_BSD)
+      results = [r for r in ps_results if r.endswith(' tor')]
+      return len(results) > 1
