@@ -9,6 +9,8 @@ import stem.prereq
 import stem.version
 import test.runner
 
+from test.runner import require_controller
+
 
 class TestVersion(unittest.TestCase):
   def test_get_system_tor_version(self):
@@ -32,28 +34,24 @@ class TestVersion(unittest.TestCase):
     # try running against a command that doesn't exist
     self.assertRaises(IOError, stem.version.get_system_tor_version, 'blarg')
 
+  @require_controller
   def test_get_system_tor_version_value(self):
     """
     Checks that the get_system_tor_version() provides the same value as our
     test instance provides.
     """
 
-    if test.runner.require_control(self):
-      return
-
     runner = test.runner.get_runner()
     system_tor_version = stem.version.get_system_tor_version(runner.get_tor_command())
-    self.assertEquals(runner.get_tor_version(), system_tor_version)
+    self.assertEqual(runner.get_tor_version(), system_tor_version)
 
+  @require_controller
   def test_getinfo_version_parsing(self):
     """
     Issues a 'GETINFO version' query to our test instance and makes sure that
     we can parse it.
     """
 
-    if test.runner.require_control(self):
-      return
-
     control_socket = test.runner.get_runner().get_tor_socket()
     control_socket.send('GETINFO version')
     version_response = control_socket.recv()
