@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -20,6 +20,7 @@ HEADER_ATTR = (term.Color.CYAN, term.Attr.BOLD)
 CATEGORY_ATTR = (term.Color.GREEN, term.Attr.BOLD)
 
 NO_NL = 'no newline'
+STDERR = 'stderr'
 
 # formatting for various categories of messages
 
@@ -46,23 +47,32 @@ LINE_ATTR = {
   LineType.CONTENT: (term.Color.CYAN,),
 }
 
+SUPPRESS_STDOUT = False  # prevent anything from being printed to stdout
+
 
 def println(msg = '', *attr):
+  if SUPPRESS_STDOUT and STDERR not in attr:
+    return
+
   attr = _flatten(attr)
   no_newline = False
+  stream = sys.stderr if STDERR in attr else sys.stdout
 
   if NO_NL in attr:
     no_newline = True
     attr.remove(NO_NL)
 
-  if COLOR_SUPPORT:
+  if STDERR in attr:
+    attr.remove(STDERR)
+
+  if COLOR_SUPPORT and attr:
     msg = term.format(msg, *attr)
 
-  if no_newline:
-    sys.stdout.write(msg)
-    sys.stdout.flush()
-  else:
-    print msg
+  if not no_newline:
+    msg += '\n'
+
+  stream.write(msg)
+  stream.flush()
 
 
 def print_divider(msg, is_header = False):
@@ -71,11 +81,14 @@ def print_divider(msg, is_header = False):
 
 
 def print_logging(logging_buffer):
+  if SUPPRESS_STDOUT:
+    return
+
   if not logging_buffer.is_empty():
     for entry in logging_buffer:
       println(entry.replace('\n', '\n  '), term.Color.MAGENTA)
 
-    print
+    print()
 
 
 def apply_filters(testing_output, *filters):
@@ -183,6 +196,7 @@ class ErrorTracker(object):
 
   def __init__(self):
     self._errors = []
+    self._error_modules = set()
     self._category = None
     self._error_noted = False
 
@@ -219,10 +233,18 @@ class ErrorTracker(object):
         else:
           self._errors.append(line_content)
 
+        module_match = re.match('.*\((test\.\S+)\.\S+\).*', line_content)
+
+        if module_match:
+          self._error_modules.add(module_match.group(1))
+
       return line_content
 
     return _error_tracker
 
+  def get_modules(self):
+    return self._error_modules
+
   def __iter__(self):
     for error_line in self._errors:
       yield error_line
