@@ -1,4 +1,4 @@
-# Copyright 2011-2014, Damian Johnson and The Tor Project
+# Copyright 2011-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -12,7 +12,7 @@ about the tor test instance they're running against.
   TorInaccessable - Tor can't be queried for the information
 
   skip - skips the current test if we can
-  require_control - skips the test unless tor provides a controller endpoint
+  require_controller - skips the test unless tor provides a controller endpoint
   require_version - skips the test unless we meet a tor version requirement
   require_online - skips unless targets allow for online tests
   only_run_once - skip the test if it has been ran before
@@ -117,67 +117,69 @@ def skip(test_case, message):
     test_case.skipTest(message)
 
 
-def require_control(test_case):
+def require_controller(func):
   """
   Skips the test unless tor provides an endpoint for controllers to attach to.
-
-  :param unittest.TestCase test_case: test being ran
-
-  :returns: True if test should be skipped, False otherwise
   """
 
-  if not get_runner().is_accessible():
-    skip(test_case, '(no connection)')
-    return True
+  def wrapped(self, *args, **kwargs):
+    if get_runner().is_accessible():
+      return func(self, *args, **kwargs)
+    else:
+      skip(self, '(no connection)')
+
+  return wrapped
 
 
-def require_version(test_case, req_version):
+def require_version(req_version):
   """
   Skips the test unless we meet the required version.
 
-  :param unittest.TestCase test_case: test being ran
   :param stem.version.Version req_version: required tor version for the test
-
-  :returns: True if test should be skipped, False otherwise
   """
 
-  if get_runner().get_tor_version() < req_version:
-    skip(test_case, '(requires %s)' % req_version)
-    return True
+  def decorator(func):
+    def wrapped(self, *args, **kwargs):
+      if get_runner().get_tor_version() >= req_version:
+        return func(self, *args, **kwargs)
+      else:
+        skip(self, '(requires %s)' % req_version)
+
+    return wrapped
+
+  return decorator
 
 
-def require_online(test_case):
+def require_online(func):
   """
   Skips the test if we weren't started with the ONLINE target, which indicates
   that tests requiring network connectivity should run.
-
-  :param unittest.TestCase test_case: test being ran
-
-  :returns: True if test should be skipped, False otherwise
   """
 
-  if Target.ONLINE not in get_runner().attribute_targets:
-    skip(test_case, '(requires online target)')
-    return True
+  def wrapped(self, *args, **kwargs):
+    if Target.ONLINE in get_runner().attribute_targets:
+      return func(self, *args, **kwargs)
+    else:
+      skip(self, '(requires online target)')
+
+  return wrapped
 
 
-def only_run_once(test_case, test_name):
+def only_run_once(func):
   """
   Skips the test if it has ran before. If it hasn't then flags it as being ran.
   This is useful to prevent lengthy tests that are independent of integ targets
   from being run repeatedly with ``RUN_ALL``.
-
-  :param unittest.TestCase test_case: test being ran
-  :param str test_name: name of the test being ran
-
-  :returns: True if test should be skipped, False otherwise
   """
 
-  if (test_case, test_name) in RAN_TESTS:
-    skip(test_case, '(already ran)')
-    return True
-  else:
-    RAN_TESTS.append((test_case, test_name))
+  def wrapped(self, *args, **kwargs):
+    if (self, self.id()) not in RAN_TESTS:
+      RAN_TESTS.append((self, self.id()))
+      return func(self, *args, **kwargs)
+    else:
+      skip(self, '(already ran)')
+
+  return wrapped
 
 
 def exercise_controller(test_case, controller):
@@ -199,7 +201,7 @@ def exercise_controller(test_case, controller):
   else:
     config_file_response = controller.msg('GETINFO config-file')
 
-  test_case.assertEquals('config-file=%s\nOK' % torrc_path, str(config_file_response))
+  test_case.assertEqual('config-file=%s\nOK' % torrc_path, str(config_file_response))
 
 
 def get_runner():
@@ -584,12 +586,15 @@ class Runner(object):
     except TorInaccessable:
       return stem.version.get_system_tor_version(self.get_tor_command())
 
-  def get_tor_command(self):
+  def get_tor_command(self, base_cmd = False):
     """
     Provides the command used to run our tor instance.
+
+    :param bool base_cmd: provides just the command name if true rather than
+      the full '--tor path' argument
     """
 
-    return self._get('_tor_cmd')
+    return os.path.basename(self._get('_tor_cmd')) if base_cmd else self._get('_tor_cmd')
 
   def _get(self, attr):
     """
@@ -638,13 +643,13 @@ class Runner(object):
         socket_dir = os.path.dirname(CONTROL_SOCKET_PATH)
         println('  making control socket directory (%s)... ' % socket_dir, STATUS, NO_NL)
 
-        if os.path.exists(socket_dir) and stat.S_IMODE(os.stat(socket_dir).st_mode) == 0700:
+        if os.path.exists(socket_dir) and stat.S_IMODE(os.stat(socket_dir).st_mode) == 0o700:
           println('skipped', STATUS)
         else:
           if not os.path.exists(socket_dir):
             os.makedirs(socket_dir)
 
-          os.chmod(socket_dir, 0700)
+          os.chmod(socket_dir, 0o700)
           println('done', STATUS)
       except OSError as exc:
         println('failed (%s)' % exc, ERROR)
@@ -702,7 +707,7 @@ class Runner(object):
     :raises: OSError if we either fail to create the tor process or reached a timeout without success
     """
 
-    println('Starting tor...\n', STATUS)
+    println('Starting %s...\n' % tor_cmd, STATUS)
     start_time = time.time()
 
     try:
