@@ -3,6 +3,7 @@ Unit tests for the stem.control module. The module's primarily exercised via
 integ tests, but a few bits lend themselves to unit testing.
 """
 
+import datetime
 import io
 import unittest
 
@@ -12,7 +13,7 @@ import stem.socket
 import stem.util.system
 import stem.version
 
-from stem import InvalidArguments, InvalidRequest, ProtocolError, UnsatisfiableRequest
+from stem import ControllerError, DescriptorUnavailable, InvalidArguments, InvalidRequest, ProtocolError, UnsatisfiableRequest
 from stem.control import _parse_circ_path, Listener, Controller, EventType
 from stem.exit_policy import ExitPolicy
 from test import mocking
@@ -23,6 +24,8 @@ try:
 except ImportError:
   from mock import Mock, patch
 
+NS_DESC = 'r %s %s u5lTXJKGsLKufRLnSyVqT7TdGYw 2012-12-30 22:02:49 77.223.43.54 9001 0\ns Fast Named Running Stable Valid\nw Bandwidth=75'
+
 
 class TestControl(unittest.TestCase):
   def setUp(self):
@@ -277,6 +280,36 @@ class TestControl(unittest.TestCase):
       get_info_mock.return_value = response
       self.assertRaises(stem.ProtocolError, self.controller.get_socks_listeners)
 
+  @patch('stem.control.Controller.get_info')
+  @patch('time.time', Mock(return_value = 1410723598.276578))
+  def test_get_accounting_stats(self, get_info_mock):
+    """
+    Exercises the get_accounting_stats() method.
+    """
+
+    get_info_mock.side_effect = lambda param, **kwargs: {
+      'accounting/enabled': '1',
+      'accounting/hibernating': 'awake',
+      'accounting/interval-end': '2014-09-14 19:41:00',
+      'accounting/bytes': '4837 2050',
+      'accounting/bytes-left': '102944 7440',
+    }[param]
+
+    expected = stem.control.AccountingStats(
+      1410723598.276578,
+      'awake',
+      datetime.datetime(2014, 9, 14, 19, 41),
+      62,
+      4837, 102944, 107781,
+      2050, 7440, 9490,
+    )
+
+    self.assertEqual(expected, self.controller.get_accounting_stats())
+
+    get_info_mock.side_effect = ControllerError('nope, too bad')
+    self.assertRaises(ControllerError, self.controller.get_accounting_stats)
+    self.assertEqual('my default', self.controller.get_accounting_stats('my default'))
+
   @patch('stem.connection.get_protocolinfo')
   def test_get_protocolinfo(self, get_protocolinfo_mock):
     """
@@ -329,8 +362,8 @@ class TestControl(unittest.TestCase):
     self.assertEqual('atagar', self.controller.get_user())
 
   @patch('stem.socket.ControlSocket.is_localhost', Mock(return_value = True))
-  @patch('stem.util.system.get_pid_by_name', Mock(return_value = 432))
-  @patch('stem.util.system.get_user', Mock(return_value = 'atagar'))
+  @patch('stem.util.system.pid_by_name', Mock(return_value = 432))
+  @patch('stem.util.system.user', Mock(return_value = 'atagar'))
   def test_get_user_by_system(self):
     """
     Exercise the get_user() resolution via the system module.
@@ -371,7 +404,7 @@ class TestControl(unittest.TestCase):
     open_mock.assert_called_once_with('/tmp/pid_file')
 
   @patch('stem.socket.ControlSocket.is_localhost', Mock(return_value = True))
-  @patch('stem.util.system.get_pid_by_name', Mock(return_value = 432))
+  @patch('stem.util.system.pid_by_name', Mock(return_value = 432))
   def test_get_pid_by_name(self):
     """
     Exercise the get_pid() resolution via the process name.
@@ -379,6 +412,49 @@ class TestControl(unittest.TestCase):
 
     self.assertEqual(432, self.controller.get_pid())
 
+  @patch('stem.control.Controller.get_info')
+  def test_get_network_status_for_ourselves(self, get_info_mock):
+    """
+    Exercises the get_network_status() method for getting our own relay.
+    """
+
+    # when there's an issue getting our fingerprint
+
+    get_info_mock.side_effect = ControllerError('nope, too bad')
+
+    try:
+      self.controller.get_network_status()
+      self.fail("We should've raised an exception")
+    except ControllerError as exc:
+      self.assertEqual('Unable to determine our own fingerprint: nope, too bad', str(exc))
+
+    self.assertEqual('boom', self.controller.get_network_status(default = 'boom'))
+
+    # successful request
+
+    desc = NS_DESC % ('moria1', '/96bKo4soysolMgKn5Hex2nyFSY')
+
+    get_info_mock.side_effect = lambda param, **kwargs: {
+      'fingerprint': '9695DFC35FFEB861329B9F1AB04C46397020CE31',
+      'ns/id/9695DFC35FFEB861329B9F1AB04C46397020CE31': desc,
+    }[param]
+
+    self.assertEqual(stem.descriptor.router_status_entry.RouterStatusEntryV3(desc), self.controller.get_network_status())
+
+  @patch('stem.control.Controller.get_info')
+  def test_get_network_status_when_raising_descriptor_unavailable(self, get_info_mock):
+    """
+    Exercises the get_network_status() method.
+    """
+
+    get_info_mock.side_effect = InvalidArguments(None, 'GETINFO request contained unrecognized keywords: ns/id/5AC9C5AA75BA1F18D8459B326B4B8111A856D290')
+
+    try:
+      self.controller.get_network_status('5AC9C5AA75BA1F18D8459B326B4B8111A856D290')
+      self.fail("We should've raised an exception")
+    except DescriptorUnavailable as exc:
+      self.assertEqual("Tor was unable to provide the descriptor for '5AC9C5AA75BA1F18D8459B326B4B8111A856D290'", str(exc))
+
   @patch('stem.control.Controller.get_info')
   def test_get_network_status(self, get_info_mock):
     """
@@ -389,7 +465,7 @@ class TestControl(unittest.TestCase):
 
     nickname = 'Beaver'
     fingerprint = '/96bKo4soysolMgKn5Hex2nyFSY'
-    desc = 'r %s %s u5lTXJKGsLKufRLnSyVqT7TdGYw 2012-12-30 22:02:49 77.223.43.54 9001 0\ns Fast Named Running Stable Valid\nw Bandwidth=75' % (nickname, fingerprint)
+    desc = NS_DESC % (nickname, fingerprint)
     router = stem.descriptor.router_status_entry.RouterStatusEntryV3(desc)
 
     # always return the same router status entry
@@ -526,6 +602,29 @@ class TestControl(unittest.TestCase):
     for test_input in malformed_inputs:
       self.assertRaises(ProtocolError, _parse_circ_path, test_input)
 
+  @patch('stem.control.Controller.get_conf')
+  def test_get_effective_rate(self, get_conf_mock):
+    """
+    Exercise the get_effective_rate() method.
+    """
+
+    # check default if nothing was set
+
+    get_conf_mock.side_effect = lambda param, **kwargs: {
+      'BandwidthRate': '1073741824',
+      'BandwidthBurst': '1073741824',
+      'RelayBandwidthRate': '0',
+      'RelayBandwidthBurst': '0',
+      'MaxAdvertisedBandwidth': '1073741824',
+    }[param]
+
+    self.assertEqual(1073741824, self.controller.get_effective_rate())
+    self.assertEqual(1073741824, self.controller.get_effective_rate(burst = True))
+
+    get_conf_mock.side_effect = ControllerError('nope, too bad')
+    self.assertRaises(ControllerError, self.controller.get_effective_rate)
+    self.assertEqual('my_default', self.controller.get_effective_rate('my_default'))
+
   @patch('stem.control.Controller.get_version')
   def test_drop_guards(self, get_version_mock):
     """
