@@ -7,10 +7,12 @@ import unittest
 
 from stem.descriptor.networkstatus import KeyCertificate
 
-from test.mocking import get_key_certificate, \
-                         CRYPTO_BLOB, \
-                         KEY_CERTIFICATE_HEADER, \
-                         KEY_CERTIFICATE_FOOTER
+from test.mocking import (
+  get_key_certificate,
+  CRYPTO_BLOB,
+  KEY_CERTIFICATE_HEADER,
+  KEY_CERTIFICATE_FOOTER,
+)
 
 
 class TestKeyCertificate(unittest.TestCase):
@@ -39,7 +41,7 @@ class TestKeyCertificate(unittest.TestCase):
     """
 
     certificate = get_key_certificate({'pepperjack': 'is oh so tasty!'})
-    self.assertEquals(['pepperjack is oh so tasty!'], certificate.get_unrecognized_lines())
+    self.assertEqual(['pepperjack is oh so tasty!'], certificate.get_unrecognized_lines())
 
   def test_first_and_last_lines(self):
     """
@@ -51,7 +53,7 @@ class TestKeyCertificate(unittest.TestCase):
 
     for cert_text in (b'dir-address 127.0.0.1:80\n' + content,
                       content + b'\ndir-address 127.0.0.1:80'):
-      self.assertRaises(ValueError, KeyCertificate, cert_text)
+      self.assertRaises(ValueError, KeyCertificate, cert_text, True)
 
       certificate = KeyCertificate(cert_text, False)
       self.assertEqual('127.0.0.1', certificate.address)
@@ -66,7 +68,7 @@ class TestKeyCertificate(unittest.TestCase):
 
     for excluded_field in mandatory_fields:
       content = get_key_certificate(exclude = (excluded_field,), content = True)
-      self.assertRaises(ValueError, KeyCertificate, content)
+      self.assertRaises(ValueError, KeyCertificate, content, True)
 
       certificate = KeyCertificate(content, False)
 
@@ -90,15 +92,15 @@ class TestKeyCertificate(unittest.TestCase):
     """
 
     certificate = get_key_certificate({'dir-key-certificate-version': '3'})
-    self.assertEquals(3, certificate.version)
+    self.assertEqual(3, certificate.version)
 
     content = get_key_certificate({'dir-key-certificate-version': '4'}, content = True)
-    self.assertRaises(ValueError, KeyCertificate, content)
-    self.assertEquals(4, KeyCertificate(content, False).version)
+    self.assertRaises(ValueError, KeyCertificate, content, True)
+    self.assertEqual(4, KeyCertificate(content, False).version)
 
     content = get_key_certificate({'dir-key-certificate-version': 'boo'}, content = True)
-    self.assertRaises(ValueError, KeyCertificate, content)
-    self.assertEquals(None, KeyCertificate(content, False).version)
+    self.assertRaises(ValueError, KeyCertificate, content, True)
+    self.assertEqual(None, KeyCertificate(content, False).version)
 
   def test_dir_address(self):
     """
@@ -110,23 +112,23 @@ class TestKeyCertificate(unittest.TestCase):
     self.assertEqual(80, certificate.dir_port)
 
     test_values = (
-      ('', None, None),
-      ('   ', None, None),
-      ('127.0.0.1', None, None),
-      ('127.0.0.1:', None, None),
-      ('80', None, None),
-      (':80', '', 80),
-      ('127.0.0.1a:80', '127.0.0.1a', 80),
-      ('127.0.0.1:80a', None, None),
+      (''),
+      ('   '),
+      ('127.0.0.1'),
+      ('127.0.0.1:'),
+      ('80'),
+      (':80'),
+      ('127.0.0.1a:80'),
+      ('127.0.0.1:80a'),
     )
 
-    for test_value, expected_address, expected_port in test_values:
+    for test_value in test_values:
       content = get_key_certificate({'dir-address': test_value}, content = True)
-      self.assertRaises(ValueError, KeyCertificate, content)
+      self.assertRaises(ValueError, KeyCertificate, content, True)
 
       certificate = KeyCertificate(content, False)
-      self.assertEqual(expected_address, certificate.address)
-      self.assertEqual(expected_port, certificate.dir_port)
+      self.assertEqual(None, certificate.address)
+      self.assertEqual(None, certificate.dir_port)
 
   def test_fingerprint(self):
     """
@@ -142,10 +144,10 @@ class TestKeyCertificate(unittest.TestCase):
 
     for test_value in test_values:
       content = get_key_certificate({'fingerprint': test_value}, content = True)
-      self.assertRaises(ValueError, KeyCertificate, content)
+      self.assertRaises(ValueError, KeyCertificate, content, True)
 
       certificate = KeyCertificate(content, False)
-      self.assertEqual(test_value.strip(), certificate.fingerprint)
+      self.assertEqual(None, certificate.fingerprint)
 
   def test_time_fields(self):
     """
@@ -164,10 +166,10 @@ class TestKeyCertificate(unittest.TestCase):
     for field, attr in (('dir-key-published', 'published'), ('dir-key-expires', 'expires')):
       for test_value in test_values:
         content = get_key_certificate({field: test_value}, content = True)
-        self.assertRaises(ValueError, KeyCertificate, content)
+        self.assertRaises(ValueError, KeyCertificate, content, True)
 
         certificate = KeyCertificate(content, False)
-        self.assertEquals(None, getattr(certificate, attr))
+        self.assertEqual(None, getattr(certificate, attr))
 
   def test_key_blocks(self):
     """
@@ -183,11 +185,19 @@ class TestKeyCertificate(unittest.TestCase):
     test_value = '\n-----BEGIN ID SIGNATURE-----%s-----END UGABUGA SIGNATURE-----' % CRYPTO_BLOB
 
     for field, attr in (('dir-identity-key', 'identity_key'),
-                       ('dir-signing-key', 'signing_key'),
-                       ('dir-key-crosscert', 'crosscert'),
-                       ('dir-key-certification', 'certification')):
+                        ('dir-signing-key', 'signing_key'),
+                        ('dir-key-crosscert', 'crosscert'),
+                        ('dir-key-certification', 'certification')):
       content = get_key_certificate({field: test_value}, content = True)
-      self.assertRaises(ValueError, KeyCertificate, content)
+      self.assertRaises(ValueError, KeyCertificate, content, True)
 
       certificate = KeyCertificate(content, False)
-      self.assertEquals(None, getattr(certificate, attr))
+      self.assertEqual(None, getattr(certificate, attr))
+
+  def test_wrong_block_type(self):
+    """
+    Checks that we validate the type of crypto content we receive.
+    """
+
+    content = get_key_certificate({'dir-identity-key': '\n-----BEGIN MD5SUM-----%s-----END MD5SUM-----' % CRYPTO_BLOB}, content = True)
+    self.assertRaises(ValueError, KeyCertificate, content, True)
