@@ -40,7 +40,6 @@ class TestRouterStatusEntry(unittest.TestCase):
     # checks with some malformed inputs
     for arg in ('', '20wYcb', '20wYcb' * 30):
       self.assertRaises(ValueError, _base64_to_hex, arg, True)
-      self.assertEqual(None, _base64_to_hex(arg, False))
 
   def test_minimal_v2(self):
     """
@@ -130,7 +129,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     """
 
     entry = get_router_status_entry_v3({'z': 'New tor feature: sparkly unicorns!'})
-    self.assertEquals(['z New tor feature: sparkly unicorns!'], entry.get_unrecognized_lines())
+    self.assertEqual(['z New tor feature: sparkly unicorns!'], entry.get_unrecognized_lines())
 
   def test_proceeding_line(self):
     """
@@ -138,7 +137,8 @@ class TestRouterStatusEntry(unittest.TestCase):
     """
 
     content = b'z some stuff\n' + get_router_status_entry_v3(content = True)
-    self._expect_invalid_attr(content, '_unrecognized_lines', ['z some stuff'])
+    self.assertRaises(ValueError, RouterStatusEntryV3, content, True)
+    self.assertEqual(['z some stuff'], RouterStatusEntryV3(content, False).get_unrecognized_lines())
 
   def test_blank_lines(self):
     """
@@ -158,7 +158,7 @@ class TestRouterStatusEntry(unittest.TestCase):
 
     for index, duplicate_line in enumerate(lines):
       content = b'\n'.join(lines[:index] + [duplicate_line] + lines[index:])
-      self.assertRaises(ValueError, RouterStatusEntryV3, content)
+      self.assertRaises(ValueError, RouterStatusEntryV3, content, True)
 
       entry = RouterStatusEntryV3(content, False)
       self.assertEqual('caerSidi', entry.nickname)
@@ -215,7 +215,7 @@ class TestRouterStatusEntry(unittest.TestCase):
       if value == '':
         value = None
 
-      self._expect_invalid_attr(content, 'nickname', value)
+      self._expect_invalid_attr(content, 'nickname')
 
   def test_malformed_fingerprint(self):
     """
@@ -275,7 +275,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     for value in test_values:
       r_line = ROUTER_STATUS_ENTRY_V3_HEADER[0][1].replace('71.35.150.29', value)
       content = get_router_status_entry_v3({'r': r_line}, content = True)
-      self._expect_invalid_attr(content, 'address', value)
+      self._expect_invalid_attr(content, 'address')
 
   def test_malformed_port(self):
     """
@@ -304,10 +304,9 @@ class TestRouterStatusEntry(unittest.TestCase):
             r_line = r_line[:-1] + value
 
           attr = 'or_port' if include_or_port else 'dir_port'
-          expected = int(value) if value.isdigit() else None
 
           content = get_router_status_entry_v3({'r': r_line}, content = True)
-          self._expect_invalid_attr(content, attr, expected)
+          self._expect_invalid_attr(content, attr)
 
   def test_ipv6_addresses(self):
     """
@@ -321,7 +320,7 @@ class TestRouterStatusEntry(unittest.TestCase):
 
     for a_line, expected in test_values.items():
       entry = get_router_status_entry_v3({'a': a_line})
-      self.assertEquals(expected, entry.or_addresses)
+      self.assertEqual(expected, entry.or_addresses)
 
     # includes multiple 'a' lines
 
@@ -335,7 +334,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     ]
 
     entry = RouterStatusEntryV3(content)
-    self.assertEquals(expected, entry.or_addresses)
+    self.assertEqual(expected, entry.or_addresses)
 
     # tries some invalid inputs
 
@@ -362,7 +361,7 @@ class TestRouterStatusEntry(unittest.TestCase):
 
     for s_line, expected in test_values.items():
       entry = get_router_status_entry_v3({'s': s_line})
-      self.assertEquals(expected, entry.flags)
+      self.assertEqual(expected, entry.flags)
 
     # tries some invalid inputs
     test_values = {
@@ -389,8 +388,8 @@ class TestRouterStatusEntry(unittest.TestCase):
 
     for v_line, expected in test_values.items():
       entry = get_router_status_entry_v3({'v': v_line})
-      self.assertEquals(expected, entry.version)
-      self.assertEquals(v_line, entry.version_line)
+      self.assertEqual(expected, entry.version)
+      self.assertEqual(v_line, entry.version_line)
 
     # tries an invalid input
     content = get_router_status_entry_v3({'v': 'Tor ugabuga'}, content = True)
@@ -411,10 +410,10 @@ class TestRouterStatusEntry(unittest.TestCase):
 
     for w_line, expected in test_values.items():
       entry = get_router_status_entry_v3({'w': w_line})
-      self.assertEquals(expected[0], entry.bandwidth)
-      self.assertEquals(expected[1], entry.measured)
-      self.assertEquals(expected[2], entry.is_unmeasured)
-      self.assertEquals(expected[3], entry.unrecognized_bandwidth_entries)
+      self.assertEqual(expected[0], entry.bandwidth)
+      self.assertEqual(expected[1], entry.measured)
+      self.assertEqual(expected[2], entry.is_unmeasured)
+      self.assertEqual(expected[3], entry.unrecognized_bandwidth_entries)
 
     # tries some invalid inputs
     test_values = (
@@ -451,7 +450,7 @@ class TestRouterStatusEntry(unittest.TestCase):
 
     for p_line, expected in test_values.items():
       entry = get_router_status_entry_v3({'p': p_line})
-      self.assertEquals(expected, entry.exit_policy)
+      self.assertEqual(expected, entry.exit_policy)
 
     # tries some invalid inputs
     test_values = (
@@ -487,7 +486,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     for m_line, expected in test_values.items():
       content = get_router_status_entry_v3({'m': m_line}, content = True)
       entry = RouterStatusEntryV3(content, document = mock_document)
-      self.assertEquals(expected, entry.microdescriptor_hashes)
+      self.assertEqual(expected, entry.microdescriptor_hashes)
 
     # try with multiple 'm' lines
 
@@ -501,7 +500,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     ]
 
     entry = RouterStatusEntryV3(content, document = mock_document)
-    self.assertEquals(expected, entry.microdescriptor_hashes)
+    self.assertEqual(expected, entry.microdescriptor_hashes)
 
     # try without a document
     content = get_router_status_entry_v3({'m': '8,9,10,11,12'}, content = True)
@@ -525,10 +524,10 @@ class TestRouterStatusEntry(unittest.TestCase):
     value when we're constructed without validation.
     """
 
-    self.assertRaises(ValueError, RouterStatusEntryV3, content)
+    self.assertRaises(ValueError, RouterStatusEntryV3, content, True)
     entry = RouterStatusEntryV3(content, False)
 
     if attr:
-      self.assertEquals(expected_value, getattr(entry, attr))
+      self.assertEqual(expected_value, getattr(entry, attr))
     else:
-      self.assertEquals('caerSidi', entry.nickname)
+      self.assertEqual('caerSidi', entry.nickname)
