@@ -3,14 +3,27 @@ Tests for the examples given in stem's tutorial.
 """
 
 import itertools
-import StringIO
 import unittest
 
+try:
+  # added in python 2.7
+  from collections import OrderedDict
+except ImportError:
+  from stem.util.ordereddict import OrderedDict
+
+try:
+  from StringIO import StringIO
+except ImportError:
+  from io import StringIO
+
 import stem.response
 import stem.descriptor.remote
+import stem.prereq
 
+from stem import str_type
 from stem.control import Controller
 from stem.descriptor.remote import DIRECTORY_AUTHORITIES
+
 from test import mocking
 from test.mocking import (
   get_relay_server_descriptor,
@@ -31,7 +44,7 @@ PURPOSE=%s'
 
 PATH_CONTENT = '$%s=%s,$%s=%s,$%s=%s'
 
-LIST_CIRCUITS_OUTPUT = """\
+LIST_CIRCUITS_OUTPUT = str_type("""\
 
 Circuit 4 (GENERAL)
  |- B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C (ByTORAndTheSnowDog, 173.209.180.61)
@@ -47,9 +60,9 @@ Circuit 10 (GENERAL)
  |- B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C (ByTORAndTheSnowDog, 173.209.180.61)
  |- 00C2C2A16AEDB51D5E5FB7D6168FC66B343D822F (ph3x, 86.59.119.83)
  +- 65242C91BFF30F165DA4D132C81A9EBA94B71D62 (torexit16, 176.67.169.171)
-"""
+""")
 
-EXIT_USED_OUTPUT = """\
+EXIT_USED_OUTPUT = str_type("""\
 Tracking requests for tor exits. Press 'enter' to end.
 
 Exit relay for our connection to 64.15.112.44:80
@@ -58,15 +71,15 @@ Exit relay for our connection to 64.15.112.44:80
   nickname: chaoscomputerclub19
   locale: unknown
 
-"""
+""")
 
-OUTDATED_RELAYS_OUTPUT = """\
+OUTDATED_RELAYS_OUTPUT = str_type("""\
 Checking for outdated relays...
 
   0.1.0           Sambuddha Basu
 
 2 outdated relays found, 1 had contact information
-"""
+""")
 
 COMPARE_FLAGS_OUTPUT = """\
 maatuska has the Running flag but moria1 doesn't: E2BB13AA2F6960CD93ABE5257A825687F3973C62
@@ -77,7 +90,7 @@ moria1 has the Running flag but maatuska doesn't: DCAEC3D069DC39AAE43D13C8AF31B5
 """
 
 VOTES_BY_BANDWIDTH_AUTHORITIES_OUTPUT = """\
-Getting gabelmoo's vote from http://212.112.245.170:80/tor/status-vote/current/authority:
+Getting gabelmoo's vote from http://131.188.40.189:80/tor/status-vote/current/authority:
   5935 measured entries and 1332 unmeasured
 Getting tor26's vote from http://86.59.21.38:80/tor/status-vote/current/authority:
   5735 measured entries and 1690 unmeasured
@@ -126,7 +139,13 @@ def _get_router_status(address = None, port = None, nickname = None, fingerprint
 
 
 class TestTutorialExamples(unittest.TestCase):
-  @patch('sys.stdout', new_callable = StringIO.StringIO)
+  def assert_equal_unordered(self, expected, actual):
+    if stem.prereq.is_python_3():
+      self.assertCountEqual(expected.splitlines(), actual.splitlines())
+    else:
+      self.assertEquals(sorted(expected.splitlines()), sorted(actual.splitlines()))
+
+  @patch('sys.stdout', new_callable = StringIO)
   @patch('stem.control.Controller.from_port', spec = Controller)
   def test_list_circuits(self, from_port_mock, stdout_mock):
     def tutorial_example():
@@ -140,8 +159,7 @@ class TestTutorialExamples(unittest.TestCase):
           if circ.status != CircStatus.BUILT:
             continue
 
-          print
-          print "Circuit %s (%s)" % (circ.id, circ.purpose)
+          print('\nCircuit %s (%s)' % (circ.id, circ.purpose))
 
           for i, entry in enumerate(circ.path):
             div = '+' if (i == len(circ.path) - 1) else '|'
@@ -150,7 +168,7 @@ class TestTutorialExamples(unittest.TestCase):
             desc = controller.get_network_status(fingerprint, None)
             address = desc.address if desc else 'unknown'
 
-            print " %s- %s (%s, %s)" % (div, fingerprint, nickname, address)
+            print(' %s- %s (%s, %s)' % (div, fingerprint, nickname, address))
 
     path_1 = ('B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C', 'ByTORAndTheSnowDog')
     path_2 = ('0DD9935C5E939CFA1E07B8DDA6D91C1A2A9D9338', 'afo02')
@@ -168,19 +186,19 @@ class TestTutorialExamples(unittest.TestCase):
     controller.get_circuits.return_value = [circuit_4, circuit_6, circuit_10]
 
     controller.get_network_status.side_effect = lambda fingerprint, *args: {
-      path_1[0]: _get_router_status("173.209.180.61"),
-      path_2[0]: _get_router_status("87.238.194.176"),
-      path_3[0]: _get_router_status("109.163.234.10"),
-      path_4[0]: _get_router_status("46.165.197.96"),
-      path_5[0]: _get_router_status("96.47.226.20"),
-      path_6[0]: _get_router_status("86.59.119.83"),
-      path_7[0]: _get_router_status("176.67.169.171")
+      path_1[0]: _get_router_status('173.209.180.61'),
+      path_2[0]: _get_router_status('87.238.194.176'),
+      path_3[0]: _get_router_status('109.163.234.10'),
+      path_4[0]: _get_router_status('46.165.197.96'),
+      path_5[0]: _get_router_status('96.47.226.20'),
+      path_6[0]: _get_router_status('86.59.119.83'),
+      path_7[0]: _get_router_status('176.67.169.171')
     }[fingerprint]
 
     tutorial_example()
-    self.assertEqual(LIST_CIRCUITS_OUTPUT, stdout_mock.getvalue())
+    self.assert_equal_unordered(LIST_CIRCUITS_OUTPUT, stdout_mock.getvalue())
 
-  @patch('sys.stdout', new_callable = StringIO.StringIO)
+  @patch('sys.stdout', new_callable = StringIO)
   @patch('stem.control.Controller.from_port', spec = Controller)
   def test_exit_used(self, from_port_mock, stdout_mock):
     def tutorial_example(mock_event):
@@ -190,8 +208,7 @@ class TestTutorialExamples(unittest.TestCase):
       from stem.control import EventType, Controller
 
       def main():
-        print "Tracking requests for tor exits. Press 'enter' to end."
-        print
+        print("Tracking requests for tor exits. Press 'enter' to end.\n")
 
         with Controller.from_port() as controller:
           controller.authenticate()
@@ -208,12 +225,11 @@ class TestTutorialExamples(unittest.TestCase):
           exit_fingerprint = circ.path[-1][0]
           exit_relay = controller.get_network_status(exit_fingerprint)
 
-          print "Exit relay for our connection to %s" % (event.target)
-          print "  address: %s:%i" % (exit_relay.address, exit_relay.or_port)
-          print "  fingerprint: %s" % exit_relay.fingerprint
-          print "  nickname: %s" % exit_relay.nickname
-          print "  locale: %s" % controller.get_info("ip-to-country/%s" % exit_relay.address, 'unknown')
-          print
+          print('Exit relay for our connection to %s' % (event.target))
+          print('  address: %s:%i' % (exit_relay.address, exit_relay.or_port))
+          print('  fingerprint: %s' % exit_relay.fingerprint)
+          print('  nickname: %s' % exit_relay.nickname)
+          print('  locale: %s\n' % controller.get_info('ip-to-country/%s' % exit_relay.address, 'unknown'))
 
       main()
 
@@ -231,10 +247,9 @@ class TestTutorialExamples(unittest.TestCase):
     controller.get_info.return_value = 'unknown'
 
     tutorial_example(event)
+    self.assert_equal_unordered(EXIT_USED_OUTPUT, stdout_mock.getvalue())
 
-    self.assertEqual(EXIT_USED_OUTPUT, stdout_mock.getvalue())
-
-  @patch('sys.stdout', new_callable = StringIO.StringIO)
+  @patch('sys.stdout', new_callable = StringIO)
   @patch('stem.descriptor.remote.DescriptorDownloader')
   def test_outdated_relays(self, downloader_mock, stdout_mock):
     def tutorial_example():
@@ -244,19 +259,17 @@ class TestTutorialExamples(unittest.TestCase):
       downloader = DescriptorDownloader()
       count, with_contact = 0, 0
 
-      print "Checking for outdated relays..."
-      print
+      print('Checking for outdated relays...\n')
 
       for desc in downloader.get_server_descriptors():
         if desc.tor_version < Version('0.2.3.0'):
           count += 1
 
           if desc.contact:
-            print '  %-15s %s' % (desc.tor_version, desc.contact.decode("utf-8", "replace"))
+            print('  %-15s %s' % (desc.tor_version, desc.contact.decode('utf-8', 'replace')))
             with_contact += 1
 
-      print
-      print "%i outdated relays found, %i had contact information" % (count, with_contact)
+      print('\n%i outdated relays found, %i had contact information' % (count, with_contact))
 
     downloader_mock().get_server_descriptors.return_value = [
       get_relay_server_descriptor({'platform': 'node-Tor 0.2.3.0 on Linux x86_64'}),
@@ -267,9 +280,9 @@ class TestTutorialExamples(unittest.TestCase):
 
     tutorial_example()
 
-    self.assertEqual(OUTDATED_RELAYS_OUTPUT, stdout_mock.getvalue())
+    self.assert_equal_unordered(OUTDATED_RELAYS_OUTPUT, stdout_mock.getvalue())
 
-  @patch('sys.stdout', new_callable = StringIO.StringIO)
+  @patch('sys.stdout', new_callable = StringIO)
   @patch('stem.descriptor.remote.Query')
   @patch('stem.descriptor.remote.get_authorities')
   def test_compare_flags(self, get_authorities_mock, query_mock, stdout_mock):
@@ -279,7 +292,7 @@ class TestTutorialExamples(unittest.TestCase):
       # Query all authority votes asynchronously.
 
       downloader = remote.DescriptorDownloader(document_handler = DocumentHandler.DOCUMENT)
-      queries = {}
+      queries = OrderedDict()
 
       for name, authority in remote.get_authorities().items():
         if authority.v3ident is None:
@@ -306,15 +319,15 @@ class TestTutorialExamples(unittest.TestCase):
         maatuska_vote = votes['maatuska'].routers.get(fingerprint)
 
         if not moria1_vote and not maatuska_vote:
-          print "both moria1 and maatuska haven't voted about %s" % fingerprint
+          print("both moria1 and maatuska haven't voted about %s" % fingerprint)
         elif not moria1_vote:
-          print "moria1 hasn't voted about %s" % fingerprint
+          print("moria1 hasn't voted about %s" % fingerprint)
         elif not maatuska_vote:
-          print "maatuska hasn't voted about %s" % fingerprint
+          print("maatuska hasn't voted about %s" % fingerprint)
         elif 'Running' in moria1_vote.flags and 'Running' not in maatuska_vote.flags:
-          print "moria1 has the Running flag but maatuska doesn't: %s" % fingerprint
+          print("moria1 has the Running flag but maatuska doesn't: %s" % fingerprint)
         elif 'Running' in maatuska_vote.flags and 'Running' not in moria1_vote.flags:
-          print "maatuska has the Running flag but moria1 doesn't: %s" % fingerprint
+          print("maatuska has the Running flag but moria1 doesn't: %s" % fingerprint)
 
     get_authorities_mock().items.return_value = [('moria1', DIRECTORY_AUTHORITIES['moria1']), ('maatuska', DIRECTORY_AUTHORITIES['maatuska'])]
 
@@ -350,9 +363,10 @@ class TestTutorialExamples(unittest.TestCase):
     ]
 
     tutorial_example()
-    self.assertEqual(COMPARE_FLAGS_OUTPUT, stdout_mock.getvalue())
 
-  @patch('sys.stdout', new_callable = StringIO.StringIO)
+    self.assert_equal_unordered(COMPARE_FLAGS_OUTPUT, stdout_mock.getvalue())
+
+  @patch('sys.stdout', new_callable = StringIO)
   @patch('stem.descriptor.remote.get_authorities')
   @patch('stem.descriptor.remote.DescriptorDownloader.query')
   def test_votes_by_bandwidth_authorities(self, query_mock, get_authorities_mock, stdout_mock):
@@ -373,7 +387,7 @@ class TestTutorialExamples(unittest.TestCase):
 
       for authority_name, query in queries.items():
         try:
-          print "Getting %s's vote from %s:" % (authority_name, query.download_url)
+          print("Getting %s's vote from %s:" % (authority_name, query.download_url))
 
           measured, unmeasured = 0, 0
 
@@ -383,9 +397,9 @@ class TestTutorialExamples(unittest.TestCase):
             else:
               unmeasured += 1
 
-          print '  %i measured entries and %i unmeasured' % (measured, unmeasured)
+          print('  %i measured entries and %i unmeasured' % (measured, unmeasured))
         except Exception as exc:
-          print "  failed to get the vote (%s)" % exc
+          print('  failed to get the vote (%s)' % exc)
 
     directory_values = [
       DIRECTORY_AUTHORITIES['gabelmoo'],
@@ -394,14 +408,14 @@ class TestTutorialExamples(unittest.TestCase):
       DIRECTORY_AUTHORITIES['maatuska'],
     ]
 
-    directory_values[0].address = '212.112.245.170'
+    directory_values[0].address = '131.188.40.189'
     get_authorities_mock().values.return_value = directory_values
 
     entry_with_measurement = get_router_status_entry_v3({'w': 'Bandwidth=1 Measured=1'})
     entry_without_measurement = get_router_status_entry_v3()
 
     query1 = Mock()
-    query1.download_url = 'http://212.112.245.170:80/tor/status-vote/current/authority'
+    query1.download_url = 'http://131.188.40.189:80/tor/status-vote/current/authority'
     query1.run.return_value = [entry_with_measurement] * 5935 + [entry_without_measurement] * 1332
 
     query2 = Mock()
@@ -419,9 +433,9 @@ class TestTutorialExamples(unittest.TestCase):
     query_mock.side_effect = [query1, query2, query3, query4]
 
     tutorial_example()
-    self.assertEqual(VOTES_BY_BANDWIDTH_AUTHORITIES_OUTPUT, stdout_mock.getvalue())
+    self.assert_equal_unordered(VOTES_BY_BANDWIDTH_AUTHORITIES_OUTPUT, stdout_mock.getvalue())
 
-  @patch('sys.stdout', new_callable = StringIO.StringIO)
+  @patch('sys.stdout', new_callable = StringIO)
   @patch('stem.descriptor.parse_file')
   @patch('%s.open' % __name__, create = True)
   @patch('stem.descriptor.remote.Query')
@@ -446,7 +460,7 @@ class TestTutorialExamples(unittest.TestCase):
       ))
 
       for fingerprint, relay in consensus.routers.items():
-        print "%s: %s" % (fingerprint, relay.nickname)
+        print('%s: %s' % (fingerprint, relay.nickname))
 
     network_status = get_network_status_document_v3(routers = (get_router_status_entry_v3(),))
     query_mock().run.return_value = [network_status]
