@@ -32,6 +32,18 @@ unix  3      [ ]         STREAM     CONNECTED     34164276 15843/tor
 unix  3      [ ]         STREAM     CONNECTED     7951     -
 """
 
+NETSTAT_WINDOWS_OUTPUT = """\
+Active Connections
+
+  Proto  Local Address          Foreign Address        State           PID
+  TCP    0.0.0.0:135            0.0.0.0:0              LISTENING       852
+  TCP    0.0.0.0:445            0.0.0.0:0              LISTENING       4
+  TCP    0.0.0.0:902            0.0.0.0:0              LISTENING       2076
+  TCP    0.0.0.0:912            0.0.0.0:0              LISTENING       2076
+  TCP    192.168.0.1:44284      38.229.79.2:443        ESTABLISHED     15843
+  TCP    0.0.0.0:37782          0.0.0.0:0              LISTENING       4128
+"""
+
 SS_OUTPUT = """\
 Netid  State      Recv-Q Send-Q     Local Address:Port       Peer Address:Port
 tcp    CLOSE-WAIT 1      0           192.168.0.1:43780      53.203.145.45:443    users:(("firefox",20586,118))
@@ -105,17 +117,19 @@ BSD_PROCSTAT_OUTPUT = """\
 
 
 class TestConnection(unittest.TestCase):
+  @patch('os.access')
   @patch('stem.util.system.is_available')
   @patch('stem.util.proc.is_available')
-  def test_system_resolvers(self, proc_mock, is_available_mock):
+  def test_system_resolvers(self, proc_mock, is_available_mock, os_mock):
     """
     Checks the system_resolvers function.
     """
 
     is_available_mock.return_value = True
     proc_mock.return_value = False
+    os_mock.return_value = True
 
-    self.assertEqual([], stem.util.connection.system_resolvers('Windows'))
+    self.assertEqual([Resolver.NETSTAT_WINDOWS], stem.util.connection.system_resolvers('Windows'))
     self.assertEqual([Resolver.LSOF], stem.util.connection.system_resolvers('Darwin'))
     self.assertEqual([Resolver.LSOF], stem.util.connection.system_resolvers('OpenBSD'))
     self.assertEqual([Resolver.BSD_SOCKSTAT, Resolver.BSD_PROCSTAT, Resolver.LSOF], stem.util.connection.system_resolvers('FreeBSD'))
@@ -124,7 +138,7 @@ class TestConnection(unittest.TestCase):
     proc_mock.return_value = True
     self.assertEqual([Resolver.PROC, Resolver.NETSTAT, Resolver.SOCKSTAT, Resolver.LSOF, Resolver.SS], stem.util.connection.system_resolvers('Linux'))
 
-    # check that calling without an argument is equivilant to calling for this
+    # check that calling without an argument is equivalent to calling for this
     # platform
 
     self.assertEqual(stem.util.connection.system_resolvers(platform.system()), stem.util.connection.system_resolvers())
@@ -182,6 +196,21 @@ class TestConnection(unittest.TestCase):
     call_mock.side_effect = OSError('Unable to call netstat')
     self.assertRaises(IOError, stem.util.connection.get_connections, Resolver.NETSTAT, process_pid = 1111)
 
+  @patch('stem.util.system.call')
+  def test_get_connections_by_windows_netstat(self, call_mock):
+    """
+    Checks the get_connections function with the Windows netstat resolver.
+    """
+
+    call_mock.return_value = NETSTAT_WINDOWS_OUTPUT.split('\n')
+    expected = [Connection('192.168.0.1', 44284, '38.229.79.2', 443, 'tcp')]
+    self.assertEqual(expected, stem.util.connection.get_connections(Resolver.NETSTAT_WINDOWS, process_pid = 15843, process_name = 'tor'))
+
+    self.assertRaises(IOError, stem.util.connection.get_connections, Resolver.NETSTAT_WINDOWS, process_pid = 1111, process_name = 'tor')
+    call_mock.side_effect = OSError('Unable to call netstat')
+
+    self.assertRaises(IOError, stem.util.connection.get_connections, Resolver.NETSTAT_WINDOWS, process_pid = 1111)
+
   @patch('stem.util.system.call')
   def test_get_connections_by_ss(self, call_mock):
     """
@@ -404,7 +433,7 @@ class TestConnection(unittest.TestCase):
     }
 
     for test_arg, expected in test_values.items():
-      self.assertEquals(expected, stem.util.connection.expand_ipv6_address(test_arg))
+      self.assertEqual(expected, stem.util.connection.expand_ipv6_address(test_arg))
 
     self.assertRaises(ValueError, stem.util.connection.expand_ipv6_address, '127.0.0.1')
 
@@ -413,10 +442,10 @@ class TestConnection(unittest.TestCase):
     Checks the get_mask_ipv4 function.
     """
 
-    self.assertEquals('255.255.255.255', stem.util.connection.get_mask_ipv4(32))
-    self.assertEquals('255.255.255.248', stem.util.connection.get_mask_ipv4(29))
-    self.assertEquals('255.255.254.0', stem.util.connection.get_mask_ipv4(23))
-    self.assertEquals('0.0.0.0', stem.util.connection.get_mask_ipv4(0))
+    self.assertEqual('255.255.255.255', stem.util.connection.get_mask_ipv4(32))
+    self.assertEqual('255.255.255.248', stem.util.connection.get_mask_ipv4(29))
+    self.assertEqual('255.255.254.0', stem.util.connection.get_mask_ipv4(23))
+    self.assertEqual('0.0.0.0', stem.util.connection.get_mask_ipv4(0))
 
     self.assertRaises(ValueError, stem.util.connection.get_mask_ipv4, -1)
     self.assertRaises(ValueError, stem.util.connection.get_mask_ipv4, 33)
@@ -426,9 +455,9 @@ class TestConnection(unittest.TestCase):
     Checks the get_mask_ipv6 function.
     """
 
-    self.assertEquals('FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF', stem.util.connection.get_mask_ipv6(128))
-    self.assertEquals('FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFE:0000', stem.util.connection.get_mask_ipv6(111))
-    self.assertEquals('0000:0000:0000:0000:0000:0000:0000:0000', stem.util.connection.get_mask_ipv6(0))
+    self.assertEqual('FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF', stem.util.connection.get_mask_ipv6(128))
+    self.assertEqual('FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFE:0000', stem.util.connection.get_mask_ipv6(111))
+    self.assertEqual('0000:0000:0000:0000:0000:0000:0000:0000', stem.util.connection.get_mask_ipv6(0))
 
     self.assertRaises(ValueError, stem.util.connection.get_mask_ipv6, -1)
     self.assertRaises(ValueError, stem.util.connection.get_mask_ipv6, 129)
@@ -438,10 +467,10 @@ class TestConnection(unittest.TestCase):
     Checks the _get_masked_bits function.
     """
 
-    self.assertEquals(32, stem.util.connection._get_masked_bits('255.255.255.255'))
-    self.assertEquals(29, stem.util.connection._get_masked_bits('255.255.255.248'))
-    self.assertEquals(23, stem.util.connection._get_masked_bits('255.255.254.0'))
-    self.assertEquals(0, stem.util.connection._get_masked_bits('0.0.0.0'))
+    self.assertEqual(32, stem.util.connection._get_masked_bits('255.255.255.255'))
+    self.assertEqual(29, stem.util.connection._get_masked_bits('255.255.255.248'))
+    self.assertEqual(23, stem.util.connection._get_masked_bits('255.255.254.0'))
+    self.assertEqual(0, stem.util.connection._get_masked_bits('0.0.0.0'))
 
     self.assertRaises(ValueError, stem.util.connection._get_masked_bits, 'blarg')
     self.assertRaises(ValueError, stem.util.connection._get_masked_bits, '255.255.0.255')
@@ -463,7 +492,7 @@ class TestConnection(unittest.TestCase):
     }
 
     for test_arg, expected in test_values.items():
-      self.assertEquals(expected, stem.util.connection._get_address_binary(test_arg))
+      self.assertEqual(expected, stem.util.connection._get_address_binary(test_arg))
 
     self.assertRaises(ValueError, stem.util.connection._get_address_binary, '')
     self.assertRaises(ValueError, stem.util.connection._get_address_binary, 'blarg')
