@@ -1,4 +1,4 @@
-# Copyright 2012-2014, Damian Johnson and The Tor Project
+# Copyright 2012-2015, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -11,8 +11,6 @@ Helper functions for our test framework.
 
   get_prereq - provides the tor version required to run the given target
   get_torrc_entries - provides the torrc entries for a given target
-  get_help_message - provides usage information for running our tests
-  get_python3_destination - location where a python3 copy of stem is exported to
 
 Sets of :class:`~test.util.Task` instances can be ran with
 :func:`~test.util.run_tasks`. Functions that are intended for easy use with
@@ -28,17 +26,10 @@ Tasks are...
   |- check_pep8_version - checks our version of pep8
   |- clean_orphaned_pyc - removes any *.pyc without a corresponding *.py
   +- check_for_unused_tests - checks to see if any tests are missing from our settings
-
-  Testing Python 3
-  |- python3_prereq - checks that we have python3 and 2to3
-  |- python3_clean - deletes our prior python3 export
-  |- python3_copy_stem - copies our codebase and converts with 2to3
-  +- python3_run_tests - runs python 3 tests
 """
 
 import re
 import os
-import shutil
 import sys
 
 import stem
@@ -53,8 +44,6 @@ import test.output
 from test.output import STATUS, ERROR, NO_NL, println
 
 CONFIG = stem.util.conf.config_dict('test', {
-  'msg.help': '',
-  'target.description': {},
   'target.prereq': {},
   'target.torrc': {},
   'integ.test_directory': './test/data',
@@ -62,6 +51,15 @@ CONFIG = stem.util.conf.config_dict('test', {
   'test.integ_tests': '',
 })
 
+# Integration targets fall into two categories:
+#
+# * Run Targets (like RUN_COOKIE and RUN_PTRACE) which customize our torrc.
+#   We do an integration test run for each run target we get.
+#
+# * Attribute Target (like CHROOT and ONLINE) which indicates
+#   non-configuration changes to your test runs. These are applied to all
+#   integration runs that we perform.
+
 Target = stem.util.enum.UppercaseEnum(
   'ONLINE',
   'RELATIVE',
@@ -82,74 +80,55 @@ Target = stem.util.enum.UppercaseEnum(
 
 STEM_BASE = os.path.sep.join(__file__.split(os.path.sep)[:-2])
 
+# Store new capabilities (events, descriptor entries, etc.)
 
-def get_unit_tests(module_substring = None):
+NEW_CAPABILITIES = []
+
+
+def get_unit_tests(module_prefix = None):
   """
   Provides the classes for our unit tests.
 
-  :param str module_substring: only provide the test if the module includes this substring
+  :param str module_prefix: only provide the test if the module starts with
+    this substring
 
   :returns: an **iterator** for our unit tests
   """
 
-  return _get_tests(CONFIG['test.unit_tests'].splitlines(), module_substring)
+  if module_prefix and not module_prefix.startswith('test.unit.'):
+    module_prefix = 'test.unit.' + module_prefix
+
+  return _get_tests(CONFIG['test.unit_tests'].splitlines(), module_prefix)
 
 
-def get_integ_tests(module_substring = None):
+def get_integ_tests(module_prefix = None):
   """
   Provides the classes for our integration tests.
 
-  :param str module_substring: only provide the test if the module includes this substring
+  :param str module_prefix: only provide the test if the module starts with
+    this substring
 
   :returns: an **iterator** for our integration tests
   """
 
-  return _get_tests(CONFIG['test.integ_tests'].splitlines(), module_substring)
+  if module_prefix and not module_prefix.startswith('test.integ.'):
+    module_prefix = 'test.integ.' + module_prefix
+
+  return _get_tests(CONFIG['test.integ_tests'].splitlines(), module_prefix)
 
 
-def _get_tests(modules, module_substring):
+def _get_tests(modules, module_prefix):
   for import_name in modules:
     if import_name:
-      if module_substring and module_substring not in import_name:
-        continue
+      module, module_name = import_name.rsplit('.', 1)  # example: util.conf.TestConf
 
-      # Dynamically imports test modules. The __import__() call has a couple
-      # quirks that make this a little clunky...
-      #
-      #   * it only accepts modules, not the actual class we want to import
-      #
-      #   * it returns the top level module, so we need to transverse into it
-      #     for the test class
+      if not module_prefix or module.startswith(module_prefix):
+        yield import_name
+      elif module_prefix.startswith(module):
+        # single test for this module
 
-      module_name = '.'.join(import_name.split('.')[:-1])
-      module = __import__(module_name)
-
-      for subcomponent in import_name.split('.')[1:]:
-        module = getattr(module, subcomponent)
-
-      yield module
-
-
-def get_help_message():
-  """
-  Provides usage information, as provided by the '--help' argument. This
-  includes a listing of the valid integration targets.
-
-  :returns: **str** with our usage information
-  """
-
-  help_msg = CONFIG['msg.help']
-
-  # gets the longest target length so we can show the entries in columns
-  target_name_length = max(map(len, Target))
-  description_format = '\n    %%-%is - %%s' % target_name_length
-
-  for target in Target:
-    help_msg += description_format % (target, CONFIG['target.description'].get(target, ''))
-
-  help_msg += '\n'
-
-  return help_msg
+        test_module = module_prefix.rsplit('.', 1)[1]
+        yield '%s.%s' % (import_name, test_module)
 
 
 def get_prereq(target):
@@ -200,15 +179,15 @@ def get_torrc_entries(target):
   return torrc_opts
 
 
-def get_python3_destination():
+def get_new_capabilities():
   """
-  Provides the location where a python 3 copy of stem is exported to for
-  testing.
+  Provides a list of capabilities tor supports but stem doesn't, as discovered
+  while running our tests.
 
-  :returns: **str** with the relative path to our python 3 location
+  :returns: **list** of (type, message) tuples for the capabilities
   """
 
-  return os.path.join(CONFIG['integ.test_directory'], 'python3')
+  return NEW_CAPABILITIES
 
 
 def check_stem_version():
@@ -301,53 +280,15 @@ def check_for_unused_tests(paths):
     raise ValueError('Test modules are missing from our test/settings.cfg:\n%s' % '\n'.join(unused_tests))
 
 
-def python3_prereq():
-  for required_cmd in ('2to3', 'python3'):
-    if not stem.util.system.is_available(required_cmd):
-      raise ValueError("Unable to test python 3 because %s isn't in your path" % required_cmd)
+def register_new_capability(capability_type, msg):
+  """
+  Register new capability found during the tests.
 
+  :param str capability_type: type of capability this is
+  :param str msg: description of what we found
+  """
 
-def python3_clean(skip = False):
-  location = get_python3_destination()
-
-  if not os.path.exists(location):
-    return 'skipped'
-  elif skip:
-    return ["Reusing '%s'. Run again with '--clean' if you want a fresh copy." % location]
-  else:
-    shutil.rmtree(location, ignore_errors = True)
-    return 'done'
-
-
-def python3_copy_stem():
-  destination = get_python3_destination()
-
-  if os.path.exists(destination):
-    return 'skipped'
-
-  # skips the python3 destination (to avoid an infinite loop)
-  def _ignore(src, names):
-    if src == os.path.normpath(destination):
-      return names
-    else:
-      return []
-
-  os.makedirs(destination)
-  shutil.copytree('stem', os.path.join(destination, 'stem'))
-  shutil.copytree('test', os.path.join(destination, 'test'), ignore = _ignore)
-  shutil.copy('run_tests.py', os.path.join(destination, 'run_tests.py'))
-  stem.util.system.call('2to3 --write --nobackups --no-diffs %s' % get_python3_destination())
-
-  return 'done'
-
-
-def python3_run_tests():
-  println()
-  println()
-
-  python3_runner = os.path.join(get_python3_destination(), 'run_tests.py')
-  exit_status = os.system('python3 %s %s' % (python3_runner, ' '.join(sys.argv[1:])))
-  sys.exit(exit_status)
+  NEW_CAPABILITIES.append((capability_type, msg))
 
 
 def _is_test_data(path):
